#!/usr/bin/perl
use File::Copy::Recursive qw(fcopy rcopy dircopy fmove rmove dirmove);
use File::Copy;
use Statistics::LineFit;
use Statistics::Test::WilcoxonRankSum;
use Statistics::Zed 0.072;
use Cwd;
use strict;
local $| = 1;

# Program to explore different properties of trees. The following options are available for the command line:
#-fun: a (a priori screening for bootstrap support), b (a posteriori screening for bootstrap suport), d (automatic pruning of paralogs), e (calculation of LB scores), f (calculation of average BS support) and i (summary of bipartition tables of BS support across different datasets)
#-gts: when a masking approach for strongly supported clades shall be used, the filtering has to be turned on using this option with Y or Yes. Moreover, if later a filtering based on strong conflict (-possc) shall be used, this option has also be turned on as the masking and strong conflict are logically connected. The files are provided via txt-files starting with GroupedTaxa (e.g., GroupedTaxaPhylum.txt). (Default=No) (-fun a and b)
#-lowbs: lower BS value for the screening in a and b (values between 0 and 100 are possible as long as they are integers, default = 95) and the lowest BS value still to be summarized in f (values between 1 and 100 are possible here as long as they are integers, default = 1) (-fun a and b)
#-upbs: upper BS value for the screening in a and b (values between 0 and 100 are possible as long as they are integers and equal to or larger than -lowbs, default = 100)  (-fun a and b)
# potential paralogs can be sorted based on different filter criteria and different orders, the following options turn on the criteria and and provide their order number
#-possc: sorting position based on the strong conflict filter, 0 = turned off, 1-3 = turned on and definition of position in the order of filters (1 = 1st, 2 = 2nd, 3 = 3rd), default = 0  (-fun a and b)
#-possb: sorting position based on the short terminal branch filter, 0 = turned off, 1-3 = turned on and definition of position in the order of filters (1 = 1st, 2 = 2nd, 3 = 3rd), default = 0  (-fun a and b)
#-poslb: sorting position based on the long internal branch filter, 0 = turned off, 1-3 = turned on and definition of position in the order of filters (1 = 1st, 2 = 2nd, 3 = 3rd), default = 0  (-fun a and b)
# These three filters cannot have the same value except for 0.
#-maxtaxa: maximum number of taxa, which are allowed in a clade for the short terminal branch filter (default = 2, but option is required to be turned on for -possb) (-fun a and b)
#-blt: threshold of branch length less than it for the short terminal branch filter (default = 0, but option is required to be turned on for -possb) (-fun a and b)
#-lowbl: lowest factor by which the branch length has to be at least larger than the average internal branch length for the long internal branch filter (default = 0, but option is required to be turned on for -poslb) (-fun a and b)
#-upbl: highest factor by which the branch length has to be at least larger than the average internal branch length for the long internal branch filter (default = 0, but option is required to be turned on for -poslb), must be equal to or larger than the value of -lowbl (-fun a and b)
#-trf: name of a treefile containing a single tree, which provides the clades for the a posteriori screening approach (-fun b), in Newick format (e.g. RAxML_besttree... or RAxML_bipartition....)  
#-ppf: name of a file containing the paralog sequences, which shall be pruned from the alignments (-fun c and d)
#-tf: name of taxa across trees (this information is needed for the LB score calculation and summary of BS support across datasets (-fun e and i)
#-ipt: name of a file containing the names of the tree files, which shall be investigated for (-fun a, b, c, d, e, f, g and i)
#-ipa: name of a file containing the names of the alignment files, which shall be investigated for saturation (-fun c, d and g)
#-aln: name of alignment file for combination of different partitions (-fun h)
#-partf: name of definition file for partitions (RAxML style) (-fun h and j)
#-ubnp: upperbound of number of partitions to be combined (-fun h)
#-lbnp: lowerbound of number of partitions to be combined (-fun h)
#-nper: number of permutated datasets to generate (-fun h)
#-per: when a permutated datasets shall be generated or the significance of PABA results based on permutation shall be, the option has to be turned on using -per Y or -per Yes. (-fun h and j) 
#-ori: provides the name of the dataset with the original results for the significance test based on permutattions and has to be also part of the list of result files provided by -tabf (-fun j)
#-wsrt: when a WSRT test shall be performed, the option has to be turned on using -wsrt Y or -wsrt Yes. (-fun j)
#-bpf: provides a list of files containing the list of bipartitions, which shall be independently tested by the WSRT test (-fun j)
#-mint: minimum threshold of support values for the PABA calculations (default = 0), input format either a number or N/None if not applicable (e.g. Bremer support based values) (-fun i and j)
#-maxt: maximum threshold of support values for the PABA calculations (default = 100), input format either a number or N/None if not applicable (e.g. Bremer support based values) (-fun j)
#-tabf: name of a file listing the names of the results files of the tabulated support values for different combinations of partitions as, for example generated by -fun i, which shall be used for PABA calculations (-fun j)
#-path: if files are not in the same folder as TreSpEx, provide the path from root by -path path-from-the-root (for all -fun)
#-db1: name of database, which shall be used for as the first database against which the blast search shall be conducted (for -fun c)
#-db2: name of database, which shall be used for as the second database against which the blast search shall be conducted (for -fun c)
#-ediff: the difference in magnitude between the best e-value and the higher e-value, which is still considered to be part of the confidence set, default 5 (for -fun c), exceptions: if e-value = 0.0 than only other hits which also have an e-value of 0.0 will be considered for the confidence, similarly if e-value of best hit > 0 or smaller than e-diff magnitude (e.g. e-5) than all hits will part of the confidence set
#-evalue: the e-value threshold for the BLAST search, can have the form of 0.00001 or 1e-5, default 10 (for -fun c)
#-ltp: lower threshold below which a sequence will be regarded as being paralogous, the value is a proportion and has to be between 0 and 1 and lower than the value provided by -upt, default is 0.0 (-fun c)
#-utp: upper threshold above which a sequence will be regarded as being not paralogous, the value is a proportion and has to be between 0 and 1 and higher than the value provided by -lpt, default is 1.0 (-fun c)
#
# for -fun a-e: RaxML_bipartions.-files are needed
# for -fun f: RAxML_bootstrap.-files are needed
# IMPORTANT: The taxa names have to have a least one alphabetic character, so that they can be differentiated from bootstrap values and branch length in the tree 
# additionally, for -fun c: RAxML_info and the corresponding .phy-files are needed
# for -fun j: one file listing files with bootstrap matrix (e.g. Compiled_Bipartition_Tables_1a.txt\nCompiled_Bipartition_Tables_2a.txt\n)
# for -fun j: one file listing partitions (RAxML style, similar to the Combintation option)
# for -fun j: one file listing files with set of bipartitions for WSRT tests (e.g., AllBiparts.txt\nSetOfBiparts_1.txt\nSetOfBiparts_2.txt)
# for -fun j: files with bootstrap matrix (as output files generated with summary bootstrap option): first row names including combinations of partitions, first column identifer (which have also be used in the files determining sets of bipartitions for WSRT tests), additional columns are possible (but have to include one non-digit character), columns with bootstrap values with the corresponding combination heading
# for -fun j: files with set of bipartitions for WSRT tests: list of identifers for bipartitions (e.g. Bipart1\nBipart2\n and so on), have to be same identifer as in the first column of the bootstrap matrix
# download helper scripts (see next lines) with, for example, sudo cpanm Statistics::LineFit

#Version 1.1
#-included run time measurement
#-fixed minor bug in fun h for checking of nper option and in fun j for checking the ori option
#-fixed major bug concerning compatibility with some Linux versions

#Version 1.2
#-fixed major bug concerning function b (a posteriori) screening in that the most inclusive pair of parentheses (the most outer ones on each side) is not used
#-changed the setting from a strict congruence in the a posteriori screening for bootstrap support to a relaxed setting. In the strict setting all taxa of a clade of interest, which are present in the tree, had also to be present in a detected clade of potential paralogy. In the relaxed setting this is not the case anymore. Now the detected clade of potential clade has only to comprise taxa from the pre-defined clade of interest, but not all taxa of this clade. This means, that other taxa of this clade can also be present outside the detected clade in the tree. Therefore, it is advisable to combine this search strategy with the option -gts predefining groups with a priori evidence of monophyly for such subclades in the clade of interest to minimize the number of instances to be further analyzed.
#-fixed bug concerning trees with posterior probabilities in the LB score and saturation calculations
#-fixed a bug concerning asking for the bpf-file despite a WRST test is not used

#Version 1.3
#-fixed a bug that allow LB score index to be calculated for trees with a taxon name occuring twice in the tree, these results are not correct

my $begin = 95;
my $end = 100;
my $lbpos = 0;
my $lbbegin = 0;
my $lbend = 0;
my $scpos = 0;
my $sbpos = 0;
my $blthreshold = 0;
my $mintaxa = 2;
my $paralogfile=();
my $taxanames=();
my $treefile=();
my $lowthres=1;
my $gtscreen=0;
my $treefiles = ();
my $alignfiles = ();
my $alignment = ();
my $partitionfile = ();
my $maxx = 0;
my $minx = 0;
my $nper = 100; 
my $per = 0;
my $tablefiles = ();
my $partitionfile = ();
my $min = 0; 
my $max = 100; 
my $wsrtc = 0;
my $bipartsets = ();
my $permutc = 0; 
my $original = ();
my $path = ();
my $trespexpath = ();
my $returnpath = ();
my $database1 = ();
my $database2 = ();
my $ediff = 5;
my $lowparathres = 0.0;
my $upparathres = 1.0;
my $evalue = 10;

foreach my $a (0..$#ARGV) {
 if ($ARGV[$a] =~ m/^-h$/) {
  &help ($a)
 }
}
my %input = @ARGV;
my @key = keys %input;
my $funcheck = 0;
foreach my $a (0..$#key) {
 unless ($key[$a] =~ m/^-/) {die "Each option has to start with a dash (-).If that is the case, then check if all provided options occur in option/value pairs.\n"};
 if ($key[$a] =~ m/^-fun$/) {$funcheck=1};
}
my $pathfound = 0;
foreach my $a (0..$#key) {
 if ($key[$a] =~ m/^-path$/) {
  $pathfound = 1; 
 }
}
 if ($pathfound == 1) {
  $trespexpath = getcwd();
  chdir "$input{-path}" or die "The folder $input{-path} not found.\n";
  $returnpath = $input{-path};
 } else {
  $trespexpath = getcwd();
  $returnpath = $trespexpath;
 }
unless ($funcheck==1) {die "A function has to be provided via the -fun option. If that is the case, then check if all provided options occur in option/value pairs.\n"}
my $fun = $input{-fun};
unless ($fun =~ m/^a$|^b$|^c$|^d$|^e$|^f$|^g$|^h$|^i$|^j$/) {die "For the option -fun one of the following values has to be provided: a, b, c, d, e, f, g, h, i or j.\n"};
if ($fun =~ m/^a$|^b$/) {
 if ($input{-lowbs} =~ m/^\d+$/ and $input{-lowbs} <= 100 and $input{-lowbs} >= 0) {$begin = $input{-lowbs}} else {die "The value of option -lowbs has to be between 0 and 100 and an interger.\n"};
 if ($input{-upbs} =~ m/^\d+$/ and $input{-lowbs} <= 100 and $input{-lowbs} >= 0 and $input{-lowbs} >= $begin) {$end = $input{-upbs}} else {die "The value of option -upbs has to be between 0 and 100, an integer and larger or equal to the value of option -lowbs.\n"};
 if ($input{-gts} =~ m/Y|Yes|N|No/) {if ($input{-gts} =~ m/Y|Yes/) {$gtscreen=1}} else {die "The option -gts requires the input Y, Yes, N or No.\n"}
 my $lbposcheck = 0;
 my $iptcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
 }
 unless ($iptcheck==1) {die "An input files containing the names of the treefiles has to be provided by -ipt filename.\n"}
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-poslb$/) {
   $lbposcheck=1;
   my $poslbcheck=0;
   if ($input{-poslb} =~ m/^0$/) {$poslbcheck=1};
   if ($input{-poslb} =~ m/^1$|^2$|^3$/) {if ($input{-poslb} != $input{-possc} and $input{-poslb} != $input{-possb}) {$poslbcheck=1}};
   if ($poslbcheck==1) {$lbpos = $input{-poslb}} else {die "The option -poslb has to have value of 0, 1, 2 or 3 or should not be provided at all. It also should not have the same value as -possc or -possb except for 0.\n"};
  } 
 }
 unless ($lbposcheck==1) {print "\n\n!!!WARNING!!!\nThe option -poslb has not been provided. No filtering based on internal long branches will done!!!\n!!!WARNING!!!\n\n"}
 my $scposcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-possc$/) {
   $scposcheck=1;
   my $possccheck=0;
   if ($input{-possc} =~ m/^0$/) {$possccheck=1};
   if ($input{-possc} =~ m/^1$|^2$|^3$/) {if ($input{-possc} != $input{-poslb} and $input{-possc} != $input{-possb}) {$possccheck=1}};
   if ($possccheck==1) {$scpos = $input{-possc}} else {die "The option -possc has to have value of 0, 1, 2 or 3 or should not be provided at all. It also should not have the same value as -poslb or -possb except for 0.\n"};
   if ($scpos == 1 or $scpos == 2 or $scpos == 3) {unless ($input{-gts} =~ m/Y|Yes/) {die "The option -possc requires that the option -gts is turned on (-gts Y or -gts Yes).\n"}}
  } 
 }
 unless ($scposcheck==1) {print "\n\n!!!WARNING!!!\nThe option -possc has not been provided. No filtering based on strong conflict will done!!!\n!!!WARNING!!!\n\n"}
 my $sbposcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-possb$/) {
   $sbposcheck=1;
   my $possbcheck=0;
   if ($input{-possb} =~ m/^0$/) {$possbcheck=1};
   if ($input{-possb} =~ m/^1$|^2$|^3$/) {if ($input{-possb} != $input{-poslb} and $input{-possb} != $input{-possc}) {$possbcheck=1}};
   if ($possbcheck==1) {$sbpos = $input{-possb}} else {die "The option -possb has to have value of 0, 1, 2 or 3 or should not be provided at all. It also should not have the same value as -poslb or -possc except for 0.\n"};
  } 
 }
 unless ($sbposcheck==1) {print "\n\n!!!WARNING!!!\nThe option -possb has not been provided. No filtering based on short terminal branches will done!!!\n!!!WARNING!!!\n\n"}
 if ($sbpos>0) {
  my $sbcheck = 0;
  foreach my $a (0..$#key) {
   if ($key[$a] =~ m/^-blt$|^-maxtaxa$/) {$sbcheck++};
  }
  unless ($sbcheck==2) {die "When using the filtering option for short terminal branches (-possb) the two options -blt and -maxtaxa have to be provided.\n"}
  if ($input{-blt} >= 0 and $input{-blt} =~m/\d+\.\d+/) {$blthreshold = $input{-blt}} else {die "The option -blt has to have a value larger than or equal to 0.\n"};
  if ($input{-maxtaxa} >= 2 and $input{-maxtaxa} =~m/\d+/) {$mintaxa = $input{-maxtaxa}} else {die "The option -maxtaxa has to have a value larger than or equal to 2 and be an integer.\n"};
 }
 if ($lbpos>0) {
  my $lbcheck = 0;
  foreach my $a (0..$#key) {
   if ($key[$a] =~ m/^-lowbl$|^-upbl$/) {$lbcheck++};
  }
  unless ($lbcheck==2) {die "When using the filtering option for long internal branches (-poslb) the two options -lowbl and -upbl have to be provided.\n"}
  if ($input{-lowbl} >= 0 and $input{-lowbl} =~ m/\d+|\d+\.\d+/) {$lbbegin = $input{-lowbl}} else {die "The option -lowbl has to have a value larger than or equal to 0.\n"};
  if ($input{-upbl} >= 0 and $input{-upbl} =~m/\d+/ and $input{-upbl} >= $lbbegin) {$lbend = $input{-upbl}} else {die "The option -upbl has to have a value larger than or equal to 0 and be larger than or equal to the value of option -lowbl.\n"};
 }
}

if ($fun =~ m/^b$/) {
 my $trfcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-trf$/) {
   $trfcheck=1;
   if ($input{-trf} =~ m/^\w/) {$treefile = $input{-trf}} else {die "The option -trf requires a name which starts with a letter or number.\n"};
  }
 }
 unless ($trfcheck==1) {die "The a posteriori screening requires an input tree containing the nodes, which shall be checked.\n"}
}

if ($fun =~ m/^c$/) {
 my $iptcheck = 0;
 my $ipacheck = 0;
 my $ppfcheck = 0;
 my $db1check = 0;
 my $db2check = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-ipa$/) {
   $ipacheck=1;
   if ($input{-ipa} =~ m/^\w/) {$alignfiles = $input{-ipa}} else {die "The option -ipa requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-ppf$/) {
   $ppfcheck=1;
   if ($input{-ppf} =~ m/^\w/) {$paralogfile = $input{-ppf}} else {die "The option -ppf requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-db1$/) {
   $db1check=1;
   if ($input{-db1} =~ m/^\w/) {$database1 = $input{-db1}} else {die "The option -db1 requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-db2$/) {
   $db2check=1;
   if ($input{-db2} =~ m/^\w/) {$database2 = $input{-db2}} else {die "The option -db2 requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-ediff$/) {
   if ($input{-ediff} =~ m/^\d+/) {$ediff = $input{-ediff}} else {die "The option -ediff requires a number.\n"};
  } 
  if ($key[$a] =~ m/^-evalue$/) {
   if ($input{-evalue} =~ m/^\d+/) {$evalue = $input{-evalue}} else {die "The option -evalue requires a number.\n"};
  } 
  if ($key[$a] =~ m/^-ltp$/) {
   if ($input{-ltp} >= 0 and $input{-ltp} <= 1) {$lowparathres = $input{-ltp}} else {die "The option -ltp requires a value between 0 and 1 and which lower than the value provided by -upt.\n"};
  } 
  if ($key[$a] =~ m/^-utp$/) {
   if ($input{-utp} >= 0 and $input{-utp} <= 1) {$upparathres = $input{-utp}} else {die "The option -utp requires a value between 0 and 1 and which higher than the value provided by -lpt.\n"};
  } 
 }
 unless ($iptcheck==1 and $ipacheck==1) {die "For the blast searches input files containing the names of both the treefiles and the corresponding alignments has to be provided by -ipt filename and -ipa filename.\n"}
 unless ($ppfcheck==1) {die "For the blast searches an input file containing the sequences of each relevant partition to be pruned has to be provided by -ppf filename.\n"}
 unless ($db1check==1 and $db2check==1) {die "For the blast searches input files containing the names of two databases against which the BLAST search shall be conducted has to be provided by -db1 filename and -db2 filename.\n"}
 if ($lowparathres > $upparathres) {die "The value provided by -ltp has to be smaller than the value provided by -utp.\n"}
} 

if ($fun =~ m/^d$/) {
 my $iptcheck = 0;
 my $ipacheck = 0;
 my $ppfcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-ipa$/) {
   $ipacheck=1;
   if ($input{-ipa} =~ m/^\w/) {$alignfiles = $input{-ipa}} else {die "The option -ipa requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-ppf$/) {
   $ppfcheck=1;
   if ($input{-ppf} =~ m/^\w/) {$paralogfile = $input{-ppf}} else {die "The option -ppf requires a name which starts with a letter or number.\n"};
  } 
 }
 unless ($iptcheck==1 and $ipacheck==1) {die "For the automatic pruning input files containing the names of both the treefiles and the corresponding alignments has to be provided by -ipt filename and -ipa filename.\n"}
 unless ($ppfcheck==1) {die "For the automatic pruning an input file containing the sequences of each relevant partition to be pruned has to be provided by -ppf filename.\n"}
}

if ($fun =~ m/^e$/) {
 my $iptcheck = 0;
 my $tfcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-tf$/) {
   $tfcheck=1;
   if ($input{-tf} =~ m/^\w/) {$taxanames = $input{-tf}} else {die "The option -tf requires a name which starts with a letter or number.\n"};
  } 
 }
 unless ($tfcheck==1) {die "For LB score calculations an input file containing the taxa names of each relevant tree has to be provided by -tf filename.\n"}
 unless ($iptcheck==1) {die "For LB score calculations an input files containing the names of the treefiles has to be provided by -ipt filename.\n"}
}

if ($fun =~ m/^f$/) {
 my $iptcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
 }
 unless ($iptcheck==1) {die "For the calculation of average bootstrap supports in trees an input files containing the names of the treefiles has to be provided by -ipt filename.\n"}
}

if ($fun =~ m/^g$/) {
 my $iptcheck = 0;
 my $ipacheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-ipa$/) {
   $ipacheck=1;
   if ($input{-ipa} =~ m/^\w/) {$alignfiles = $input{-ipa}} else {die "The option -ipa requires a name which starts with a letter or number.\n"};
  } 
 }
 unless ($iptcheck==1 and $ipacheck==1) {die "For the calculation of the correlations of the pairwise patristic and p distances input files containing the names of both the treefiles and the corresponding alignments has to be provided by -ipt filename and -ipa filename.\n"}
}

if ($fun =~ m/^h$/) {
 my $afcheck = 0;
 my $pfcheck = 0;
 my $ubnpcheck = 0;
 my $lbnpcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-aln$/) {
   $afcheck=1;
   if ($input{-aln} =~ m/^\w/) {$alignment = $input{-aln}} else {die "The option -aln requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-partf$/) {
   $pfcheck=1;
   if ($input{-partf} =~ m/^\w/) {$partitionfile = $input{-partf}} else {die "The option -partf requires a name which starts with a letter or number.\n"};
  }
  if ($key[$a] =~ m/^-ubnp$/) {
   $ubnpcheck = 1;
   if ($input{-ubnp} =~ m/^\d+$/ and $input{-ubnp} > 0) {$maxx = $input{-ubnp}} else {die "The value of option -ubnp has to be above 0 and an integer.\n"}; 
  }
  if ($key[$a] =~ m/^-lbnp$/) {
   $lbnpcheck = 1;
   if ($input{-lbnp} =~ m/^\d+$/ and $input{-lbnp} > 0 and $input{-lbnp} <= $input{-ubnp}) {$minx = $input{-lbnp}} else {die "The value of option -lbnp has to be above 0 as well as less than or equal to the value of -ubnp and an integer.\n"}; 
  }
 }
 if ($input{-per} =~ m/Y|Yes|N|No/) {if ($input{-per} =~ m/Y|Yes/) {$per=1}} else {die "The option -per requires the input Y, Yes, N or No.\n"}
 if ($input{-per} =~ m/Y|Yes/) {
  if ($input{-nper} =~ m/^\d+$/ and $input{-nper} >= 1) {$nper = $input{-nper}} else {die "The value of option -nper has to be above 0 and an integer.\n"}; 
 }
 unless ($afcheck==1) {die "For the generation of all possible combinations of partitions an input file containing the concatenated alignment has to be provided by -aln filename.\n"}
 unless ($pfcheck==1) {die "For the generation of all possible combinations of partitions an input file containing the partition definitions in RAxML style has to be provided by -partf filename.\n"}
 unless ($ubnpcheck==1 and $lbnpcheck==1) {die "For the generation of all possible combinations of partitions the minimal and maximal number of partitions, which shall be combined, has to be provided by -lbnp integer and -ubnp integer .\n"}
} 

if ($fun =~ m/^i$/) {
 my $iptcheck = 0;
 my $tfcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-ipt$/) {
   $iptcheck=1;
   if ($input{-ipt} =~ m/^\w/) {$treefiles = $input{-ipt}} else {die "The option -ipt requires a name which starts with a letter or number.\n"};
  } 
  if ($key[$a] =~ m/^-tf$/) {
   $tfcheck=1;
   if ($input{-tf} =~ m/^\w/) {$taxanames = $input{-tf}} else {die "The option -tf requires a name which starts with a letter or number.\n"};
  }
  if ($key[$a] =~m/^-mint$/) {
   if ($input{-mint} =~ m/^\d+$/ and $input{-mint} <= 100 and $input{-mint} >= 1) {$lowthres = $input{-mint}} else {die "The value of option -mint has to be between 1 and 100 and an integer for -fun i.\n"}; 
  }
 }
 unless ($tfcheck==1) {die "For the summary of bootstrap support across partitions an input file containing the taxa names has to be provided by -tf filename.\n"}
 unless ($iptcheck==1) {die "For the summary of bootstrap support across partitions an input files containing the names of the treefiles has to be provided by -ipt filename.\n"}
}

if ($fun =~ m/^j$/) {
 my $tabfcheck = 0;
 my $pfcheck = 0;
 foreach my $a (0..$#key) {
  if ($key[$a] =~ m/^-per$/) {
   if ($input{-per} =~ m/Y|Yes|N|No/) {if ($input{-per} =~ m/Y|Yes/) {$permutc=1}} else {die "The option -per requires the input Y, Yes, N or No.\n"}
  }
  if ($key[$a] =~ m/^-wsrt$/) {
   if ($input{-wsrt} =~ m/Y|Yes|N|No/) {if ($input{-wsrt} =~ m/Y|Yes/) {$wsrtc=1}} else {die "The option -wsrt requires the input Y, Yes, N or No.\n"}
  }
  if ($key[$a] =~ m/^-partf$/) {
   $pfcheck=1;
   if ($input{-partf} =~ m/^\w/) {$partitionfile = $input{-partf}} else {die "The option -partf requires a name which starts with a letter or number.\n"};
  }
  if ($key[$a] =~ m/^-tabf$/) {
   $tabfcheck=1;
   if ($input{-tabf} =~ m/^\w/) {$tablefiles = $input{-tabf}} else {die "The option -tabf requires a name which starts with a letter or number.\n"};
  }
  if ($key[$a] =~ m/^-mint$/) {
   unless ($input{-mint} =~ m/^\d+$|^\d+\.\d+$/) {
    if ($input{-mint} =~ m/^N$|^none$/i) {$min = $input{-mint}} else {die "The option -mint requires as an input either N, none or threshold number for the minimum value possible for this support measurement (e.g., 0 for bootstrapping).\n"};
   } else {$min = $input{-mint}}
  }
  if ($key[$a] =~ m/^-maxt$/) {
   unless ($input{-maxt} =~ m/^\d+$|^\d+\.\d+$/) {
    if ($input{-maxt} =~ m/^N$|^none$/i) {$min = $input{-maxt}} else {die "The option -maxt requires as an input either N, none or threshold number for the maximum value possible for this support measurement (e.g., 100 for bootstrapping).\n"};
   } else {$min = $input{-maxt}}
  }
 }
 unless ($tabfcheck==1) {die "For the calculations of PABA results an input file listing the names of the files with the tabulated results of the different combinations of partitions has to be provided by -tabf filename.\n"}
 unless ($pfcheck==1) {die "For the calculations of PABA results an input file containing the partition definitions in RAxML style has to be provided by -partf filename.\n"}
 my $oricheck = 0;
 if ($permutc==1) {
  foreach my $a (0..$#key) {
   if ($key[$a] =~ m/^-ori$/) {
    $oricheck=1;
    if ($input{-ori} =~ m/^\w/) {$original = $input{-ori}} else {die "The option -ori requires a name which starts with a letter or number.\n"};
   } 
  } 
 unless ($oricheck==1) {die "For the calculation of significance based on permutated datasets an input file containing the name of the dataset with the original has to be provided by -ori filename.\n"}
 }
 my $bpfcheck = 0;
 if ($wsrtc==1) {
  foreach my $a (0..$#key) {
   if ($key[$a] =~ m/^-bpf$/) {
    if ($input{-bpf} =~ m/^\w/) {$bipartsets = $input{-bpf}} else {die "The option -bpf requires a name which starts with a letter or number.\n"}
   } else {die "For the WSRT test an input file containing the names of the files listing sets of bipartitions, which shall be independently tested, has to be provided by -bpf filename.\n"};
  } 
 }

}

#Program frame---------------------------------------------------------------------------------------------------
#A priori bootstrap support screening----------------------------------------------------------------------------
if ($fun =~ m/^a$/) {
foreach my $a ($begin..$end) {
  &BSSupportScreen ($a, $treefiles);
}
&NotFilterCompiled ($begin, $end);
if ($lbpos>0) {
 foreach my $lbthreshold ($lbbegin..$lbend) {
  if ($gtscreen == 1) {
   foreach my $taxafile (<GroupedTaxa*.txt>) {
    print "Working on ", $taxafile, " for masking.\n";
    &TaxaScreen ($taxafile, $fun);
    &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $taxafile, $blthreshold, $mintaxa, $treefiles);
    &CompileTree;  
    &FilterCompiled ($begin, $end);
    &Move ($taxafile);
   }
  &DirMove ($lbthreshold); 
  } else { open GFILE, ">GroupedTaxaNone.txt";close GFILE;
   foreach my $taxafile (<GroupedTaxaNone.txt>) {
    print "Working on filtering for screening.\n";
    &TaxaScreen ($taxafile, $fun);
    &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $taxafile, $blthreshold, $mintaxa, $treefiles);
    &CompileTree;  
    &FilterCompiled ($begin, $end);
    &Move ($taxafile);
   }
   &DirMove ($lbthreshold);
   unlink "GroupedTaxaNone.txt";
  }
 } 
} else { my $lbthreshold = "Not_provided";
  if ($gtscreen == 1) {
   foreach my $taxafile (<GroupedTaxa*.txt>) {
    print "Working on ", $taxafile, " for masking.\n";
    &TaxaScreen ($taxafile, $fun);
    &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $taxafile, $blthreshold, $mintaxa, $treefiles);
    &CompileTree;  
    &FilterCompiled ($begin, $end);
    &Move ($taxafile);
   }
  &DirMove ($lbthreshold); 
 } else { open GFILE, ">GroupedTaxaNone.txt";close GFILE;
   foreach my $taxafile (<GroupedTaxaNone.txt>) {
    print "Working on filtering for screening.\n";
    &TaxaScreen ($taxafile, $fun);
    &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $taxafile, $blthreshold, $mintaxa, $treefiles);
    &CompileTree;  
    &FilterCompiled ($begin, $end);
    &Move ($taxafile);
   }
  &DirMove ($lbthreshold);
  unlink "GroupedTaxaNone.txt";
 }
} 
}

#A posteriori bootstrap support screening----------------------------------------------------------------------------
if ($fun =~ m/^b$/) {
foreach my $a ($begin..$end) {
  &BSSupportScreen ($a, $treefiles);
}
&NotFilterCompiled ($begin, $end);
&SplitTree ($treefile); 
if ($lbpos>0) {
 foreach my $lbthreshold ($lbbegin..$lbend) {
  foreach my $apfile (<GroupedTaxa_A_posteriori_Groups.txt>) {
   print "Working on ", $apfile, " for screening.\n";
   &APScreen ($apfile);
   if ($gtscreen == 1) {
    foreach my $taxafile (<GroupedTaxa*.txt>) { unless ($taxafile eq $apfile) {
     print "Working on ", $taxafile, " for masking.\n";
     &TaxaScreen ($taxafile, $fun);
     &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $taxafile, $blthreshold, $mintaxa, $treefiles);
     &CompileTree;  
     &FilterCompiled ($begin, $end);
     &Move ($taxafile);
    } }
   } else {
   &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $apfile, $blthreshold, $mintaxa, $treefiles, $fun);
   &CompileTree;  
   &FilterCompiled ($begin, $end);
   &Move ($apfile);
   }
  }
 &DirMove ($lbthreshold);
 }
} else { my $lbthreshold = "Not_provided";
  foreach my $apfile (<GroupedTaxa_A_posteriori_Groups.txt>) {
   print "Working on ", $apfile, " for screening.\n";
   &APScreen ($apfile);
   if ($gtscreen == 1) {
    foreach my $taxafile (<GroupedTaxa*.txt>) {unless ($taxafile eq $apfile) {
     print "Working on ", $taxafile, " for masking.\n";
     &TaxaScreen ($taxafile, $fun);
     &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $taxafile, $blthreshold, $mintaxa, $treefiles);
     &CompileTree;  
     &FilterCompiled ($begin, $end);
     &Move ($taxafile);
    } }
   } else {
   &FilterScreen ($lbpos, $scpos, $sbpos, $lbthreshold, $apfile, $blthreshold, $mintaxa, $treefiles, $fun);
   &CompileTree;  
   &FilterCompiled ($begin, $end);
   &Move ($apfile);
   }
  }
 &DirMove ($lbthreshold);
}
}

#Automatic blast of problematic sequences---------------------------------------------------------------------------------
if ($fun =~ m/^c$/) {
 &blast ($paralogfile, $treefiles, $alignfiles, $database1, $database2, $ediff, $lowparathres, $upparathres, $evalue, $trespexpath, $returnpath)
}

#Automatic pruning of sequences---------------------------------------------------------------------------------
if ($fun =~ m/^d$/) {
 &autoprune ($paralogfile, $treefiles, $alignfiles)
}

#Calculations of LB scores---------------------------------------------------------------------------------
if ($fun =~ m/^e$/) {
 &lbscore ($taxanames, $treefiles)
}

#Calculations of average BS values---------------------------------------------------------------------------------
if ($fun =~ m/^f$/) {
 &aveBS ($treefiles)
}

#Determination of saturation based on plots of p vs patristic distances---------------------------------------------------------------------------------
if ($fun =~ m/^g$/) {
 &correlation ($treefiles, $alignfiles)
}

#Generation of different combinations of partitions and permutation of these combinations---------------------------------------------------------------------------------
if ($fun =~ m/^h$/) {
 &combination ($alignment, $partitionfile, $maxx, $minx, $nper, $per)
}

#Summary of bootstrap support across partitions---------------------------------------------------------------------------------
if ($fun =~ m/^i$/) {
 &comBS ($taxanames, $lowthres, $treefiles)
}

#PABA analyses---------------------------------------------------------------------------------
if ($fun =~ m/^j$/) {
 &PABA ($tablefiles, $partitionfile, $min, $max, $wsrtc, $bipartsets, $permutc, $original)
}

#print log-file with settings for the run--------------------------------------------------------------------
if ($fun =~ m/^a$|^b$/) {
 open LOGFILE, ">Bootstrap_Support_Screening.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 if ($fun =~ m/^a$/) {
  print LOGFILE "\n\nBootstrap values have been screened based on an a priori approach.\n"
 }
 if ($fun =~ m/^b$/) {
  print LOGFILE "\n\nBootstrap values have been screened based on an a posteriori approach.\n"
 }
 print LOGFILE "\nThus, these settings have been used in this run:\n\n";
 print LOGFILE "Bootstrap values have been screened in the range from ", $begin, " to ", $end, ".\n";
 if ($lbpos == 1) {
  print LOGFILE "The filtering for long internal branches was conducted at the first position.\nIt had been tested if the branch leading to the affected clade was at least ", $lbbegin, " times longer than the average of the internal branch lengths in the tree for the long internal branch screening. This has been repeated up to minimal threshold of ", $lbend, ".\n";
  if ($scpos == 2) {
   print LOGFILE "The filtering for strong conflicts was conducted at the second position.\n";
   if ($sbpos == 3) {
    print LOGFILE "The filtering for short terminal branches was conducted at the third position.\nFor the short terminal branch screening one of the terminal branches in the affected clade had to be smaller than ", $blthreshold, " and have not more than ", $mintaxa, " were in that clade.\n";
   }
  }
  if ($sbpos == 2) {
   print LOGFILE "The filtering for short terminal branches was conducted at the second position.\nFor the short terminal branch screening one of the terminal branches in the affected clade had to be smaller than ", $blthreshold, " and have not more than ", $mintaxa, " were in that clade.\n";
   if ($scpos == 3) {
    print LOGFILE "The filtering for strong conflicts was conducted at the third position.\n";
   }
  }
 }
 if ($scpos == 1) {
  print LOGFILE "The filtering for strong conflicts was conducted at the first position.\n";
  if ($lbpos == 2) {
   print LOGFILE "The filtering for long internal branches was conducted at the second position.\nIt had been tested if the branch leading to the affected clade was at least ", $lbbegin, " times longer than the average of the internal branch lengths in the tree for the long internal branch screening. This has been repeated up to minimal threshold of ", $lbend, ".\n";
   if ($sbpos == 3) {
    print LOGFILE "The filtering for short terminal branches was conducted at the third position.\nFor the short terminal branch screening one of the terminal branches in the affected clade had to be smaller than ", $blthreshold, " and have not more than ", $mintaxa, " were in that clade.\n";;
   }
  }
  if ($sbpos == 2) {
   print LOGFILE "The filtering for short terminal branches was conducted at the second position.\nFor the short terminal branch screening one of the terminal branches in the affected clade had to be smaller than ", $blthreshold, " and have not more than ", $mintaxa, " were in that clade.\n";
   if ($lbpos == 3) {
    print LOGFILE "The filtering for long internal branches was conducted at the third position.\nIt had been tested if the branch leading to the affected clade was at least ", $lbbegin, " times longer than the average of the internal branch lengths in the tree for the long internal branch screening. This has been repeated up to minimal threshold of ", $lbend, ".\n";
   }
  }
 }
 if ($sbpos == 1) {
  print LOGFILE "The filtering for short terminal branches was conducted at the first position.\nFor the short terminal branch screening one of the terminal branches in the affected clade had to be smaller than ", $blthreshold, " and have not more than ", $mintaxa, " were in that clade.\n";
  if ($lbpos == 2) {
   print LOGFILE "The filtering for long internal branches was conducted at the second position.\nIt had been tested if the branch leading to the affected clade was at least ", $lbbegin, " times longer than the average of the internal branch lengths in the tree for the long internal branch screening. This has been repeated up to minimal threshold of ", $lbend, ".\n";
   if ($scpos == 3) {
    print LOGFILE "The filtering for strong conflicts was conducted at the third position.\n";
   }
  }
  if ($scpos == 2) {
   print LOGFILE "The filtering for strong conflicts was conducted at the second position.\n";
   if ($lbpos == 3) {
    print LOGFILE "The filtering for long internal branches was conducted at the third position.\nIt had been tested if the branch leading to the affected clade was at least ", $lbbegin, " times longer than the average of the internal branch lengths in the tree for the long internal branch screening. This has been repeated up to minimal threshold of ", $lbend, ".\n";
   }
  }
 }
 print LOGFILE "\n";
 print LOGFILE "All groups passing the above filters were placed in a file named RemainingOnes.\n\n";
 if ($fun =~ m/^a$/) {
  print LOGFILE "For the taxon screening the following files have been used to mask the following groups:\n";
  foreach my $taxafile (<GroupedTaxa*.txt>) {
   print LOGFILE "\n", $taxafile, ":\n";
   open TAXAFILE, $taxafile or die "${taxafile} not found.\n";
    while (my $line = <TAXAFILE>) {
     print LOGFILE $line;
    }
   close TAXAFILE;
  }
 }
 if ($fun =~ m/^b$/) {
  print LOGFILE "Based on the tree in ", $treefile, " the BS support for following groups has been screened for:\n";
   open TAXAFILE, "GroupedTaxa_A_posteriori_Groups.txt" or die "GroupedTaxa_A_posteriori_Groups.txt not found.\n";
    while (my $line = <TAXAFILE>) {
     print LOGFILE $line;
    }
   close TAXAFILE;
 }
 if ($fun =~ m/^b$/ and $gtscreen==1) {
  print LOGFILE "For the taxon screening the following files have been used to mask the following groups:\n";
  foreach my $taxafile (<GroupedTaxa*.txt>) {
   print LOGFILE "\n", $taxafile, ":\n";
   open TAXAFILE, $taxafile or die "${taxafile} not found.\n";
    while (my $line = <TAXAFILE>) {
     print LOGFILE $line;
    }
   close TAXAFILE;
  }
 }
 print LOGFILE "\nThe follwing trees have been used for screening:\n";
  open FILES, $treefiles or die "${treefiles} not found.\n";
   while (my $fileline =<FILES>) {
    chomp $fileline;
    print LOGFILE $fileline, "\n";
   } 
  close FILES;
close LOGFILE;
}

if ($fun =~ m/^c$/) {
 open LOGFILE, ">BLAST_search_of_potentail_paralog_sequences.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $paralogfile, " provided the sequences to be pruned, the file ",$treefiles," the names of the tree files and the file ",$alignfiles," the names of the corresponding alignment files.\n";
 print LOGFILE "\nThe databases ", $database1, " and ",$database2," have been used for the BLAST searches with a e-value threshold of ",$evalue,".\n";
 print LOGFILE "\nFor the assessment if sequences were paralogous, not paralogous or an uncertain case the following thresholds of proportion have been used:\nThe lower value was ",$lowparathres,", which means that the best hit of all potentially paralogous sequences were also the best hit in less than ", ($lowparathres*100), "% of the best hits of the non-suspious sequences in at least one of the searches and hence were regarded as paralogous.\nThe upper value was ",$upparathres,", which means that the best hit of all potential paralogous sequences was also the best hit in more than ", ($upparathres*100), "% of the best hits of the non-suspious sequences in both searches and hence were regarded as not paralogous. All other sequences were regarded as uncertain.\n";
 print LOGFILE "\nFor hits to be part of the confidence set of best hits for each blasted sequence the value of the hit had to be not more than ", $ediff, " orders of magnitude higher than the e-value of the best hit.\n";
 close LOGFILE; 
}

if ($fun =~ m/^d$/) {
 open LOGFILE, ">Automatic_pruning.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $paralogfile, " provided the sequences to be pruned, the file ",$treefiles," the names of the tree files and the file ",$alignfiles," the names of the corresponding alignment files.\n";
 close LOGFILE; 
}

if ($fun =~ m/^e$/) {
 open LOGFILE, ">LB_score_calculation.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $taxanames, " provided the names of taxa for the calculation.\n";
 close LOGFILE; 
}

if ($fun =~ m/^f$/) {
 open LOGFILE, ">Summary_of_average_BS.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $treefiles, " provided for which the average bootstrap support has been calculated.\n";
 close LOGFILE; 
}

if ($fun =~ m/^g$/) {
 open LOGFILE, ">Summary_Saturation.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe files ", $treefiles," and ",$alignfiles, " provided the files for the saturation analysis based on pairwise p and patristic distances.\n";
 close LOGFILE; 
}

if ($fun =~ m/^h$/) {
 open LOGFILE, ">Summary_of_Combination_and_Permutation.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $alignment, " provided the concatenated alignment and the file ", $partitionfile," the definition of the partitions in this alignemnt.\n";
 print LOGFILE "All possible combinations of ", $minx," to ", $maxx," have been generated.\n";
 print LOGFILE "The definition of the individual partitions as given were:\n";
 open FILE, $partitionfile or die "${partitionfile} not found.\n";
  while (my $line =<FILE>) {
   chomp $line;
   print LOGFILE $line, "\n";
  }
 close FILE;
 if ($per==1) {print LOGFILE "\nThe generation of permutated datasets for statistical tests has been used for ", $nper," permutated datasets.\n"
 } else {print LOGFILE "\nThe generation of permutated datasets for statistical tests has NOT been used.\n"}
 close LOGFILE; 
}

if ($fun =~ m/^i$/) {
 open LOGFILE, ">Summary_of_BS_across_datasets.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $taxanames, " provided the names of taxa and all BS values larger than or equal to ",$lowthres," were kept.\n";
 close LOGFILE; 
}

if ($fun =~ m/^j$/) {
 open LOGFILE, ">Summary_of_PABA_calculations.log";
 print LOGFILE "TreSpEx was called as follows: perl TreSpEx.v1.pl ";
 foreach my $a (0..$#ARGV) {print LOGFILE $ARGV[$a], " "};
 print LOGFILE "\nThe file ", $tablefiles, " provided the names of the result files containing the matrix of support values vs. bipartitions.\n";
 print LOGFILE "The name of the result files were given:\n";
 open FILE, $tablefiles  or die "${tablefiles} not found.\n";
  while (my $line =<FILE>) {
   chomp $line;
   print LOGFILE $line, "\n";
  }
 close FILE;
 print LOGFILE "\nThe file ", $partitionfile," the definition of the partitions in this alignemnt.\n";
 print LOGFILE "The definition of the individual partitions as given were:\n";
 open FILE, $partitionfile or die "${partitionfile} not found.\n";
  while (my $line =<FILE>) {
   chomp $line;
   print LOGFILE $line, "\n";
  }
 close FILE;
 unless ($min =~ m/^\d+$|^\d+\.\d+$/) {print LOGFILE "No lower threshold for this support measurement was necessary.\n"} else {print LOGFILE "The lower threshold was ",$min,".\n"}
 unless ($max =~ m/^\d+$|^\d+\.\d+$/) {print LOGFILE "No upper threshold for this support measurement was necessary.\n"} else {print LOGFILE "The lower threshold was ",$max,".\n"}
 print LOGFILE "All possible combinations of ", $minx," to ", $maxx," have been generated.\n";
 if ($permutc==1) {print LOGFILE "\nThe siginificance of the PABA calculations of dataset ", $original," was determined using permutated datasets provided by ",$tablefiles,", which also included the original dataset (see above).\n"
 } else {print LOGFILE "\nThe determination based on permutated datasets has NOT been used.\n"}
 if ($wsrtc==1) {print LOGFILE "\nFor the individual sets of bipartitions provided by ",$bipartsets," WSRT tests were conducted for each partition provided by ",$partitionfile," and each dataset in ",$tablefiles,".\n";
  print LOGFILE "The definition of each individual set of bipartitions were given in the files:\n";
  open FILE, $bipartsets or die "${bipartsets} not found.\n";
   while (my $line =<FILE>) {
    chomp $line;
    print LOGFILE $line, "\n";
   }
 close FILE;
 } else {print LOGFILE "\nThe WSRT test has NOT been used.\n"}
 close LOGFILE; 
}

foreach my $a (0..$#key) {
 if ($key[$a] =~ m/^-path$/) {
  chdir $trespexpath or die "The folder ",$trespexpath," not found.\n";
 }
}

my ($usertime, $systemtime, $childsystem, $childuser) = times();
my @times = ($usertime, $systemtime, $childsystem, $childuser);
@times = sort { $a <=> $b } @times;
my $max = $times[$#times];
print("total time used: $max\n");

#Screening for Bootstrap values--------------------------------------------------------------------
sub BSSupportScreen {

my $a = $_[0];
my $treefiles = $_[1];
my $lowthreshold = $a-0.5;
my $highthreshold = $a+0.5;
my @files=();
open FILES, $treefiles or die "${treefiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @files, $fileline;
 } 
close FILES;
print "Screening for BS value = ", $a, "\n";
foreach my $ipf (0..$#files) {
 my $file = $files[$ipf];
 open FILE, $file or die "${file} not found.\n";
 while (my $line = <FILE>) {
  my $tree = $line;
  my $arraytree = $line;
  $arraytree =~ s/:/,:,/g;
  $arraytree =~ s/\(/\(,/g;
  $arraytree =~ s/\)/\),/g;
  $arraytree =~ s/(\d)\)/\1,\)/g;
  my @tree = reverse split /,/, $arraytree;
  foreach my $b (0..$#tree) {
  my $z=0;
  my $y=0;
  my $printout = ();
  my $taxa = ();
  my $blq = $b+1;
   if ($tree[$b] >= $lowthreshold and $tree[$b] < $highthreshold and $tree[$b] =~ m/\d/ and $tree[$blq] !~ /:/) {
    my $bl = $b - 2;
    my $first = 0;
    foreach my $bc ($b..$#tree) {
     if ($tree[$bc] eq "\)") {
      $z++; 
     }; 
     if ($tree[$bc] eq "\(") {
      $y++;
     }; 
     if ($z == $y and $tree[$bc] eq "\(" and $first == 0) {
      foreach my $wo ($b..$bc) {
       unless ($tree[$wo] eq "\)" or $tree[$wo] eq "\(" or $tree[$wo] eq ":" or $tree[$wo] !~ /^[A-Z]/i ) {
        $taxa .= "$tree[$wo] ";
       }
       $printout = $file;
       $printout .= " $tree[$b] ";
       $printout .= "$tree[$bl] ";
       $printout .= $taxa;
       $printout .= "\n";
       $first ++;
      }
     }
    }
   open OUTFILE, ">>PotentialParalogsBootstrap.txt";
    print OUTFILE "$printout";
   close OUTFILE;
   }
  } 
 }
 close FILE;
}

}

#Compiling data from non filtered bootstrap screen-----------------------------------------------
sub NotFilterCompiled {

my $begin = $_[0];
my $end = $_[1];
open OUTFILE, ">NOT_filtered_Hits_per_BS_value.csv";
 print OUTFILE "BS_value\tHits\n";
close OUTFILE;
foreach my $BSvalue ($begin..$end) {
 my $hitsfound = 0;
 open PARALOG, "PotentialParalogsBootstrap.txt" or die "PotentialParalogsBootstrap.txt not found.\n";
  while (my $paralogline = <PARALOG>) {
  my @paralogscreen = ();
  @paralogscreen = split /\s/, $paralogline;
   if ($paralogscreen[1] == $BSvalue) {
    $hitsfound ++ unless $paralogscreen[1] =~ m/\d+\.\d+/;
   }
  }
 close PARALOG;
 open OUTFILE, ">>NOT_filtered_Hits_per_BS_value.csv";
  print OUTFILE $BSvalue, "\t", $hitsfound, "\n";
 close OUTFILE;
}

}

#Filtering of screening results for taxa----------------------------------------------------------
sub TaxaScreen {

my $taxafile = $_[0];
my $fun = $_[1];
#In-group screening
my $file ="PotentialParalogsBootstrap.txt";
if ($fun =~ /^b$/) {$file="AP_PotentialParalogsBootstrap.txt"} 
print "Ingroup screening: ", $file, "\n";
open PARALOG, $file or die "${file} not found.\n";
 while (my $paralogline = <PARALOG>) {
 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my $match = $#paralogscreen;
 $match = $match-2;
 my $printnot = 0;
  
   open TAXA, $taxafile or die "${taxafile} not found.\n";
    while (my $taxaline = <TAXA>) {
     my @taxascreen = split /\s+/, $taxaline;
     my $pos = 0;
     foreach my $t (0..$#taxascreen) {
      foreach my $p (3..$#paralogscreen) {
       if ($paralogscreen[$p] eq $taxascreen[$t]) {
        $pos ++;
       }
      }
     }
     if ($pos == $match) {
      $printnot = 1;
     }
    }
   close TAXA;
  
 if ($printnot == 0) {
  open OUTFILE, ">>TMP1_PotentialParalogsBootstrap.txt";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 }
close PARALOG;


#Out-group screening
print "Outgroup screening: TMP1_PotentialParalogsBootstrap.txt\n";
open PARALOG, "TMP1_PotentialParalogsBootstrap.txt" or die "TMP1_PotentialParalogsBootstrap.txt not found.\n";
 while (my $paralogline = <PARALOG>) {
 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my $printnot = 0;
 my @tree=();
  open TREEFILE, $paralogscreen[0] or die "${paralogscreen[0]} not found.\n";
   while (my $treeline = <TREEFILE>) {
    $treeline =~ s/,/:/g;
    $treeline =~ s/\(//g;
    $treeline =~ s/\)//g;
    $treeline =~ s/\d+\.\d+//g;
    $treeline =~ s/;//g;
    $treeline =~ s/\:+/:/g;
    @tree = split /:/, $treeline;   
   } 
  close TREEFILE;
  my $totaltaxa = $#tree;

   open TAXA, $taxafile or die "${taxafile} not found.\n";
    while (my $taxaline = <TAXA>) {
     my @taxascreen = split /\s+/, $taxaline;
     my $pos = 0;
     my $outmatch = 0;
     my @ingroup = @tree;
     foreach my $t (0..$#taxascreen) {
      foreach my $a (0..$#ingroup) {
       if ($ingroup[$a] eq $taxascreen[$t]) {
        splice (@ingroup, $a, 1, "outgroup");
        $outmatch++;
       }
      }
     }
     foreach my $a (0..$#ingroup) {
      foreach my $p (3..$#paralogscreen) {
       if ($paralogscreen[$p] eq $ingroup[$a]) {
        $pos ++;
       }
      }
     }
     my $match = $pos + $outmatch;
     if ($totaltaxa == $match) {
      $printnot = 1;
     }
    }
   close TAXA;
  
 if ($printnot == 0) {
  open OUTFILE, ">>TMP2_PotentialParalogsBootstrap.txt";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 }
close PARALOG;

}

#Long branch, strong conflict and short terminal branch screening-----------------------------------------------
sub FilterScreen {

my $lbpos = $_[0]; 
my $scpos = $_[1]; 
my $sbpos = $_[2];
my $lbthreshold = $_[3];
my $taxafile = $_[4];
my $blthreshold = $_[5];
my $mintaxa = $_[6];
my $treefiles = $_[7];
my $fun = "a";
$fun = $_[8];
my $file ="TMP2_PotentialParalogsBootstrap.txt";
if ($fun =~ /^b$/) {$file="AP_PotentialParalogsBootstrap.txt"} 
print "Long branch, strong conflict and short terminal branch screening: ", $file, "\n";
my $contafile = $file;
$contafile =~ s/^TMP2/Pruned/;
$contafile =~ s/^AP/Pruned/;
$contafile =~ s/Paralogs/ShortTerminal/;
my $lbfile = $file;
$lbfile =~ s/^TMP2/Pruned/;
$lbfile =~ s/^AP/Pruned/;
$lbfile =~ s/Paralogs/LongBranchParalogs/;
my $scfile = $file;
$scfile =~ s/^TMP2/Pruned/;
$scfile =~ s/^AP/Pruned/;
$scfile =~ s/Paralogs/StrongConflict/;
my $restfile = $file;
$restfile =~ s/^TMP2/Pruned/;
$restfile =~ s/^AP/Pruned/;
$restfile =~ s/Paralogs/RemainingOnes/;
my $parafile = $file;
$parafile =~ s/^TMP2/Pruned/;
$parafile =~ s/^AP/Pruned/;
open PARALOG, $file or die "${file} not found.\n";
 while (my $paralogline = <PARALOG>) {
 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my $printconta = 0;
 my $printlb = 0; 
 my $printsc = 0; 
 my @lbtree=();
 my $lbtreefile = $paralogscreen[0]; 
 my $tcv = 0;
 my @files=();
 open FILES, $treefiles or die "${treefiles} not found.\n";
  while (my $fileline =<FILES>) {
   chomp $fileline;
   $fileline =~ s/\s//g;
   push @files, $fileline;
  } 
 close FILES;
 foreach my $ipf (0..$#files) {
  if ($paralogscreen[0] eq $files[$ipf]) {$tcv = 1}
 }
if ($tcv == 1) {
 open LBTREEFILE, $lbtreefile or die "${lbtreefile} not found.\n";
  while (my $lbtreeline = <LBTREEFILE>) {
   $lbtreeline =~ s/,/:/g;
   $lbtreeline =~ s/\(/\(:/g;
   $lbtreeline =~ s/\)\d+/\)/g;
   $lbtreeline =~ s/(\d)\)/\1:\)/g;
   @lbtree = split /:/, $lbtreeline;   
  } 
 close LBTREEFILE;
 
if ($lbpos == 1) {
 $printlb = &longbranch ($lbthreshold, $lbfile, $paralogline, \@paralogscreen, \@lbtree);
 if ($scpos == 2) {
  unless ($printlb == 1) {$printsc = &strongconflict ($taxafile, $scfile, $paralogline, \@paralogscreen, \@lbtree)};
  if ($sbpos == 3) {
   unless ($printlb == 1 or $printsc == 1) {$printconta = &shortbranch ($blthreshold, $contafile, $paralogline, \@paralogscreen, $mintaxa)};
   }
 }
 if ($sbpos == 2) {
  unless ($printlb == 1) {$printconta = &shortbranch ($blthreshold, $contafile, $paralogline, \@paralogscreen, $mintaxa)};
  if ($scpos == 3) {
   unless ($printlb == 1 or $printconta == 1) {$printsc = &strongconflict ($taxafile, $scfile, $paralogline, \@paralogscreen, \@lbtree)};
   }
 }
}

if ($scpos == 1) {
 $printsc = &strongconflict ($taxafile, $scfile, $paralogline, \@paralogscreen, \@lbtree);
 if ($lbpos == 2) {
  unless ($printsc == 1) {$printlb = &longbranch ($lbthreshold, $lbfile, $paralogline, \@paralogscreen, \@lbtree)};
  if ($sbpos == 3) {
   unless ($printlb == 1 or $printsc == 1) {$printconta = &shortbranch ($blthreshold, $contafile, $paralogline, \@paralogscreen, $mintaxa)};
   }
 }
 if ($sbpos == 2) {
  unless ($printsc == 1) {$printconta = &shortbranch ($blthreshold, $contafile, $paralogline, \@paralogscreen, $mintaxa)};
  if ($lbpos == 3) {
   unless ($printsc == 1 or $printconta == 1) {$printlb = &longbranch ($lbthreshold, $lbfile, $paralogline, \@paralogscreen, \@lbtree)};
   }
 }
}

if ($sbpos == 1) {
 $printconta = &shortbranch ($blthreshold, $contafile, $paralogline, \@paralogscreen, $mintaxa);
 if ($lbpos == 2) {
  unless ($printconta == 1) {$printlb = &longbranch ($lbthreshold, $lbfile, $paralogline, \@paralogscreen, \@lbtree)};
  if ($scpos == 3) {
   unless ($printlb == 1 or $printconta == 1) {$printsc = &strongconflict ($taxafile, $scfile, $paralogline, \@paralogscreen, \@lbtree)};
   }
 }
 if ($scpos == 2) {
  unless ($printconta == 1) {$printsc = &strongconflict ($taxafile, $scfile, $paralogline, \@paralogscreen, \@lbtree)};
  if ($lbpos == 3) {
   unless ($printsc == 1 or $printconta == 1) {$printlb = &longbranch ($lbthreshold, $lbfile, $paralogline, \@paralogscreen, \@lbtree)};
   }
 }
}
 
#The remaining cases
 unless ($printconta == 1 or $printsc == 1 or $printlb == 1) {
  open OUTFILE, ">>$restfile";
   print OUTFILE "$paralogline";
  close OUTFILE;
 } 
 
 #All pruned cases
  open OUTFILE, ">>$parafile";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 }
close PARALOG;

foreach my $file (<TMP*.txt>) {
 unlink $file;
}


}

#Compiling trees-------------------------------------------------------------------------------
sub CompileTree {

foreach my $file (<Pruned*.txt>) {
print "Trees compiling: ", $file, "\n";
my $filename = $file;
$filename =~ s/.txt/.tre/;
$filename =~ s/^Pruned/PrunedTrees/;
open PARATREES, ">>$filename";
 print PARATREES "#NEXUS\nBegin trees;\n";
close PARATREES;
open POTPARA, $file or die "${file} not found.\n";
my $previous = 0;
 while (my $paralogline = <POTPARA>) {
 my @paralogscreen = split /\s/, $paralogline;
 my $header = $#paralogscreen;
  unless ($paralogscreen[0] eq $previous or $header == 0) {
   open TREES, $paralogscreen[0] or die "${paralogscreen[0]} not found.\n";
    while (my $treeline = <TREES>) {
     open PARATREES, ">>$filename";
      print PARATREES "tree $paralogscreen[0] = [&U] ";
      print PARATREES "$treeline\n";
     close PARATREES;
    }
   close TREES;
  }
  $previous = $paralogscreen[0];
 }
close POTPARA;
open PARATREES, ">>$filename";
 print PARATREES "End;\n";
close PARATREES;
}

}

#Long branch screening---------------------------------------------------------------------------------
sub longbranch {

 my $lbthreshold = $_[0];
 my $lbfile = $_[1];
 my $paralogline = $_[2];
 my @paralogscreen = @{$_[3]};
 my @lbtree = @{$_[4]};
 my $printlb = 0;
 my $ibcounter = 0;
 my $ibsum = 0;
 my $average = 0;
 foreach my $lb (0..$#lbtree) {
  my $prev = $lb - 1;
  if ($lbtree[$lb] =~ m/\d+\.\d+/ and $lbtree[$prev] =~ m/\)/) {
   $ibcounter++;
   $ibsum = $ibsum + $lbtree[$lb];
  }
 }
 my $average = $ibsum/$ibcounter;
 my $lbthres = $lbthreshold * $average;
 if ($paralogscreen[2] > $lbthres) {
  $printlb = 1;
 }
 if ($printlb == 1) {
  open OUTFILE, ">>$lbfile";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 return $printlb;
 
}

#Strong conflict screening-------------------------------------------------------------------------------
sub strongconflict {

 my $taxafile = $_[0];
 my $scfile = $_[1];
 my $paralogline = $_[2];
 my @paralogscreen = @{$_[3]};
 my @lbtree = @{$_[4]};
 my $printsc = 0;
  open TAXAFILE, $taxafile or die "${taxafile} not found.\n";
   while (my $gtline = <TAXAFILE>) {
    my $treehits = 0;
    my $screenhits = 0;
    my @taxascreen = split /\s+/, $gtline;
    foreach my $t (0..$#taxascreen) {
     foreach my $lb (0..$#lbtree) {
      if ($lbtree[$lb] eq $taxascreen[$t]) {
       $treehits++;
      }
     }
    }
    foreach my $t (0..$#taxascreen) {
     foreach my $ps (0..$#paralogscreen) {
      if ($paralogscreen[$ps] eq $taxascreen[$t]) {
       $screenhits++;
      }
     }
    }
    if ($screenhits > 0 and $screenhits < $treehits) {
     $printsc = 1;
    }
   }
  close TAXAFILE;
  if ($printsc == 1) {
   open OUTFILE, ">>$scfile";
    print OUTFILE "$paralogline";
   close OUTFILE;
  }
  return $printsc;

} 

#Contamination screening--------------------------------------------------------------------------
sub shortbranch {
  
  my $blthreshold = $_[0];
  my $contafile = $_[1];
  my $paralogline = $_[2];
  my @paralogscreen = @{$_[3]};
  my $mintaxa = $_[4];
  my $printconta = 0;
  my @tree=();
  my $treefile = $paralogscreen[0];
  my $numtaxa = $#paralogscreen - 2;
  open TREEFILE, $treefile or die "${treefile} not found.\n";
   while (my $treeline = <TREEFILE>) {
    $treeline =~ s/\)\d+/\)/g;
    $treeline =~ s/\):\d+\.\d+//g;
    $treeline =~ s/\(//g;
    $treeline =~ s/\)//g;
    $treeline =~ s/;//g;
    $treeline =~ s/,/:/g;
    $treeline =~ s/\:+/:/g;
    @tree = split /:/, $treeline;   
  } 
  close TREEFILE;
  my $maxtaxa = ($#tree+1)/2 - $mintaxa; #necessary to screen outgroup for short terminal branches given that clade is small
  #ingroup screening 
  if ($numtaxa <= $mintaxa) { 
  foreach my $p (3..$#paralogscreen) {
   foreach my $t (0..$#tree) {
    if ($paralogscreen[$p] eq $tree[$t]) {
     my $next = $t +1;
     if ($tree[$next] < $blthreshold) {
      $printconta = 1;
     }
    }
   }
  }
  }
  #outgroup screening
  if ($numtaxa >= $maxtaxa) { 
  my @ingroup = @tree;
  foreach my $p (3..$#paralogscreen) {
   foreach my $t (0..$#ingroup) {
    if ($ingroup[$t] eq $paralogscreen[$p]) {
     splice (@ingroup, $t, 1, "not_relevant");
    }
   }
  }
  foreach my $t (0..$#ingroup) {
   unless ($ingroup[$t] eq "not_relevant" or $ingroup[$t] =~ m/\d+\.\d+/) {
     my $next = $t +1;
     if ($ingroup[$next] < $blthreshold) {
      $printconta = 1;
     }   
   }
  }
  }
 if ($printconta == 1) {
  open OUTFILE, ">>$contafile";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 return $printconta;

}

#Compiling data from filtered bootstrap screen--------------------------------------------------------------
sub FilterCompiled {

my $begin = $_[0];
my $end = $_[1];
my @files=();
foreach my $file (<Pruned_Potential*.txt>){
 push (@files, $file)
}
open OUTFILE, ">PrunedHits_per_BS_value.csv";
 print OUTFILE "BS_value\tHits for:\n\t";
 foreach my $a (0..$#files){
  my $category = $files[$a];
  $category =~ s/PrunedPotential//;
  $category =~ s/.txt//;
  print OUTFILE $category, "\t";
 }
 print OUTFILE "\n";
close OUTFILE;
foreach my $BSvalue ($begin..$end) {
 open OUTFILE, ">>PrunedHits_per_BS_value.csv";
  print OUTFILE $BSvalue, "\t";
 close OUTFILE;
 foreach my $a (0..$#files) {
  my $file = $files[$a];
  my $hitsfound = 0;
  open PARALOG, $file or die "${file} not found.\n";
   while (my $paralogline = <PARALOG>) {
    my @paralogscreen = ();
    @paralogscreen = split /\s/, $paralogline;
       if ($paralogscreen[1] == $BSvalue) {
        $hitsfound ++ unless $paralogscreen[1] =~ m/\d+\.\d+/;
       }
   }
  close PARALOG;
  open OUTFILE, ">>PrunedHits_per_BS_value.csv";
   print OUTFILE $hitsfound, "\t";
  close OUTFILE;
 }
 open OUTFILE, ">>PrunedHits_per_BS_value.csv";
  print OUTFILE "\n";
 close OUTFILE;
}

}

#Moving results from taxa screening in subfolder-----------------------------------------------------------------------
sub Move {

my $taxafile = $_[0];
my $dir = $taxafile;
$dir =~ s/^GroupedTaxa/TaxaScreening/;
$dir =~ s/.txt//;
print "Moving files to: ", $dir, "\n";
mkdir "${dir}";
foreach my $file (<Pruned*>) {
 rename $file, "$dir/$file";
}

}


#Moving folders into one folder------------------------------------------------------------------------------
sub DirMove {

my $lbthreshold = $_[0];
my $lbdir = "LBfactor";
$lbdir .= $lbthreshold;
print "Moving files to: ", $lbdir, "\n";
mkdir "${lbdir}";  
chdir $lbdir or die "The folder ",$lbdir," not found.\n";
my $location = ();
$location = getcwd();
chdir "..";  
 foreach my $folder (<TaxaScreening*>) {
  chdir $folder or die "The folder ",$folder," not found.\n";
  my $oldlocation = ();
  $oldlocation = getcwd();
  chdir "..";
  my $newlocation = $location;  
  $newlocation .= "/$folder";
  dirmove($oldlocation,$newlocation);
 }

}

#Filtering of screening results for taxa----------------------------------------------------------
sub APScreen {

my $taxafile = $_[0];
#In-group screening
my $file ="PotentialParalogsBootstrap.txt";
print "Ingroup screening: ", $file, "\n";
open PARALOG, $file or die "${file} not found.\n";
 while (my $paralogline = <PARALOG>) {
 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my $match = $#paralogscreen;
 $match = $match-2;
# my @tree=();
#  open TREEFILE, $paralogscreen[0] or die "${paralogscreen[0]} not found.\n";
#   while (my $treeline = <TREEFILE>) {
#    $treeline =~ s/,/:/g;
#    $treeline =~ s/\(//g;
#    $treeline =~ s/\)//g;
#    $treeline =~ s/\d+\.\d+//g;
#    $treeline =~ s/;//g;
#    $treeline =~ s/\:+/:/g;
#    @tree = split /:/, $treeline;   
#   } 
#  close TREEFILE;
 my $print = 0;
 
  
   open TAXA, $taxafile or die "${taxafile} not found.\n";
    while (my $taxaline = <TAXA>) {
      my @taxascreen = split /\s+/, $taxaline;
#     my $treehits = 0;
     my $pos = 0;
     foreach my $t (0..$#taxascreen) {
      foreach my $p (3..$#paralogscreen) {
       if ($paralogscreen[$p] eq $taxascreen[$t]) {
        $pos ++;
       }
      }
     }
#     foreach my $t (0..$#taxascreen) {
#      foreach my $lb (0..$#tree) {
#       if ($tree[$lb] eq $taxascreen[$t]) {
#        $treehits++;
#       }
#      }
#     }
     if ($pos == $match) { #and $treehits == $match
      $print = 1;
     }
    }
   close TAXA;
  
 if ($print == 1) {
  open OUTFILE, ">>AP_PotentialParalogsBootstrap.txt";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 }
close PARALOG;


#Out-group screening
print "Outgroup screening: PotentialParalogsBootstrap.txt\n";
open PARALOG, "PotentialParalogsBootstrap.txt" or die "PotentialParalogsBootstrap.txt not found.\n";
 while (my $paralogline = <PARALOG>) {
 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my $paramatch = $#paralogscreen;
 $paramatch = $paramatch-2;
 my $print = 0;
 my @tree=();
  open TREEFILE, $paralogscreen[0] or die "${paralogscreen[0]} not found.\n";
   while (my $treeline = <TREEFILE>) {
    $treeline =~ s/,/:/g;
    $treeline =~ s/\(//g;
    $treeline =~ s/\)//g;
    $treeline =~ s/\d+\.\d+//g;
    $treeline =~ s/;//g;
    $treeline =~ s/\:+/:/g;
    @tree = split /:/, $treeline;   
   } 
  close TREEFILE;
  my $totaltaxa = $#tree;

   open TAXA, $taxafile or die "${taxafile} not found.\n";
    while (my $taxaline = <TAXA>) {
     my @taxascreen = split /\s+/, $taxaline;
     my $pos = 0;
     my $outmatch = 0;
     my @ingroup = @tree;
     foreach my $t (0..$#taxascreen) {
      foreach my $a (0..$#ingroup) {
       if ($ingroup[$a] eq $taxascreen[$t]) {
        splice (@ingroup, $a, 1, "outgroup");
        $outmatch++;
       }
      }
     }
     foreach my $a (0..$#ingroup) {
      foreach my $p (3..$#paralogscreen) {
       if ($paralogscreen[$p] eq $ingroup[$a]) {
        $pos ++;
       }
      }
     }
     my $match = $pos + $outmatch;
     if ($totaltaxa == $match and $pos == $paramatch) {
      $print = 1;
     }
    }
   close TAXA;
  
 if ($print == 1) {
  open OUTFILE, ">>AP_PotentialParalogsBootstrap.txt";
   print OUTFILE "$paralogline";
  close OUTFILE;
 }
 }
close PARALOG;


}

#Filtering of screening results for taxa----------------------------------------------------------
sub SplitTree {

my $treefile = $_[0];
print "Working on ", $treefile, ".\n";
my @nodes=();
my $treecounter=0;
open TREEFILE, $treefile or die "${treefile} not found.\n";
 while (my $treeline = <TREEFILE>) {
 if ($treeline =~ /^\(/ and $treeline =~ /\);$/) {$treecounter++};
 }
close TREEFILE;
if ($treecounter == 0) {die "The treefile ", $treefile," does not contain a tree.\n"}
if ($treecounter > 1) {die "The treefile ", $treefile," contain more than one tree. Only one tree can be processed in this approach at a time.\n"}

#Build an array containing the tree
open TREEFILE, $treefile or die "${treefile} not found.\n";
 my @tree = ();
  while (my $treeline = <TREEFILE>) {
   $treeline =~ s/^\(//;
   $treeline =~ s/\);$//;
   $treeline =~ s/:\d+\.\d+/:/g;
   $treeline =~ s/,/:/g;
   $treeline =~ s/\(/\(:/g;
   $treeline =~ s/\)\d+/\)/g;
   $treeline =~ s/\)/:\):/g;
   $treeline =~ s/\:+/:/g;
   @tree = split /:/, $treeline;   
  } 
close TREEFILE;

#Determination of all clades in the tree 
 foreach my $b (0..$#tree) {
  my $z=0;
  my $y=0;
  my $printout = ();
  my $taxa = ();
   if ($tree[$b] eq "\(") {
    my $first = 0;
    foreach my $bc ($b..$#tree) {
     if ($tree[$bc] eq "\(") {
      $y++;
     }; 
     if ($tree[$bc] eq "\)") {
      $z++; 
     }; 
     if ($z == $y and $tree[$bc] eq "\)" and $first == 0) {
      my $node=();
       foreach my $wo ($b..$bc) {
        unless ($tree[$wo] =~ /^\($|^\)$/) {
         $node .= $tree[$wo]." ";
        }
        $first ++;
       }
       push @nodes, $node;
     }
    }
  }
 }


open OUTFILE, ">GroupedTaxa_A_posteriori_Groups.txt";
foreach my $a (0..$#nodes) {
 print OUTFILE $nodes[$a], "\n";
}
close OUTFILE;

}

#Automatic pruning of suspious sequences--------------------------------------------------------------------
sub autoprune {

mkdir "FilesPruned";
print "Automatic pruning of sequences.\nAs the output of the bootstrap screen depends on the root it is determined if the listed taxa are less than the non-listed taxa in the dataset.\nThe clade, which is smaller, will be pruned later.\n";
my $paralogfile = $_[0];
my $treefiles = $_[1];
my $alignfiles = $_[2];
my @treefiles=();
my @alignmentfiles=();
open FILES, $treefiles or die "${treefiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @treefiles, $fileline;
 } 
close FILES;
open FILES, $alignfiles or die "${alignfiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @alignmentfiles, $fileline;
 } 
close FILES;
unless ($#alignmentfiles == $#treefiles) {die "The two files providing the tree and alignmentfiles should have the same number of files in the same order.\n"}

open PARALOG, $paralogfile or die "${paralogfile} not found.\n";
while (my $paralogline = <PARALOG>) {
 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my @tree=();
  open TREEFILE, $paralogscreen[0] or die "${paralogscreen[0]} not found.\n";
   while (my $treeline = <TREEFILE>) {
    $treeline =~ s/,/:/g;
    $treeline =~ s/\(//g;
    $treeline =~ s/\)//g;
    $treeline =~ s/\d+\.\d+//g;
    $treeline =~ s/;//g;
    $treeline =~ s/\:+/:/g;
    @tree = split /:/, $treeline;   
   } 
  close TREEFILE;
  my $inmatch = 0;
  my $outmatch = 0;
  my @outgroup = @tree;
  foreach my $p (3..$#paralogscreen) {
   foreach my $a (0..$#outgroup) {
    if ($outgroup[$a] eq $paralogscreen[$p]) {
     splice (@outgroup, $a, 1, "ingroup");
     $inmatch++;
    }
   }
  }
  foreach my $o (0..$#outgroup) {
   unless ($outgroup[$o] eq "ingroup" or $outgroup[$o] !~ /[A-Z]/i) {
    $outmatch++;
   }
  }
  if (($outmatch + $inmatch) != $#tree) {
   print "Ingroup and outgroup numbers do not match number of taxa in the tree ", $paralogline and die;
  }
  if ($inmatch != ($#paralogscreen - 2)) {
   print "Ingroup number does not match number of taxa in the file provided ", $paralogline and die;
  }
  if ($outmatch <= $inmatch) {
   open OUTFILE, ">>IngroupOutgroup_$paralogfile";
    print OUTFILE $paralogscreen[0], " ", $paralogscreen[1], " ", $paralogscreen[2];
    foreach my $o (0..$#outgroup) {
     unless ($outgroup[$o] eq "ingroup" or $outgroup[$o] !~ /[A-Z]/i) {
      print OUTFILE " ", $outgroup[$o];
     }
    }
    print OUTFILE "\n";    
   close OUTFILE; 
  } 
  if ($outmatch > $inmatch) {
   open OUTFILE, ">>IngroupOutgroup_$paralogfile";
    print OUTFILE $paralogline;
   close OUTFILE; 
  } 
}
close PARALOG;

print "Compiling taxa to be pruned for each dataset.\n";
foreach my $file (<IngroupOutgroup*>) {
 my $compilefile = $file;
 $compilefile =~ s/^IngroupOutgroup_/Compiled_/;
 open FILE, $file or die "${file} not found.\n";
  my @filescreen = ();
  while (my $line =<FILE>) {
   my @paralogscreen = ();
   @paralogscreen = split /\s/, $line;
   my $push = 1;
   foreach my $a (0..$#filescreen) {
    if ($paralogscreen[0] eq $filescreen[$a]) {
     $push = 0;
    }
   }
   if ($push == 1) {
    push (@filescreen, $paralogscreen[0]);
   }
  } 
 close FILE;
 foreach my $a (0..$#filescreen) {
 open OUTFILE, ">>$compilefile";
  print OUTFILE $filescreen[$a];
 close OUTFILE;
  my @taxascreen = ();
  open FILE, $file or die "${file} not found.\n";
   while (my $line = <FILE>) {
   my @paralogscreen = ();
   @paralogscreen = split /\s/, $line;
   if ($paralogscreen[0] eq $filescreen[$a]) {
    foreach my $p (3..$#paralogscreen) {
     my $push = 1;
     foreach my $t (0..$#taxascreen) {
      if ($paralogscreen[$p] eq $taxascreen[$t]) {
       $push = 0;
      }
     }
     if ($push == 1) {
      push (@taxascreen, $paralogscreen[$p]);
     }
    }   
   }   
   }
  close FILE;
  foreach my $t (0..$#taxascreen) {
   open OUTFILE, ">>$compilefile";
    print OUTFILE " ", $taxascreen[$t];
   close OUTFILE;
  }
  open OUTFILE, ">>$compilefile";
   print OUTFILE "\n";
  close OUTFILE;
 }
}

print "Pruning taxa from the datasets.\nDatasets pruned:\n";
my @filespruned = ();
foreach my $excludefile (<Compiled*>) {
open EXCLUDE, $excludefile or die "${excludefile} not found.\n";
 while (my $paralogline = <EXCLUDE>) {

 my @paralogscreen = ();
 @paralogscreen = split /\s/, $paralogline;
 my $numdelt = $#paralogscreen;
 my $infofile = $paralogscreen[0];
 my $seqfile = ();
 foreach my $a (0..$#treefiles) {
  if ($paralogscreen[0] eq $treefiles[$a]) {$seqfile = $alignmentfiles[$a]}
 }
 print $seqfile, "\n";
 push (@filespruned, $seqfile);
 open SEQFILE, $seqfile or die "${seqfile} not found.\n";
  while (my $seqline =<SEQFILE>) {
   my $firstline = 0;
   my $printout = 1;
   if ($seqline =~ /^\d/ and $firstline == 0) {
    my @phyheader = split /\s/, $seqline;
    my $newnt = $phyheader[0] - $numdelt;
    open OUTFILE, ">>FilesPruned/${seqfile}";
     print OUTFILE $newnt, "\t", $phyheader[1], "\n";
    close OUTFILE;
    $printout = 0;
    $firstline = 1;
   }
   foreach my $p (1..$#paralogscreen) {
    if ($seqline =~ /^$paralogscreen[$p]/) {
     $printout = 0;
    }
   }
   if ($printout == 1) {
      open OUTFILE, ">>FilesPruned/${seqfile}";
       print OUTFILE $seqline;
      close OUTFILE;
   }

  }
 close SEQFILE;

 }
close EXCLUDE;
}

print "Copying datasets, from which no taxa have been pruned, to the folder \"FilesNotPruned\".\n";
mkdir "FilesNotPruned";
my $location = ();
$location = getcwd();
foreach my $old (0..$#alignmentfiles) {
 my $oldlocation = $location;
 $oldlocation .= "/" . $alignmentfiles[$old];
 my $newlocation = $location;
 $newlocation .= "/FilesNotPruned/" . $alignmentfiles[$old];
 my $move = 1;
 foreach my $pruned (0..$#filespruned) {
  if ($filespruned[$pruned] eq $alignmentfiles[$old]) {
   $move = 0;
  }
 }
 if ($move == 1) {
  copy ($oldlocation,$newlocation);
 }
}

}

#Calculations of LB scores---------------------------------------------------------------------------------
sub lbscore {

#Determination all taxa across all trees
mkdir "LBscore_Results";
chdir "LBscore_Results" or die "The folder LBscore_Results not found.\n";
mkdir "PD_matrices";
chdir "..";
my $taxanames = $_[0];
my $treefiles = $_[1];
my @taxafile=();
my @treefiles=();
my @treevalues=();
my @TRvalues=();#new
my @hetero=();
my @TRhetero=();#new
my @avePD=();
my @files=();
open FILES, $treefiles or die "${treefile} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @files, $fileline;
 } 
close FILES;
open TAXAFILE, $taxanames or die "${taxanames} not found.\n";
 while (my $taxaline =<TAXAFILE>) {
  chomp $taxaline;
  push @taxafile, $taxaline;
 } 
close TAXAFILE;

my %LBmatrix=();
my %TRmatrix=(); #new
foreach my $ipf (0..$#files) {
my $treefile = $files[$ipf];
print "Working on ", $treefile, ".\n";
push @treefiles, $treefile;
my %nodes=();
my %PDmatrix=();
my %LBscore=();
my @TaxaScores=();
my @TRScores=();#new

#Build an array containing the tree
 open TREEFILE, $treefile or die "${treefile} not found.\n";
 my @tree = ();
  while (my $treeline = <TREEFILE>) {
   $treeline =~ s/0\.0;//g;
   $treeline =~ s/\)\d+\.\d+:/\):/g;
   $treeline =~ s/\)\d+:/\):/g;
   $treeline =~ s/,/:/g;
   $treeline =~ s/\(/\(:/g;
   $treeline =~ s/(\d)\)/\1:\)/g;
   @tree = split /:/, $treeline;   
  } 
 close TREEFILE;

#Determination of taxa in the specific tree, number of taxa and hash pairs of taxon and terminal branch length 
 my @treetaxa=();
 my $taxacount=0;
 foreach my $a (0..$#tree) {
  if ($tree[$a] =~ /\w/ and $tree[$a] !~ /^\d+$/ and $tree[$a] !~ /^\d+\.\d+$/) {
   push @treetaxa, $tree[$a];
   my $bl = $a+1;
   $nodes{$tree[$a]}=$tree[$bl];
   $taxacount++;
  }
 }
 my $matcount=$taxacount-1;

#Crosscheck if all taxa in the specific tree are in the taxa list for all trees and occur only once 
 foreach my $a (0..$#treetaxa) {
  my $found = 0;
  foreach $b (0..$#taxafile) {
   if ($taxafile[$b] eq $treetaxa[$a]) {
    $found++;
   }
  }
  if ($found == 0) {print $treetaxa[$a], " not found in taxon file.\n" and die};
  if ($found > 1) {print $treetaxa[$a], " found ", $found, " times in taxon file.\n" and die};
 }

#Crosscheck if all taxa in the specific tree occur only once 
 foreach my $a (0..$#taxafile) {
  my $found = 0;
  foreach my $b (0..$#treetaxa) {
   if ($taxafile[$a] eq $treetaxa[$b]) {
    $found++;
   }
  }
 if ($found > 1) {print $taxafile[$a], " found ", $found, " times in the tree.\n" and die};
 } 
 
#Determination of all internal nodes in the tree and hash pairs of the node and internal branch length leading to that node 
 foreach my $b (0..$#tree) {
  my $z=0;
  my $y=0;
  my $printout = ();
  my $taxa = ();
   if ($tree[$b] eq "\(") {
    my $first = 0;
    foreach my $bc ($b..$#tree) {
     if ($tree[$bc] eq "\(") {
      $y++;
     }; 
     if ($tree[$bc] eq "\)") {
      $z++; 
     }; 
     if ($z == $y and $tree[$bc] eq "\)" and $first == 0) {
      my $bl = $bc+1;
      my $node=();
      foreach my $wo ($b..$bc) {
        $node .= $tree[$wo];
       $first ++;
      }
      $nodes{$node}=$tree[$bl];
     }
    }
  }
 }
 
 my $treedist=();
 foreach my $c (0..$#taxafile) {
  my $taxadist=();
  my $PDlist=();
  foreach my $d (0..$#taxafile) {
   my $cfound = 0;
   my $dfound = 0;
    foreach my $e (0..$#treetaxa) {
     if ($taxafile[$c] eq $treetaxa[$e]) {
      $cfound++;
     }
    }
    foreach my $f (0..$#treetaxa) {
     if ($taxafile[$d] eq $treetaxa[$f]) {
      $dfound++;
     }
    }
  my $patdist=();
   unless ($cfound == 0 or $dfound == 0) {
    while ((my $key, my $value) = each %nodes) {
     if ($key =~ /^${taxafile[$c]}$/) {
      unless ($key =~ /^${taxafile[$d]}$/) {
       $patdist+=$value;
      }
     }
     if ($key =~ /${taxafile[$c]}\d/) {
      unless ($key =~ /${taxafile[$d]}\d/) {
       $patdist+=$value;
      }
     }
     if ($key =~ /^${taxafile[$d]}$/) {
      unless ($key =~ /^${taxafile[$c]}$/) {
       $patdist+=$value;
      }
     }
     if ($key =~ /${taxafile[$d]}\d/) {
      unless ($key =~ /${taxafile[$c]}\d/) {
       $patdist+=$value;
      }
     }
    }
   }
   $PDlist .= "\t${patdist}";
   $taxadist+=$patdist;
   $treedist+=$patdist;
  }
  $taxadist/=$matcount;
  $PDmatrix{$taxafile[$c]}=$PDlist;
  $LBscore{$taxafile[$c]}=$taxadist;
 }
 $treedist/=($matcount*$taxacount);
 push @avePD, $treedist;

#Calculation of tip to root distances 
 foreach my $c (0..$#taxafile) {
   my $cfound = 0;
    foreach my $e (0..$#treetaxa) {
     if ($taxafile[$c] eq $treetaxa[$e]) {
      $cfound++;
     }
    }
  my $TRdist=();
   unless ($cfound == 0) {
    while ((my $key, my $value) = each %nodes) {
     if ($key =~ /^${taxafile[$c]}$/) {
       $TRdist+=$value;
     }
     if ($key =~ /${taxafile[$c]}\d/) {
       $TRdist+=$value;
     }
    }
   }
  $TRmatrix{$taxafile[$c]}.="\t${TRdist}";
  if (defined $TRdist) {push @TRScores, $TRdist};
 }

 @TRScores = sort {$b<=>$a} (@TRScores);
 my $TRsumup=();
 my $TRaverage=0;
 foreach my $a (0..$#TRScores) {
  $TRsumup+=$TRScores[$a];
 }
 my $TRaverage=($TRsumup/$taxacount);
 my $TRsqtotal = 0;
 foreach my $a (0..$#TRScores) {
  $TRsqtotal+=($TRScores[$a]-$TRaverage) ** 2;
 }
 my $TRstd = ($TRsqtotal/($taxacount-1)) ** 0.5;
 push @TRhetero, $TRstd;
 
 my $TRsumupper=();
 my $TRuppertaxa = sprintf("%.0f",($taxacount/4));
 my $TRupperbound = $TRuppertaxa-1;
 foreach my $a (0..$TRupperbound) {
  $TRsumupper+=$TRScores[$a];
 }
 push @TRvalues, ($TRsumupper/$TRuppertaxa);

#Printout of pairwise patristic distance matrix
 open OUTFILE, ">LBscore_Results/PD_matrices/${treefile}_PDmatrix.txt";
 print OUTFILE "\t";
 foreach my $a (0..$#taxafile) {
    print OUTFILE $taxafile[$a], "\t";
 }
 print OUTFILE "\n";
 foreach my $a (0..$#taxafile) {
  while ((my $key, my $value) = each %PDmatrix) {
   if ($key =~ /^${taxafile[$a]}$/) {
    print OUTFILE $key, $value, "\n"
   }
  }
 }
 close OUTFILE;

#Calculation of LB score
 foreach my $a (0..$#taxafile) {
  while ((my $key, my $value) = each %LBscore) {
   if ($key =~ /^${taxafile[$a]}$/) {
    if ($value == 0) {$value=()} else {     
     $value/=$treedist;
     $value-=1;
     $value*=100;
     push @TaxaScores, $value;
    }
    $LBmatrix{$key}.="\t${value}";
   }
  }
 }
 @TaxaScores = sort {$b<=>$a} (@TaxaScores);
 my $sumup=();
 my $average=0;
 foreach my $a (0..$#TaxaScores) {
  $sumup+=$TaxaScores[$a];
 }
 my $average=($sumup/$taxacount);
 my $sqtotal = 0;
 foreach my $a (0..$#TaxaScores) {
  $sqtotal+=($TaxaScores[$a]-$average) ** 2;
 }
 my $std = ($sqtotal/($taxacount-1)) ** 0.5;
 push @hetero, $std;
 
 my $sumupper=();
 my $uppertaxa = sprintf("%.0f",($taxacount/4));
 my $upperbound = $uppertaxa-1;
 foreach my $a (0..$upperbound) {
  $sumupper+=$TaxaScores[$a];
 }
 push @treevalues, ($sumupper/$uppertaxa);
}

#Printout results
open OUTFILE, ">LBscore_Results/LB_scores_perTaxon.txt";
print OUTFILE "\t";
foreach my $a (0..$#treefiles) {
    print OUTFILE $treefiles[$a], "\t";
}
print OUTFILE "\n";
foreach my $a (0..$#taxafile) {
 while ((my $key, my $value) = each %LBmatrix) {
  if ($key =~ /^${taxafile[$a]}$/) {
   print OUTFILE $key, $value, "\n";
  }
 }
}
close OUTFILE,

open OUTFILE, ">LBscore_Results/TR_scores_perTaxon.txt";
print OUTFILE "\t";
foreach my $a (0..$#treefiles) {
    print OUTFILE $treefiles[$a], "\t";
}
print OUTFILE "\n";
foreach my $a (0..$#taxafile) {
 while ((my $key, my $value) = each %TRmatrix) {
  if ($key =~ /^${taxafile[$a]}$/) {
   print OUTFILE $key, $value, "\n";
  }
 }
}
close OUTFILE,

open OUTFILE, ">LBscore_Results/LB_scores_summary_perPartition.txt";
print OUTFILE "Gene\tLB_score_upper_quartile\tLB_score_Heterogeneity\tTip_to_Root_upper_quartile\tTip_to_Root_Heterogeneity\tAverage_PD\n"; #new
foreach my $a (0..$#treefiles) {
 print OUTFILE $treefiles[$a], "\t", $treevalues[$a], "\t", $hetero[$a], "\t", $TRvalues[$a], "\t", $TRhetero[$a], "\t", $avePD[$a], "\n"; #new
}
close OUTFILE;


}

#Calculations of average bootstrap values per partition---------------------------------------------------------------------------------
sub aveBS {

mkdir "Average_BS_Results";
my $treefiles = $_[0];
my @treefiles=();
my @treevalues=();
my @files=();
open FILES, $treefiles or die "${treefiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @files, $fileline;
 } 
close FILES;

foreach my $ipf (0..$#files) {
my $treefile = $files[$ipf];
print "Working on ", $treefile, ".\n";
push @treefiles, $treefile;
my %nodes=();

#Build an array containing the tree
 open TREEFILE, $treefile or die "${treefile} not found.\n";
 my @tree = ();
  while (my $treeline = <TREEFILE>) {
   $treeline =~ s/:\d+\.\d+/:/g;
   $treeline =~ s/,/:/g;
   $treeline =~ s/\(/\(:/g;
   $treeline =~ s/\)(\d)/\):\1/g;
   $treeline =~ s/\:+/:/g;
   @tree = split /:/, $treeline;   
  } 
 close TREEFILE;

#Calculation of average bootstrap support for each tree 
 foreach my $b (0..$#tree) {
  my $z=0;
  my $y=0;
  my $printout = ();
  my $taxa = ();
   if ($tree[$b] eq "\(") {
    my $first = 0;
    foreach my $bc ($b..$#tree) {
     if ($tree[$bc] eq "\(") {
      $y++;
     }; 
     if ($tree[$bc] eq "\)") {
      $z++; 
     }; 
     if ($z == $y and $tree[$bc] eq "\)" and $first == 0) {
      my $bl = $bc+1;
      my $node=();
      unless ($tree[$bl] =~ /\D+/) {
       foreach my $wo ($b..$bc) {
        unless ($tree[$wo] =~ /^\($|^\)$|^\d+$/) {
         $node .= $tree[$wo]." ";
        }
        $first ++;
       }
       $nodes{$node}=$tree[$bl];
      }
     }
    }
  }
 }

my $BSsum = 0;
my $NumBS = 0;
foreach my $key (sort keys %nodes) {
 $BSsum += $nodes{$key};
 $NumBS++;
}
my $AveBS = $BSsum/$NumBS;
push @treevalues, $AveBS;
}

open OUTFILE, ">Average_BS_Results/Average_BS_perPartition.txt";
print OUTFILE "Gene\tAverage_BS\n";
foreach my $a (0..$#treefiles) {
 print OUTFILE $treefiles[$a], "\t", $treevalues[$a], "\n";
}
close OUTFILE;


}

#Summary of bootstrap support across partitions---------------------------------------------------------------------------------
sub comBS {

my $taxanames = $_[0];
my $lowthres = $_[1];
my $treefiles = $_[2];
my $lowval = $lowthres;
$lowval = $lowval-1;
mkdir "Bipartition_Results";
my @files=();
open FILES, $treefiles or die "${treefiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @files, $fileline;
 } 
close FILES;
my @treefiles=();
my @treevalues=();
my @taxafile=();
open TAXAFILE, $taxanames or die "${taxanames} not found.\n";
 while (my $taxaline =<TAXAFILE>) {
  chomp $taxaline;
  push @taxafile, $taxaline;
 } 
close TAXAFILE;

foreach my $ipf (0..$#files) {
my %BSvalue=();
my %biinv=();
my %biingroup=();
my %bioutgroup=();
my $treefile = $files[$ipf];

#Build an array containing the tree
 open TREEFILE, $treefile or die "${treefile} not found.\n";
 print "Working on ",$treefile,".\n";
 print "Progress  ";
 my $linecounter;
 my @tree = ();
  while (my $treeline = <TREEFILE>) {
   if ($treeline =~ /^\(/) {
    $linecounter++;
    (my $decicheck = $linecounter) /= 10;
    if ($decicheck !~ m/\d+\.\d+/) {print $linecounter} else {print "."};
   }
   $treeline =~ s/,/:/g;
   $treeline =~ s/\(/\(:/g;
   $treeline =~ s/\)/:\):/g;
   $treeline =~ s/\:+/:/g;
   @tree = split /:/, $treeline;   

#Crosscheck if all taxa in the specific tree are in the taxa list for all trees and occur only once
 foreach my $a (0..$#taxafile) {
  my $found=0;
  foreach my $b (0..$#tree) {
   if ($tree[$b] eq $taxafile[$a]) {
    $found++;
   }
  }
  if ($found == 0) {print $taxafile[$a], " not found in ", $treefile, ".\n" and die};
  if ($found > 1) {print $taxafile[$a], " found ", $found, " times in ", $treefile, ".\n" and die};
 }
 
#Determination of bipartitions across all partitions and BS values of individual partitions 
 DONE: foreach my $b (0..$#tree) {
  my $z=0;
  my $y=0;
  my $printout = ();
  my $taxa = ();
   if ($tree[$b] eq "\(") {
    my $first = 0;
    foreach my $bc ($b..$#tree) {
     if ($tree[$bc] eq "\(") {
      $y++;
     }; 
     if ($tree[$bc] eq "\)") {
      $z++; 
     }; 
     if ($z == $y and $tree[$bc] eq "\)" and $first == 0) {
      my $intaxa=();
      my $outtaxa=();
       foreach my $wo ($b..$bc) {
        unless ($tree[$wo] =~ /^\($|^\)$/) {
         $intaxa .= $tree[$wo]." ";
        }
        $first ++;
       }
       foreach my $ot (0..$#taxafile) {
        my @intaxa = split /\s/, $intaxa;
        my $infound=0;
        foreach my $it (0..$#intaxa) {
         if ($intaxa[$it] eq $taxafile[$ot]) {$infound=1}
        }
        unless ($infound==1) {
         $outtaxa .= $taxafile[$ot]." ";
        }
       }
      my $bipart = ();
      my $inverse = ();
      foreach my $taxon (0..$#taxafile) {
       my $taxfound = 0;
       my @intaxa = split /\s/, $intaxa;
        foreach my $it (0..$#intaxa) {
         if ($intaxa[$it] eq $taxafile[$taxon]) {$taxfound = 1}
        }
       if ($taxfound==1) {$bipart .= "i"} else {$bipart .= "o"}
       if ($taxfound==1) {$inverse .= "o"} else {$inverse .= "i"}
      }
      my $keyfound = 0;
      foreach my $key (sort keys %biinv) {
        if ($key eq $bipart) {
         $keyfound=1;
         $BSvalue{$key}++;
         next DONE;
        }
        unless ($keyfound==1) {
         if ($key eq $inverse) {
          $keyfound=1;
          $BSvalue{$key}++;
          next DONE;
         }
        }
      }
      unless ($keyfound==1) { 
       $biinv{$bipart}=$inverse;
       $biingroup{$bipart}=$intaxa;
       $bioutgroup{$bipart}=$outtaxa;
       $BSvalue{$bipart}++;
      }
     }
    }
  }
 }

 } 
 print "\nGenerating output file for ",$treefile," in the folder Bipartition_Results.\n",$linecounter," trees found.\n";
 open OUTFILE, ">Bipartition_Results/Bipartition_Table_${treefile}.txt";
 print OUTFILE "\t\t\t\t";
 print OUTFILE "BS_value\n";
 foreach my $key (sort keys %biinv)  {
  my $bipartinv = $biinv{$key};
  my $ingroup = $biingroup{$key};
  my $outgroup = $bioutgroup{$key};
  my $BSvalues = $BSvalue{$key};
  $BSvalues /= $linecounter;
  $BSvalues *= 100;
  if ($BSvalues >= $lowthres) {
   print OUTFILE $key, "\t", $bipartinv, "\t", $ingroup, "\t", $outgroup, "\t", $BSvalues, "\n";
  }
 }
 close OUTFILE,
 close TREEFILE;
}
print "All bipartitions and BS values of partitions were determined.\n";

chdir "Bipartition_Results" or die "The folder Bipartition_Results not found.\n";
#Determination of all bipartitions across all partitions
my %BSmatrix=();
my %bipart=();
my %biingroup=();
my %bioutgroup=();
foreach my $bipartfile (<Bipartition_Table_*.txt>) {
open BIFILE, $bipartfile or die "${bipartfile} not found.\n";
DONE: while (my $bipartline = <BIFILE>) {
  my $keyfound=0;
  chomp $bipartline;
  if ($bipartline =~ m/^\w/) {
   my @bipartline = split /\t/, $bipartline;
   my $bipart = $bipartline[0];
   my $inverse = $bipartline[1];
   my $intaxa = $bipartline[2];
   my $outtaxa = $bipartline[3];
      foreach my $key (sort keys %bipart) {
        if ($key eq $bipart) {
         $keyfound=1;
         next DONE;
        }
        unless ($keyfound==1) {
         if ($key eq $inverse) {
          $keyfound=1;
          next DONE;
         }
        }
      }
      unless ($keyfound==1) { 
       $bipart{$bipart}=$inverse;
       $biingroup{$bipart}=$intaxa;
       $bioutgroup{$bipart}=$outtaxa;
      }
  }
 }
close BIFILE;

}

foreach my $bipartfile (<Bipartition_Table_*.txt>) {
(my $treefile=$bipartfile) =~ s/Bipartition_Table_//;
$treefile =~ s/.txt//;
push @treefiles, $treefile;
my @biparts = ();

#Determination of BS values per bipartition and partition 
open BIFILE, $bipartfile or die "${bipartfile} not found.\n";
 while (my $bipartline = <BIFILE>) {
  chomp $bipartline;
  if ($bipartline =~ m/^\w/) {
   push @biparts, $bipartline;
  }
 }
close BIFILE;
foreach my $key (sort keys %bipart) {
 my $found=0;
 foreach my $a (0..$#biparts) {  
  my @bipartline = split /\t/, $biparts[$a];
   if ($key eq $bipartline[0]) {
    my $value = $bipartline[4];
    $found=1;
    $BSmatrix{$key}.="\t${value}";
   }
   unless ($found==1) {
    if ($key eq $bipartline[1]) {
     my $value = $bipartline[4];
     $found=1;
     $BSmatrix{$key}.="\t${value}";
    }
   }
  }
  unless ($found==1) {
  $BSmatrix{$key}.="\t".$lowval;
  } 
 }
 print "BS values of ",$bipartfile," were compiled.\n";

}

print "Generating output file in the folder Bipartition_Results.\n";
open OUTFILE, ">Compiled_Bipartition_Tables.txt";
print OUTFILE "\t\t\t\t";
foreach my $a (0..$#treefiles) {
    print OUTFILE $treefiles[$a], "\t";
}
print OUTFILE "\n";
foreach my $key (sort keys %bipart)  {
  my $bipartinv = $bipart{$key};
  my $ingroup = $biingroup{$key};
  my $outgroup = $bioutgroup{$key};
  my $BSvalues = $BSmatrix{$key};
 print OUTFILE $key, "\t", $bipartinv,"\t", $ingroup,"\t", $outgroup, $BSvalues, "\n";
}
close OUTFILE,
chdir "..";

}

#Determination of saturation based on plots of p vs patristic distances---------------------------------------------------------------------------------
sub correlation {

#Determination all taxa across all trees
mkdir "Correlation_Results";
chdir "Correlation_Results" or die "The folder Correlation_Results not found.\n";
mkdir "PD_matrices";
mkdir "P_matrices";
chdir "..";
open LOGFILE, ">Saturation_Files_used.log";
close LOGFILE;
open OUTFILE, ">Correlation_Results/Correlation_Slope_Summary.txt";
print OUTFILE "Treefiles\tSlope\tR2\n";
close OUTFILE;
my $treefiles = $_[0];
my $alignfiles = $_[1];
my @treefiles=();
my @alignmentfiles=();
my @treevalues=();
my @hetero=();
my @avePD=();
open FILES, $treefiles or die "${treefiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @treefiles, $fileline;
 } 
close FILES;
open FILES, $alignfiles or die "${alignfiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @alignmentfiles, $fileline;
 } 
close FILES;
unless ($#alignmentfiles == $#treefiles) {die "The two files providing the tree and alignmentfiles should have the same number of files in the same order.\n"}
foreach my $a (0..$#alignmentfiles) {
 my @PD = ();
 my @P = ();
 my %nodes=();
 my @taxanames=();
 my %PDmatrix=();
 my %Amatrix=();
 my %Pmatrix=();
 print "Working on: Treefile = ", $treefiles[$a], "\tAlignmentfile = ", $alignmentfiles[$a], "\t";
 my $amino = 0;
 open ALIGNMENT, $alignmentfiles[$a] or die "$alignmentfiles[$a] not found.\n";
 while (my $alignment = <ALIGNMENT>) {
  chomp $alignment;
  my @alignment = ();
  unless ($alignment =~ m/\d+\s\d+/) {
   @alignment = split /\s+/, $alignment;
   if ($#alignment > 1) {print "No white spaces in sequence of ", $alignment[0], " allowed.\n" and die}
   push @taxanames, $alignment[0];
   $Amatrix{$alignment[0]}=$alignment[1];
   if ($alignment[1] =~ /I|E|L|Q|F|P|X/) {
   $amino = 1;
   } 
  }
 }
 if ($amino == 1) {print "Aminoacid alignment!\n"} else {print "Nucleotide alignment!\n"};
 open LOGFILE, ">>Saturation_Files_used.log";
 if ($amino == 1) {print LOGFILE "Treefile: ", $treefiles[$a], "\tAlignmentfile: ", $alignmentfiles[$a], "\tAminoacid alignment!\n"
 } else {print LOGFILE "Treefile: ", $treefiles[$a], "\tAlignmentfile: ", $alignmentfiles[$a], "\tNucleotide alignment!\n"};
 close LOGFILE; 
 close ALIGNMENT;
 
#Build an array containing the tree
 open TREEFILE, $treefiles[$a] or die "${treefiles[$a]} not found.\n";
 my @tree = ();
  while (my $treeline = <TREEFILE>) {
   $treeline =~ s/;//g;
   $treeline =~ s/\)\d+\.\d+:/\):/g;
   $treeline =~ s/\)\d+:/\):/g;
   $treeline =~ s/,/:/g;
   $treeline =~ s/\(/\(:/g;
   $treeline =~ s/(\d)\)/\1:\)/g;
   @tree = split /:/, $treeline;   
  } 
 close TREEFILE;

#Determination of taxa in the specific tree, number of taxa and hash pairs of taxon and terminal branch length 
 my @treetaxa=();
 foreach my $a (0..$#tree) {
  if ($tree[$a] =~ /\w/ and $tree[$a] !~ /^\d+$/ and $tree[$a] !~ /^\d+\.\d+$/) {
   push @treetaxa, $tree[$a];
   my $bl = $a+1;
   $nodes{$tree[$a]}=$tree[$bl];
  }
 }

#Crosscheck if all taxa in the specific tree are in the taxa list for all trees and occur only once 
 foreach my $a (0..$#treetaxa) {
  my $found = 0;
  foreach my $b (0..$#taxanames) {
   if ($taxanames[$b] eq $treetaxa[$a]) {
    $found++;
   }
  }
  if ($found == 0) {print $treetaxa[$a], " not found in the alignment.\n" and die};
  if ($found > 1) {print $treetaxa[$a], " found ", $found, " times in the alignment.\n" and die};
 } 
 foreach my $a (0..$#taxanames) {
  my $found = 0;
  foreach my $b (0..$#treetaxa) {
   if ($taxanames[$a] eq $treetaxa[$b]) {
    $found++;
   }
  }
  if ($found == 0) {print $taxanames[$a], " not found in the tree.\n" and die};
  if ($found > 1) {print $taxanames[$a], " found ", $found, " times in the tree.\n" and die};
 } 
 
#Determination of all internal nodes in the tree and hash pairs of the node and internal branch length leading to that node 
 foreach my $b (0..$#tree) {
  my $z=0;
  my $y=0;
  my $printout = ();
  my $taxa = ();
   if ($tree[$b] eq "\(") {
    my $first = 0;
    foreach my $bc ($b..$#tree) {
     if ($tree[$bc] eq "\(") {
      $y++;
     }; 
     if ($tree[$bc] eq "\)") {
      $z++; 
     }; 
     if ($z == $y and $tree[$bc] eq "\)" and $first == 0) {
      my $bl = $bc+1;
      my $node=();
      foreach my $wo ($b..$bc) {
        $node .= $tree[$wo];
       $first ++;
      }
      $nodes{$node}=$tree[$bl];
     }
    }
  }
 }
 
 foreach my $c (0..$#taxanames) {
  my $taxadist=();
  my $PDlist=();
  foreach my $d (0..$#taxanames) {
   my $cfound = 0;
   my $dfound = 0;
    foreach my $e (0..$#treetaxa) {
     if ($taxanames[$c] eq $treetaxa[$e]) {
      $cfound++;
     }
    }
    foreach my $f (0..$#treetaxa) {
     if ($taxanames[$d] eq $treetaxa[$f]) {
      $dfound++;
     }
    }
  my $patdist=();
   unless ($cfound == 0 or $dfound == 0) {
    while ((my $key, my $value) = each %nodes) {
     if ($key =~ /^${taxanames[$c]}$/) {
      unless ($key =~ /^${taxanames[$d]}$/) {
       $patdist+=$value;
      }
     }
     if ($key =~ /${taxanames[$c]}\d/) {
      unless ($key =~ /${taxanames[$d]}\d/) {
       $patdist+=$value;
      }
     }
     if ($key =~ /^${taxanames[$d]}$/) {
      unless ($key =~ /^${taxanames[$c]}$/) {
       $patdist+=$value;
      }
     }
     if ($key =~ /${taxanames[$d]}\d/) {
      unless ($key =~ /${taxanames[$c]}\d/) {
       $patdist+=$value;
      }
     }
    }
   }
   $PDlist .= "\t${patdist}";
  }
  $PDmatrix{$taxanames[$c]}=$PDlist;
 }

 foreach my $a (0..$#taxanames) {
   my @sequenceA = ();
   my $Plist=();
   my $sequenceA = $Amatrix{$taxanames[$a]};
   @sequenceA = split //, $sequenceA;
     foreach my $b (0..$#taxanames) {
     my @sequenceB = ();
     my $pdist = ();
     my $sequenceB = $Amatrix{$taxanames[$b]};
         unless ($taxanames[$b] =~ /^${taxanames[$a]}$/) {
         @sequenceB = split //, $sequenceB;
         unless ($#sequenceB == $#sequenceA) {print "Unequal number of positions (", $#sequenceA, " and ", $#sequenceB ,") in sequences of ", $taxanames[$a], " and ", $taxanames[$b], ", respectively.\n" and die}
		 my $positions = ($#sequenceB+1);
		 my $diff = 0;
		 my $undef = 0;
		 foreach my $c (0..$#sequenceB) {
		  if ($amino == 1) {
		   unless ($sequenceB[$c] =~ /^${sequenceA[$c]}$/i|$sequenceB[$c] =~ /^X$|^-$|^\?$/i|$sequenceA[$c] =~ /^X$|^-$|^\?$/i) {
		    $diff++; 
		   }
		   if ($sequenceA[$c] =~ /^X$|^-$|^\?$/i and $sequenceB[$c] =~ /^X$|^-$|^\?$/i) {
		    $undef++; 
		   }
		  } else {
		   unless ($sequenceB[$c] =~ /^${sequenceA[$c]}$/i|$sequenceB[$c] =~ /^Y$|^R$|^W$|^S$|^K$|^M$|^B$|^D$|^V$|^H$|^N$|^-$|^\?$/i|$sequenceA[$c] =~ /^Y$|^R$|^W$|^S$|^K$|^M$|^B$|^D$|^V$|^H$|^N$|^-$|^\?$/i) {
		    $diff++;
		   } 
		   if ($sequenceB[$c] =~ /^Y$|^R$|^W$|^S$|^K$|^M$|^B$|^D$|^V$|^H$|^N$|^-$|^\?$/i and $sequenceA[$c] =~ /^Y$|^R$|^W$|^S$|^K$|^M$|^B$|^D$|^V$|^H$|^N$|^-$|^\?$/i) {
		    $undef++; 
		   }		  
		  }
		 }
		 $pdist = ($diff/($positions-$undef));
        }
     $Plist .= "\t${pdist}";
     }
   $Pmatrix{$taxanames[$a]}=$Plist;
 }
#Printout of pairwise patristic distance matrix and generate arrays for correlation
 open OUTFILE, ">Correlation_Results/PD_matrices/${treefiles[$a]}_PDmatrix.txt";
 print OUTFILE "\t";
 foreach my $a (0..$#taxanames) {
    print OUTFILE $taxanames[$a], "\t";
 }
 print OUTFILE "\n";
 foreach my $a (0..$#taxanames) {
  while ((my $key, my $value) = each %PDmatrix) {
   if ($key =~ /^${taxanames[$a]}$/) {
    print OUTFILE $key, $value, "\n";
    my @PDvalues = ();
    @PDvalues = split /\t/, $value;
    foreach my $b (0..$#PDvalues) {push @PD, $PDvalues[$b]}
   }
  }
 }
 close OUTFILE;
 open OUTFILE, ">Correlation_Results/P_matrices/${alignmentfiles[$a]}_Pmatrix.txt";
 print OUTFILE "\t";
 foreach my $a (0..$#taxanames) {
    print OUTFILE $taxanames[$a], "\t";
 }
 print OUTFILE "\n";
 foreach my $a (0..$#taxanames) {
  while ((my $key, my $value) = each %Pmatrix) {
   if ($key =~ /^${taxanames[$a]}$/) {
    print OUTFILE $key, $value, "\n";
    my @Pvalues = ();
    @Pvalues = split /\t/, $value;
    foreach my $b (0..$#Pvalues) {push @P, $Pvalues[$b]}
   }
  }
 }
 close OUTFILE;
 
 my $lineFit = Statistics::LineFit->new();
 $lineFit->setData (\@PD, \@P) or die "Invalid data";
 (my $intercept, my $slope) = $lineFit->coefficients();
 defined $intercept or die "Can't fit line if x values are all equal";
 my $rSquared = $lineFit->rSquared();
 open OUTFILE, ">>Correlation_Results/Correlation_Slope_Summary.txt";
 print OUTFILE $treefiles[$a], "\t", $slope, "\t", $rSquared, "\n";
 close OUTFILE;
}


}

#Generation of all possible combinations of partitions and of permutated datasets---------------------------------------------------------------------------------
sub combination {

my $alignment = $_[0];
my $partitionfile = $_[1];
my $maxx = $_[2];
my $minx = $_[3];
my $nper = $_[4];
my $per = $_[5];

my @partitions =();
my $mask=();
my %startstop = ();
my %models = ();
my %permuts = ();

mkdir "Combinations";
#Collecting partition information
open PARTFILE, $partitionfile or die "${partitionfile} not found.\n";
 while (my $partline = <PARTFILE>) {
  chomp $partline;
  my $count = ($partline =~ tr/=//);
  if ($count <= 0) {die "A = is missing in $partitionfile in line: $partline.\n"}; 
  if ($count > 1) {die "Too many = in $partitionfile in line: $partline.\n"}; 
  $partline =~ /^(\w+,\s+)/;
  my $model = $1;
  $partline =~ s/^\w+,\s+//;
  $partline =~ s/\s+//g;
  my @partline = split /=/, $partline;
  push (@partitions, $partline[0]);
  $startstop{$partline[0]}=$partline[1];
  $models{$partline[0]}=$model;
 }
close PARTFILE;

#Collecting alignment information
my %Amatrix=();
my @taxanames=();
my $taxacount=0;
my @checks=();
open ALIGNFILE, $alignment or die "${alignment} not found.\n";
 my $first = 0;
 while (my $alignline = <ALIGNFILE>) {
  chomp $alignline;
  my @alignment = ();
  if ($alignline =~ m/^\d+\s+\d+/ and $first == 0) {
   @checks = split /\s+/, $alignline;
   $first++;
  }
  unless ($alignline =~ m/\d+\s+\d+/) {
   @alignment = split /\s+/, $alignline;
   if ($#alignment > 1) {die "No white spaces in sequence of ", $alignment[0], " allowed.\n"}
   push (@taxanames, $alignment[0]);
   $taxacount++;
   $Amatrix{$alignment[0]}=$alignment[1];
  }
 }
close ALIGNFILE;

#Check if positions and taxa numbers are correct
unless ($taxacount == $checks[0]) {die "Number of taxa in alignment $taxacount is unequal to number of taxa provided in first line $checks[0].\n"}
my $poscheck1 = ();
my $poscheck2 = ();
my $equal = 0;
my $length = ();
foreach my $a (0..$#taxanames) {
 $length = length ($Amatrix{$taxanames[$a]});
 if ($length < $poscheck1 | $poscheck1 == 0) {
  $poscheck1 = $length;
  $equal++;
 }
 if ($length > $poscheck1) {
  $equal++;
 }
}
unless ($poscheck1 == $checks[1]) {die "Number of positions in alignment $poscheck1 is unequal to number of positions provided in first line $checks[1].\n"}
if ($equal > 1) {die "The taxa in the alignment do not have the same number of positions.\n"}

#Permutations
if ($per == 1) {
 open FILE, ">Combinations/Permutations_used.txt";
 close FILE;
 foreach my $a (1..$nper) {
  chdir "Combinations" or die "The folder Combinations not found.\n";
  mkdir "Permutations_$a";
  chdir "..";
  my @RANDS = (sort { .5 > rand() ? 1 : -1 } 1 .. $checks[1]);
  foreach my $b (0..$#RANDS) {
   $permuts{$a} .= "$RANDS[$b] ";
  }
 open FILE, ">>Combinations/Permutations_used.txt";
  print FILE "For permutation #", $a, " the positions were permutated as follows:\n",$permuts{$a}, "\n";
 close FILE;
 }
}

#Generation of all possible combinations
my $many = ($#partitions+1);
my @MASK;
while ( $mask = createmask() ) {
  my @include=split /\s+/, $mask;
  my %start=();
  my %stop=();
  my %begin=();#new
  my %end=();#new
  my $name;
  my @start = ();
  my @stop = ();
  my $parpos = 0;
  foreach my $a (0..$#include) {
   $name .= $include[$a];
   if ($startstop{$include[$a]} =~ /\\\d+/) {
    if ($startstop{$include[$a]} =~ /,/) {
     $start{$include[$a]} = ($#start+1);
     my @several = split /,/, $startstop{$include[$a]};
     foreach my $d (0..$#several) {
      if ($several[$d] =~ /\\\d+/) {
       $several[$d] =~ /\\(\d+)/;
       my $times = $1;
       my $startpos = $times;
       my $line = $several[$d];
       $line =~ s/\\\d+//;
       my @startstop = split /-/, $line;
       foreach my $f ($startstop[0]..$startstop[1]) {
        unless ($startpos % $times) {
         push (@start, $f);
         push (@stop, $f);      
        }
        $startpos++;
       }     
      } else {
       my @startstop = split /-/, $several[$d];
       push (@start, $startstop[0]);
       push (@stop, $startstop[1]);
      }
     }
     $stop{$include[$a]} = $#stop;
    } else {
     $startstop{$include[$a]} =~ /\\(\d+)/;
     my $times = $1;
     my $startpos = $times;
     my $line = $startstop{$include[$a]};
     $line =~ s/\\\d+//;
     my @startstop = split /-/, $line;
     $start{$include[$a]} = ($#start+1);
     foreach my $f ($startstop[0]..$startstop[1]) {
      unless ($startpos % $times) {
       push (@start, $f);
       push (@stop, $f);      
      }
      $startpos++;
     }
     $stop{$include[$a]} = $#stop;
    }
   } else {
    if ($startstop{$include[$a]} =~ /,/) {
     my @several = split /,/, $startstop{$include[$a]};
     $start{$include[$a]} = ($#start+1);
     foreach my $d (0..$#several) {
      my @startstop = split /-/, $several[$d];
      push (@start, $startstop[0]);
      push (@stop, $startstop[1]);
     }
     $stop{$include[$a]} = $#stop;
    } else {
     my @startstop = split /-/, $startstop{$include[$a]};
     push (@start, $startstop[0]);
     push (@stop, $startstop[1]);
     $start{$include[$a]} = $#start;
     $stop{$include[$a]} = $#stop;
    }
   }
  }
  $name .= ".phy";
  foreach my $a (0..$#start) {
   $parpos += ($stop[$a]-($start[$a]-1));
  }
  print "Working on ", $name, " with ", $taxacount, " taxa and ", $parpos," positions.\n";
  open OUT, ">Combinations/$name";
  print OUT $taxacount, " ", $parpos,"\n";
  foreach my $b (0..$#taxanames) {
   my $count = 0;
   print OUT $taxanames[$b], "\t";
   my @sequence = split //, $Amatrix{$taxanames[$b]};
   foreach my $a (0..$#start) {
    foreach my $e (0..$#include) {
     if ($a == $start{$include[$e]}) {$begin{$include[$e]}=($count+1)}
    }
    my $begin = ($start[$a]-1);
    my $end = ($stop[$a]-1);
    foreach my $c ($begin..$end) {
     print OUT $sequence[$c];
     $count++;
    }
    foreach my $e (0..$#include) {
     if ($a == $stop{$include[$e]}) {$end{$include[$e]}=$count}
    }
   }
   print OUT "\n";
  }
  close OUT;
  if ($per == 1) {
   foreach my $a (1..$nper) {
    my @permuts = split /\s/, $permuts{$a};
    my $perout = $name;
    $perout =~ s/\.phy/_per$a\.phy/;
    open PEROUT, ">Combinations/Permutations_$a/$perout";
     print PEROUT $taxacount, " ", $parpos,"\n";    
     foreach my $b (0..$#taxanames) {
      print PEROUT $taxanames[$b], "\t";
      my @sequence = split //, $Amatrix{$taxanames[$b]};
      foreach my $a (0..$#start) {
       my $begin = ($start[$a]-1);
       my $end = ($stop[$a]-1);
       foreach my $c ($begin..$end) {
        my $d = ($permuts[$c]-1);
        print PEROUT $sequence[$d];
       }
      }
      print PEROUT "\n";
     }
    close PEROUT;
   }
  }
  my $parout = $name;
  $parout =~ s/\.phy/_partitions\.txt/;
  open PAROUT, ">Combinations/$parout";
   foreach my $e (0..$#include) {
    print PAROUT $models{$include[$e]}, $include[$e], " = ", $begin{$include[$e]}, "-", $end{$include[$e]}, "\n"; 
   }
  close PAROUT;  
}



#Sub all possible combinations----------------------------------------------------------------------------
sub createmask {

  while (1) {
    my $u = 1; my $e;
    for my $i (0..$#MASK) {

      $e = ( $MASK[$i] xor $u );
      $u = ( $MASK[$i] and $u );

      $MASK[$i] = $e;

      last unless $u;
    }
    $MASK[$#MASK+1]  = 1 if $u;
    return '' if $#MASK  >= $many;

    my $sum=0; my $mask='';
    for my $i (0..$many-1) {
      $mask .= "$partitions[$i] " if $MASK[$i];
      $sum ++  if $MASK[$i];
    }

    return $mask if $sum <= $maxx and $sum >= $minx
  }
}

}

#Calculation of PABA results as well as WSRT and permutation tests-----------------------------------------------------------------------------
sub PABA {

my $tablefiles = $_[0];
my $partitionfile = $_[1];
my $min = $_[2]; 
my $max = $_[3]; 
my $wsrtc = $_[4]; 
my $bipartsets = $_[5];
my $permutc = $_[6]; 
my $original = $_[7];
my @tablefiles=();
my @partitions=();
my @a005 = ('0','0','0','0','0','0', '0', '2', '3', '5', '8', '10', '13', '17', '21', '25', '29', '34', '40', '46');
my @a001 = ('0','0','0','0','0','0', '0', '0', '0', '1', '3', '5', '7', '9', '12', '15', '19', '23', '27', '32');
my @a0001 = ('0','0','0','0','0','0', '0', '0', '0', '0', '0', '0', '1', '2', '4', '6', '8', '11', '14', '18');

mkdir "PABA_calculations";
open FILES, $tablefiles or die "${tablefiles} not found.\n";
 while (my $line =<FILES>) {
  chomp $line;
  $line =~ s/\s//g;
  push @tablefiles, $line;
 } 
close FILES;

open PARTFILE, $partitionfile or die "${partitionfile} not found.\n";
 my $linecount = 0;
 while (my $partline = <PARTFILE>) {
  $linecount++;
  chomp $partline;
  my $count = ($partline =~ tr/=//);
  if ($count <= 0) {die "A = is missing in $partitionfile in line $linecount: $partline. Or the line does not contain any partition information at all.\n"}; 
  if ($count > 1) {die "Too many = in $partitionfile in line $linecount: $partline.\n"}; 
  $partline =~ s/^\w+,\s+//;
  $partline =~ s/\s+//g;
  my @partline = split /=/, $partline;
  push (@partitions, $partline[0]);
 }
close PARTFILE;
my $maxpart = ($#partitions+1);
print "The partition file contains ",$maxpart," partition definitions.\n";

my %PABA=();
my %partdiff=();
my %WSRT=();
my %addinfo=();
my @oribiparts=();
my $orintabs=();
my %inverse=();

if ($permutc==1) {
 my $first = 1;
 open TABLEFILE, $original or die "${original} not found.\n";
  while (my $line =<TABLEFILE>) {
   $orintabs = 0;
   chomp $line;
   unless ($first==0) {
    $first=0;
   } else {
    my @results = split /\t/, $line;
    push (@oribiparts, $results[0]);
    foreach my $b (0..$#results) {
     unless ($results[$b] =~ m/^\d+$|^\d+\.\d+$/ and $b != 0) {
      unless ($b == 0) {$orintabs++}
      if ($b == 1) {$inverse{$results[0]} = $results[$b]}
     } 
    }
   }
  }
 close TABLEFILE;
}

foreach my $tablefile (@tablefiles) {
 print "Working on: ", $tablefile, "\n";
 chdir "PABA_calculations" or die "The folder PABA_calculations not found.\n";
 mkdir "Results_$tablefile";
 chdir "..";
 my $first = 1;
 my @files=();
 my @biparts=();
 my %bootstrap=();
 my $ntabs=();
 
 #Reading in the matrix with bootstrap values
 open TABLEFILE, $tablefile or die "${tablefile} not found.\n";
  while (my $line =<TABLEFILE>) {
   $ntabs = 0;
   chomp $line;
   unless ($first==0) {
    @files = split /\t/, $line;
    $first=0;
   } else {
    my @results = split /\t/, $line;
    push (@biparts, $results[0]);
    foreach my $b (0..$#results) {
     if ($results[$b] =~ m/^\d+$|^\d+\.\d+$/ and $b != 0) {
      $bootstrap{$results[0]}{$files[$b]} = $results[$b];
     } else {
      unless ($b == 0) {$addinfo{$tablefile}{$results[0]} .= "\t$results[$b]";$ntabs++}
     } 
    }
   }
  }
 close TABLEFILE;

#Calculation of the support difference for each partition added to possible all other combinations of partitions in the file without this partition
 foreach my $target (@partitions) {
  foreach my $upvalue (@files) {
   if ($upvalue =~ m/$target/) {
    my $count = 0;
    my @otherpart=();
    foreach my $others (@partitions) {
     unless ($others eq $target) {
      if ($upvalue =~ m/$others/) {
       $count++;
       push (@otherpart, $others);
      }
     }
    }
   foreach my $lowvalue (@files) {
     my $match = 0;
     my $total = 0;
     foreach my $c (@partitions) {
      if ($lowvalue =~ m/$c/) {
       $total++;
      }
     }
     foreach my $d (@otherpart) {
      if ($lowvalue =~ m/$d/) {unless ($lowvalue =~ m/$target/){$match++}}
     }
     if ($match == $count and $match == $total and $count > 0) {
      $count++;
      foreach my $bipart (@biparts) {
      my $diff = ();
       if ($min =~ m/N|n/ and $max =~ m/N|n/) {
        $diff = ($bootstrap{$bipart}{$upvalue}-$bootstrap{$bipart}{$lowvalue})
       } elsif ($min =~ m/N|n/) {
       if (($bootstrap{$bipart}{$upvalue}==$max and $bootstrap{$bipart}{$lowvalue}==$max)) {
        $diff = ();
       } else {
         $diff = ($bootstrap{$bipart}{$upvalue}-$bootstrap{$bipart}{$lowvalue});     
       }       
       } elsif ($max =~ m/N|n/) {
       if (($bootstrap{$bipart}{$upvalue}==$min and $bootstrap{$bipart}{$lowvalue}==$min)) {
        $diff = ();
       } else {
         $diff = ($bootstrap{$bipart}{$upvalue}-$bootstrap{$bipart}{$lowvalue});     
       }       
       } else {
        if (($bootstrap{$bipart}{$upvalue}==$max and $bootstrap{$bipart}{$lowvalue}==$max)) {
         $diff = ();
        } elsif ($bootstrap{$bipart}{$upvalue}==$min and $bootstrap{$bipart}{$lowvalue}==$min) {
         $diff = ();      
        } else {
          $diff = ($bootstrap{$bipart}{$upvalue}-$bootstrap{$bipart}{$lowvalue});     
        }
      } 
      $partdiff{$tablefile}{$target}{$count}{$bipart}{$lowvalue} = $diff; # Hash Bootstrap file - Partition - Added as - Bipartition - File added to
      }
     }
    }
   }
  }
 }

#Average for each partition depeding on the position added
 foreach my $part (@partitions) {
  foreach my $added (2..$maxpart) {
   foreach my $bipart (@biparts) {
    my $divisor = ();
    my $sum = ();
    my $average = ();
    for my $lowvalue (keys %{$partdiff{$tablefile}{$part}{$added}{$bipart}}) {
     if (defined $partdiff{$tablefile}{$part}{$added}{$bipart}{$lowvalue}) {
      $divisor++;
      $sum += $partdiff{$tablefile}{$part}{$added}{$bipart}{$lowvalue};
     }
    }
    if (defined $divisor) {
     $average = ($sum/$divisor);
    }
    if ($permutc==0) {$PABA{$tablefile}{$part}{$added}{$bipart} = $average} # Hash Bootstrap file - Partition - Added as - Bipartition
    if ($permutc==1) {
     if ($tablefile eq $original) {$PABA{$tablefile}{$part}{$added}{$bipart} = $average}
     else {
      my $found = 0;
      foreach my $oribipart (@oribiparts) {
       if ($bipart eq $oribipart) {$found = 1; $PABA{$tablefile}{$part}{$added}{$bipart} = $average}
       if ($bipart eq $inverse{$oribipart}) {$found = 1; $PABA{$tablefile}{$part}{$added}{$oribipart} = $average}
      }
      if ($found == 0) {$PABA{$tablefile}{$part}{$added}{$bipart} = $average}
     }
    }
   }
  }
 }
 
#Print PABA results
 open OUT, ">PABA_calculations/Results_$tablefile/PABA_results_woSignificance_$tablefile.txt" ;
 foreach (1..$ntabs) {print OUT "\t"}
 foreach my $part (@partitions) {
  foreach my $added (2..$maxpart) {
   print OUT "\t",$part,"_as_",$added;
  }
 }
 print OUT "\n";
 foreach my $bipart (@biparts) {
  print OUT $bipart,$addinfo{$tablefile}{$bipart};
  foreach my $part (@partitions) {
   foreach my $added (2..$maxpart) {
    print OUT "\t", $PABA{$tablefile}{$part}{$added}{$bipart};
   }
  }
  print OUT "\n";
 }
 close OUT;

#WSRT calculations
 if ($wsrtc == 1) {
  my @bipartsets=();
  open BIPARTFILE, $bipartsets or die "${bipartsets} not found.\n";
   while (my $bipartline = <BIPARTFILE>) {
    chomp $bipartline;
    $bipartline =~ s/\s//g;
    push (@bipartsets, $bipartline);
   }
  close BIPARTFILE;

 foreach my $bipartfile (@bipartsets) {
  my @spbiparts = ();
  open BIPARTFILE, $bipartfile or die "${bipartfile} not found.\n";
   while (my $bipartline = <BIPARTFILE>) {
    chomp $bipartline;
    $bipartline =~ s/\s//g;
    push (@spbiparts, $bipartline);
   }
  close BIPARTFILE;

 
  foreach my $part (@partitions) {
   foreach my $added (2..$maxpart) {
    my @negative = ();
    my @positive = ();
    my $zeros = 0;
    my $ncount = 0;
    my $z_value = "not defined";
    my $p_value = ();
    foreach my $bipart (@spbiparts) {
     if (defined $PABA{$tablefile}{$part}{$added}{$bipart}) {
      if ($PABA{$tablefile}{$part}{$added}{$bipart} < 0) {push (@negative, ($PABA{$tablefile}{$part}{$added}{$bipart}*-1)), $ncount++}
      if ($PABA{$tablefile}{$part}{$added}{$bipart} > 0) {push (@positive, $PABA{$tablefile}{$part}{$added}{$bipart}), $ncount++}
      if ($PABA{$tablefile}{$part}{$added}{$bipart} == 0) {
       $zeros++;
       if ($zeros % 2) {push (@positive, $PABA{$tablefile}{$part}{$added}{$bipart})} else {push (@negative, $PABA{$tablefile}{$part}{$added}{$bipart})}
       $ncount++;
      }
     }
    }
    my $onlyzero = 0;
    my @negzero = sort {$a<=>$b} (@negative);
    my @poszero = sort {$a<=>$b} (@positive);
   if ($negzero[$#negzero] <= 0) {$onlyzero = 1; $WSRT{$tablefile}{$part}{$added}{$bipartfile} = ("+ >= 0.05")}
   elsif ($poszero[$#poszero] <= 0) {$onlyzero = 1; $WSRT{$tablefile}{$part}{$added}{$bipartfile} = ("- >= 0.05")}
   if ($onlyzero == 0) {
    my $wilcox_test = Statistics::Test::WilcoxonRankSum->new();
    $wilcox_test->load_data(\@negative, \@positive);
    my $negrank = $wilcox_test->rank_sum_for(1);
    my $posrank = $wilcox_test->rank_sum_for(2);
    my $poslarge = 1;
    if ($ncount < 20) {
     if ($negrank < $ posrank) {
      if ($negrank < $a0001[$ncount]) {$p_value = "< 0.001"}
      elsif ($negrank < $a001[$ncount]) {$p_value = "< 0.01"}
      elsif ($negrank < $a005[$ncount]) {$p_value = "< 0.05"}
      else {$p_value = ">= 0.05"}
     } else {
      if ($posrank < $a0001[$ncount]) {$p_value = "< 0.001"}
      elsif ($posrank < $a001[$ncount]) {$p_value = "< 0.01"}
      elsif ($posrank < $a005[$ncount]) {$p_value = "< 0.05"}
      else {$p_value = ">= 0.05"};
      $poslarge = 0;
     }
    } elsif ($ncount < 61) {
     if ($negrank < $ posrank) {$z_value = ((abs($negrank-($ncount*($ncount+1)/4))-.5)/sqrt($ncount*($ncount+1)*((2*$ncount)+1)/24))} else {$z_value = ((abs($posrank-($ncount*($ncount+1)/4))-.5)/sqrt($ncount*($ncount+1)*((2*$ncount)+1)/24)); $poslarge = 0}
    } else {
     if ($negrank < $ posrank) {$z_value = ((abs($negrank-($ncount*($ncount+1)/4)))/sqrt($ncount*($ncount+1)*((2*$ncount)+1)/24))} else {$z_value = ((abs($posrank-($ncount*($ncount+1)/4)))/sqrt($ncount*($ncount+1)*((2*$ncount)+1)/24)); $poslarge = 0}    
    }
    if ($ncount > 19) {
     my $zed = Statistics::Zed->new();
     $p_value = $zed->z2p(value => $z_value, tails => 2); 
    }
    if ($poslarge == 1) {$WSRT{$tablefile}{$part}{$added}{$bipartfile} = ("+ ".$p_value)} else {$WSRT{$tablefile}{$part}{$added}{$bipartfile} = ("- ".$p_value)}
   }
   }  
  }
  }

#Print WSRT results
 open OUT, ">PABA_calculations/Results_$tablefile/WSRT_results_$tablefile.txt" ;
 foreach my $part (@partitions) {
  foreach my $added (2..$maxpart) {
   print OUT "\t",$part,"_as_",$added;
  }
 }
 print OUT "\n";
 foreach my $bipartfile (@bipartsets) { 
  print OUT $bipartfile,$addinfo{$tablefile}{$bipartfile};
  foreach my $part (@partitions) {
   foreach my $added (2..$maxpart) {
    print OUT "\t", $WSRT{$tablefile}{$part}{$added}{$bipartfile};
   }
  }
  print OUT "\n";
 }
 close OUT;
 }
 
}

#Calculation of significance of PABA results
if ($permutc == 1) {
 my %sign = ();
 my $originalfound = 0;
 foreach my $tablefile (@tablefiles) {
  if ($tablefile eq $original) {$originalfound++}
 }
 if ($originalfound == 0) {die "The name for the file ",$original," providing the results from the original dataset has not been found in the list of results provided by ",$tablefiles,".\n"}
 if ($originalfound > 1) {die "The name for the file ",$original," providing the results from the original dataset has been found more than once in the list of results provided by ",$tablefiles,".\n"}

 foreach my $part (@partitions) {
  foreach my $added (2..$maxpart) {
   foreach my $bipart (@oribiparts) {
   my $larger = 0;
   my $totalper = 0;
    foreach my $tablefile (@tablefiles) {
     unless ($tablefile eq $original) {
      if (defined $PABA{$original}{$part}{$added}{$bipart}) {if (defined $PABA{$tablefile}{$part}{$added}{$bipart}) {
       $totalper++;       
       if ($PABA{$original}{$part}{$added}{$bipart} <= 0) {if ($PABA{$tablefile}{$part}{$added}{$bipart} < $PABA{$original}{$part}{$added}{$bipart}) {$larger++}}
       if ($PABA{$original}{$part}{$added}{$bipart} > 0) {if ($PABA{$tablefile}{$part}{$added}{$bipart} > $PABA{$original}{$part}{$added}{$bipart}) {$larger++}}
      }}
     }
    }
   if ($totalper == 0) {$sign{$part}{$added}{$bipart} = ($PABA{$original}{$part}{$added}{$bipart}."\tN/A\t".$totalper)}
   if ($totalper > 0) {my $p_value=$larger/$totalper; $sign{$part}{$added}{$bipart} = ($PABA{$original}{$part}{$added}{$bipart}."\t".$p_value."\t".$totalper)}    
   }
  }
 }
#Print Permutation results
 open OUT, ">PABA_calculations/Results_$original/PABA_results_Significance_$original.txt" ;
 foreach (1..$orintabs) {print OUT "\t"}
 foreach my $part (@partitions) {
  foreach my $added (2..$maxpart) {
   print OUT "\t",$part,"_as_",$added,"\t\t";
  }
 }
 print OUT "\n";
 foreach (1..$orintabs) {print OUT "\t"}
 foreach my $part (@partitions) {
  foreach my $added (2..$maxpart) {
   print OUT "\tresult\tp_value\tN";
  }
 }
 print OUT "\n";
 foreach my $bipart (@oribiparts) {
  print OUT $bipart,$addinfo{$original}{$bipart};
  foreach my $part (@partitions) {
   foreach my $added (2..$maxpart) {
    print OUT "\t", $sign{$part}{$added}{$bipart};
   }
  }
  print OUT "\n";
 }
 close OUT;
}

}

#BLAST searches of potential paralog sequences-----------------------------------------------------------------------------
sub blast {

my $paralogfile = $_[0];
my $treefiles = $_[1];
my $alignfiles = $_[2];
my $database1 = $_[3];
my $database2 = $_[4];
my $ediff = $_[5];
my $lowparathres = $_[6];
my $upparathres = $_[7];
my $evalue = $_[8];
my $trespexpath = $_[9]; 
my $returnpath = $_[10]; 

chdir "$trespexpath" or die "The folder ${trespexpath} not found.\n"; 
 chdir "blast/bin" or die "The folder ${trespexpath}/blast/bin not found.\n";
  open OUTFILE, ">BLAST_log.txt";close OUTFILE;
chdir "$returnpath" or die "The folder ${returnpath} not found.\n";
mkdir "blast_search";
chdir "blast_search" or die "The folder blast_search not found.\n";
mkdir "blast_results";
mkdir "blast_comparisons";
 chdir "blast_comparisons" or die "The folder ${returnpath}/blast_search/blast_comparisons not found.\n";
 mkdir "No_hits";
  chdir "No_hits" or die "The folder ${returnpath}/blast_search/blast_comparisons/No_hits not found.\n";
  open OUTFILE, ">No_hits_${paralogfile}";close OUTFILE;
  chdir "..";
 mkdir "Certain_paralogy";
  chdir "Certain_paralogy" or die "The folder ${returnpath}/blast_search/blast_comparisons/Certain_paralogy not found.\n";
  open OUTFILE, ">Certain_${paralogfile}";close OUTFILE;
  chdir "..";
 mkdir "Uncertain_paralogy";
  chdir "Uncertain_paralogy" or die "The folder ${returnpath}/blast_search/blast_comparisons/Uncertain_paralogy not found.\n";
  open OUTFILE, ">Uncertain_${paralogfile}";close OUTFILE;
  chdir "..";
 mkdir "No_paralogy";
  chdir "No_paralogy" or die "The folder ${returnpath}/blast_search/blast_comparisons/No_paralogy not found.\n";
  open OUTFILE, ">No_${paralogfile}";close OUTFILE;
  chdir "..";
 chdir "..";
chdir "..";

my %dbgiven = (
 "Anolis_carolinensis" => "Databases/Anolis_carolinensis/anoca_3173_21_prot",
 "Apis_mellifera" => "Databases/Apis_mellifera/apime_353_prot",
 "Bombyx_mori" => "Databases/Bombyx_mori/bommo_1242_prot",
 "Bos_taurus" => "Databases/Bos_taurus/bosta_355_prot",
 "Branchiostoma_floridae" => "Databases/Branchiostoma_floridae/brafl_2491_prot",
 "Caenorhabditis_elegans" => "Databases/Caenorhabditis_elegans/caeel_3_prot",
 "Capitella_teleta" => "Databases/Capitella_teleta/capsp_2853_prot",
 "Ciona_intestinalis" => "Databases/Ciona_intestinalis/cioin_5_prot",
 "Danio_rerio" => "Databases/Danio_rerio/danre_358_prot",
 "Daphnia_pulex" => "Databases/Daphnia_pulex/dappu_1391_prot",
 "Drosophila_melanogaster" => "Databases/Drosophila_melanogaster/drome_4_prot",
 "Gallus_gallus" => "Databases/Gallus_gallus/galga_359_2_prot",
 "Helobdella_robusta" => "Databases/Helobdella_robusta/helro_2711_prot",
 "Homo_sapiens" => "Databases/Homo_sapiens/homsa_8_41_prot",
 "Lottia_gigantia" => "Databases/Lottia_gigantia/lotgi_2713_prot",
 "Monodelphis_domestica" => "Databases/Monodelphis_domestica/mondo_1759_prot",
 "Mus_musculus" => "Databases/Mus_musculus/musmu_1_prot",
 "Ornithorhynchus_anatinus" => "Databases/Ornithorhynchus_anatinus/ornan_1771_prot",
 "Pediculus_humanus" => "Databases/Pediculus_humanus/pedhu_1258_prot",
 "Schmidtea_mediterranea" => "Databases/Schmidtea_mediterranea/schma_370_prot",
 "Taeniopygia_guttata" => "Databases/Taeniopygia guttata/taegu_310_1_prot",
 "Tribolium_castaneum" => "Databases/Tribolium_castaneum/trica_1449_prot",
 "Xenopus_tropicalis" => "Databases/Xenopus_tropicalis/xentr_374_prot",
);
my @treefiles=();
my @alignmentfiles=();
my @paralogcases=();
#Read in files and paralogy cases
open FILES, $treefiles or die "${treefiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @treefiles, $fileline;
 } 
close FILES;
open FILES, $alignfiles or die "${alignfiles} not found.\n";
 while (my $fileline =<FILES>) {
  chomp $fileline;
  $fileline =~ s/\s//g;
  push @alignmentfiles, $fileline;
 } 
close FILES;
unless ($#alignmentfiles == $#treefiles) {die "The two files providing the tree and alignmentfiles should have the same number of files.\n"}
open FILES, $paralogfile  or die "${paralogfile} not found.\n";
 while (my $fileline =<FILES>) {
  push @paralogcases, $fileline;
 } 
close FILES;
#Check if alignment, treefile and paralogy case fit to each other and determination if the number of taxa provided for the paralogy case is smaller or the number of taxa not provided
my $casenumber=0;
my $dbdone = 0;
my $db1 = ();
my $db2 = ();
my $aminodb1 = 0;
my $aminodb2 = 0;
foreach my $paralogline (@paralogcases) {
 $casenumber++;
 my @paralogscreen = ();
 my $aminoali = 0;
 @paralogscreen = split /\s+/, $paralogline;
 my @treetaxa=();
 my @alitaxa = ();
 my %Amatrix = ();
 my $alignfile = ();
 foreach my $a (0..$#treefiles) {
  if ($treefiles[$a] eq $paralogscreen[0]) {
   open TREEFILE, $treefiles[$a] or die "$treefiles[$a] not found.\n";
    while (my $treeline = <TREEFILE>) {
     chomp $treeline;
     $treeline =~ s/,/:/g;
     $treeline =~ s/\(//g;
     $treeline =~ s/\)//g;
     $treeline =~ s/\d+\.\d+//g;
     $treeline =~ s/;//g;
     $treeline =~ s/\:+/:/g;
     @treetaxa = split /:/, $treeline;   
    } 
   close TREEFILE;
   print "\nWorking on: Treefile = ", $treefiles[$a], "\tAlignmentfile = ", $alignmentfiles[$a], "\t";
   open ALIGNMENT, $alignmentfiles[$a] or die "$alignmentfiles[$a] not found.\n";
    while (my $alignment = <ALIGNMENT>) {
     chomp $alignment;
     my @alignment = ();
     unless ($alignment =~ m/\d+\s\d+/) {
      @alignment = split /\s+/, $alignment;
      if ($#alignment > 1) {print "No white spaces in sequence of ", $alignment[0], " allowed.\n" and die}
      push @alitaxa, $alignment[0];
      $Amatrix{$alignment[0]}=$alignment[1];
      if ($alignment[1] =~ /I|E|L|Q|F|P|X/) {
       $aminoali = 1;
      } 
     }
    }
    if ($aminoali == 1) {print "For the query data: Aminoacids!\n"} else {print "For the query data: Nucleotides!\n"}
   close ALIGNMENT;
   unless ($#alitaxa == $#treetaxa) {die "The tree and alignmentfiles do NOT have the same order as the number of taxa is different in alignment ",$alignmentfiles[$a]," ",$#alitaxa," and tree ",$treefiles[$a]," ",$#treetaxa,".\n"}
   $alignfile = $alignmentfiles[$a];
   $alignfile =~ s/\.phy$/_${casenumber}/;
  }
 }

 my $inmatch = 0;
 my $outmatch = 0;
 my @outgroup = @treetaxa;

 foreach my $p (3..$#paralogscreen) {
  foreach my $a (0..$#outgroup) {
   if ($outgroup[$a] eq $paralogscreen[$p]) {
    splice (@outgroup, $a, 1, "ingroup");
    $inmatch++;
   }
  }
 }
 foreach my $o (@outgroup) {
  unless ($o eq "ingroup" or $o !~ /[A-Z]/i) {
   $outmatch++;
  }
 }
 if (($outmatch + $inmatch) != ($#treetaxa+1)) {die "Ingroup and outgroup numbers do not match number of taxa in the tree ", $paralogline}
 if ($inmatch != ($#paralogscreen - 2)) {die "Ingroup number does not match number of taxa in the file provided ", $paralogline}

 my $aliinmatch = 0;
 my $alioutmatch = 0;
 my @alioutgroup = @alitaxa;

 foreach my $p (3..$#paralogscreen) {
  foreach my $a (0..$#alioutgroup) {
   if ($alioutgroup[$a] eq $paralogscreen[$p]) {
    splice (@alioutgroup, $a, 1, "ingroup");
    $aliinmatch++;
   }
  }
 }
 foreach my $o (@alioutgroup) {
  unless ($o eq "ingroup" or $o !~ /[A-Z]/i) {
   $alioutmatch++;
  }
 }
 if (($alioutmatch + $aliinmatch) != ($#alitaxa+1)) {die "Ingroup and outgroup numbers do not match number of taxa in the alignment ", $alignmentfiles[$a]}
 if ($aliinmatch != ($#paralogscreen - 2)) {die "Ingroup number does not match number of taxa in the file provided ", $paralogline}
#Generation taxa arrays for the two groups of the bipartition, so that the smaller will be the tested one
 my @inblast = ();
 my @outblast = ();
 if ($outmatch <= $inmatch) {
   foreach my $o (@outgroup) {
    unless ($o eq "ingroup" or $o !~ /[A-Z]/i) {
     push @inblast, $o;
    }
   }
 } 
 if ($outmatch > $inmatch) {
  foreach my $p (3..$#paralogscreen) {
    push @inblast, $paralogscreen[$p]
  }
 }
 foreach my $a (@alitaxa) {
  my $push = 1;
  foreach my $i (@inblast) {
   if ($a eq $i) {$push=0}
  }
  if ($push==1) {push @outblast, $a}
 }
#Generation of fasta files in blast-bin folder
 chdir "$trespexpath" or die "The folder ${trespexpath} not found.\n"; 
 chdir "blast/bin" or die "The folder ${trespexpath}/blast/bin not found.\n";
 my $infas = ($alignfile."_SuspectSequences.fas");
 open INFILE, ">$infas";
  foreach my $i (@inblast) {
   print INFILE ">",$i,"\n";
   print INFILE $Amatrix{$i},"\n"
  }
 close INFILE;
 my $outfas = ($alignfile."_OtherSequences.fas");
 open OUTFILE, ">$outfas";
  foreach my $o (@outblast) {
   print OUTFILE ">",$o,"\n";
   print OUTFILE $Amatrix{$o},"\n"
  }
 close OUTFILE; 
#Determination of database for blast
 if ($dbdone == 0) {
  my @keys = keys %dbgiven;
  my $dbfound1 = 0;
  my $dbfound2 = 0;
  for my $key (@keys) {
   if ($key =~ m/$database1/) {
    $dbfound1 = 1;
    $db1 = $dbgiven{$key};
    $aminodb1 = 1;
   }
   if ($key =~ m/$database2/) {
    $dbfound2 = 1;
    $db2 = $dbgiven{$key};
    $aminodb2 = 1;
   }
  }
  if ($dbfound1 == 0) {
   my $outdb = $database1;
   $outdb =~ s/\.fas|\.fa|\.fasta/_uniqueID\.fas/;
   my $seqcounter = 0;
   chdir "Databases" or die "The folder ${trespexpath}/blast/bin/Databases not found.\n";
   open OUTFILE, ">$outdb";
   close OUTFILE;
   open DATABASE, $database1 or die "${database1} not found.\n";
     while (my $line = <DATABASE>) {
      if ($line =~ m/^>/){
       $seqcounter++;
       open OUTFILE, ">>$outdb";
       print OUTFILE ">", $seqcounter, "\n"; 
       close OUTFILE;
      } else {
       if ($line =~ /I|E|L|Q|F|P|X/) {$aminodb1 = 1};
       chomp $line; 
       open OUTFILE, ">>$outdb";
       print OUTFILE $line, "\n"; 
       close OUTFILE;
      }
     }
     if ($aminodb1 == 1) {print "Data type of the database ", $database1," provided by the user: Aminoacids!\n"} else {print "Data type of the database ", $database1," provided by the user: Nucleotides!\n"}
    close ALIGNMENT;
   chdir "..";
   $db1 = "Databases/".$outdb;
   if ($aminodb1 == 1) {
    system "./makeblastdb -in ${db1} -parse_seqids -dbtype prot";
   } else {
    system "./makeblastdb -in ${db1} -parse_seqids -dbtype nucl";
   }
  }
  if ($dbfound2 == 0) {
   my $outdb = $database2;
   $outdb =~ s/\.fas|\.fa|\.fasta/_uniqueID\.fas/;
   my $seqcounter = 0;
   chdir "Databases" or die "The folder ${trespexpath}/blast/bin/Databases not found.\n";
   open OUTFILE, ">$outdb";
   close OUTFILE;
   open DATABASE, $database2 or die "${database2} not found.\n";
     while (my $line = <DATABASE>) {
      if ($line =~ m/^>/){
       $seqcounter++;
       open OUTFILE, ">>$outdb";
       print OUTFILE ">", $seqcounter, "\n"; 
       close OUTFILE;
      } else {
       if ($line =~ /I|E|L|Q|F|P|X/) {$aminodb2 = 1};
       chomp $line; 
       open OUTFILE, ">>$outdb";
       print OUTFILE $line, "\n"; 
       close OUTFILE;
      }
     }
     if ($aminodb2 == 1) {print "Data type of the database ", $database2," provided by the user: Aminoacids!\n"} else {print "Data type of the database ", $database2," provided by the user: Nucleotides!\n"}
    close ALIGNMENT;
   chdir "..";
   $db2 = "Databases/".$outdb;
   if ($aminodb2 == 1) {
    system "./makeblastdb -in ${db2} -parse_seqids -dbtype prot";
   } else {
    system "./makeblastdb -in ${db2} -parse_seqids -dbtype nucl";
   }
  }
  $dbdone = 1;
 }
#Blast searches
 for my $a ($infas, $outfas) {
  if ($aminoali == 1 and $aminodb1 == 1) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 1: ./blastp -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";close OUTFILE;
   print "BLAST search 1: ./blastp -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";   
   system "./blastp -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}";   
  } elsif ($aminoali == 1 and $aminodb1 == 0) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 1: ./tblastn -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";close OUTFILE;
   print "BLAST search 1: ./tblastn -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";   
   system "./tblastn -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}";     
  } elsif ($aminoali == 0 and $aminodb1 == 1) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 1: ./blastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";close OUTFILE;
   print "BLAST search 1: ./blastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";   
   system "./blastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}";     
  } elsif ($aminoali == 0 and $aminodb1 == 0) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 1: ./tblastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";close OUTFILE;
   print "BLAST search 1: ./tblastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}\n";   
   system "./tblastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db1} -query ${a}";       
  }
  if ($aminoali == 1 and $aminodb2 == 1) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 2: ./blastp -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}\n";close OUTFILE;
   print "BLAST search 2: ./blastp -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}\n";   
   system "./blastp -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}";   
  } elsif ($aminoali == 1 and $aminodb2 == 0) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 2: ./tblastn -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}\n";close OUTFILE;
   print "BLAST search 2: ./tblastn -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}\n";   
   system "./tblastn -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}";     
  } elsif ($aminoali == 0 and $aminodb2 == 1) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 2: ./blastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db2} -query ${a}\n";close OUTFILE;
   print "BLAST search 2: ./blastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db2} -query ${a}\n";   
   system "./blastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}";     
  } elsif ($aminoali == 0 and $aminodb2 == 0) {
   open OUTFILE, ">>BLAST_log.txt";print OUTFILE "BLAST search 2: ./tblastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db2} -query ${a}\n";close OUTFILE;
   print "BLAST search 2: ./tblastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_1st_${a}.txt -db ${db2} -query ${a}\n";   
   system "./tblastx -max_target_seqs 20 -outfmt \"6 qseqid sseqid pident score evalue\" -evalue ${evalue} -out BlastHits_2nd_${a}.txt -db ${db2} -query ${a}";       
  }
 }
 my $oldlocation = getcwd();
 my $newlocation = $returnpath."/blast_search/blast_results";
 foreach my $file (<*.fas>) {
  my $oldfile = $oldlocation;
  my $newfile = $newlocation;
  $oldfile .= "/" . $file;
  $newfile .= "/" . $file; 
  move ($oldfile,$newfile) 
 }
 my @first=();
 my @second=();
 foreach my $file (<BlastHits_*>) {
  if ($file =~ m/_1st_/) {
   if ($file =~ m/_SuspectSequences/) {$first[0]=$file}
   if ($file =~ m/_OtherSequences/) {$first[1]=$file}
  }
  if ($file =~ m/_2nd_/) {
   if ($file =~ m/_SuspectSequences/) {$second[0]=$file}
   if ($file =~ m/_OtherSequences/) {$second[1]=$file}
  }
  my $oldfile = $oldlocation;
  my $newfile = $newlocation;
  $oldfile .= "/" . $file;
  $newfile .= "/" . $file; 
  move ($oldfile,$newfile); 
 }
 chdir "$returnpath" or die "The folder ${returnpath} not found.\n";
 chdir "blast_search" or die "The folder ${returnpath}/blast_search not found.\n";
 chdir "blast_results" or die "The folder ${returnpath}/blast_search/blast_results not found.\n";
#Determination of best hit and confidence set of hits for both searches
 my %bestfirst=();
 my %confoutfirst=();
 my %conffirst=();
 my %bestsecond=();
 my %confoutsecond=();
 my %confsecond=();
 foreach my $suspect (@inblast) {
  my $s1found = 0;
  my $beste1 = ();
  my $epre1 = 0;
  my $beste2 = ();
  my $epre2 = 0;
  open FIRST, $first[0] or die "${first[0]} not found.\n";
   while (my $line = <FIRST>) {
    chomp $line;
    my @line = split /\s+/, $line;
    if ($line[0] eq $suspect and $s1found == 0) {
     $s1found = 1; 
     $bestfirst{$suspect} = $line[1];
     if ($line[4] =~ m/e-(\d+)/) {$beste1=$1;$epre1 = 1} else {$beste1=$line[4]}
    }
    my $thres = ($beste1-$ediff);
    if ($line[0] eq $suspect) {
     if ($epre1 == 0) {
      if ($beste1 == 0 and $line[4] == 0) {$conffirst{$suspect} .= "\t".$line[1]; $confoutfirst{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($beste1 > 0) {$conffirst{$suspect} .= "\t".$line[1]; $confoutfirst{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
     } else {
      if ($beste1 < $ediff) {$conffirst{$suspect} .= "\t".$line[1]; $confoutfirst{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($line[4] =~ m/e-(\d+)/ and $beste1 >= $ediff) {
       if ($1 >= $thres) {$conffirst{$suspect} .= "\t".$line[1]; $confoutfirst{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      }
     }
    }
   }
  close FIRST;
  my $s2found = 0;
  open SECOND, $second[0] or die "${second[0]} not found.\n";
   while (my $line = <SECOND>) {
    chomp $line;
    my @line = split /\s+/, $line;
    if ($line[0] eq $suspect and $s2found == 0) {
     $s2found = 1; 
     $bestsecond{$suspect} = $line[1];
     if ($line[4] =~ m/e-(\d+)/) {$beste2=$1;$epre2 = 1} else {$beste2=$line[4]}
    }
    my $thres = ($beste2-$ediff);
    if ($line[0] eq $suspect) {
     if ($epre2 == 0) {
      if ($beste2 == 0 and $line[4] == 0) {$confsecond{$suspect} .= "\t".$line[1]; $confoutsecond{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($beste2 > 0) {$confsecond{$suspect} .= "\t".$line[1]; $confoutsecond{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
     } else {
      if ($beste2 < 5) {$confsecond{$suspect} .= "\t".$line[1]; $confoutsecond{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($line[4] =~ m/e-(\d+)/ and $beste2 >= 5) {
       if ($1 >= $thres) {$confsecond{$suspect} .= "\t".$line[1]; $confoutsecond{$suspect} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      }
     }
    }
   }
  close SECOND;
 }
 foreach my $other (@outblast) {
  my $s1found = 0;
  my $beste1 = ();
  my $epre1 = 0;
  my $beste2 = ();
  my $epre2 = 0;
  open FIRST, $first[1] or die "${first[1]} not found.\n";
   while (my $line = <FIRST>) {
    chomp $line;
    my @line = split /\s+/, $line;
    if ($line[0] eq $other and $s1found == 0) {
     $s1found = 1; 
     $bestfirst{$other} = $line[1];
     if ($line[4] =~ m/e-(\d+)/) {$beste1=$1;$epre1 = 1} else {$beste1=$line[4]}
    }
    my $thres = ($beste1-$ediff);
    if ($line[0] eq $other) {
     if ($epre1 == 0) {
      if ($beste1 == 0 and $line[4] == 0) {$conffirst{$other} .= "\t".$line[1]; $confoutfirst{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($beste1 > 0) {$conffirst{$other} .= "\t".$line[1]; $confoutfirst{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
     } else {
      if ($beste1 < 5) {$conffirst{$other} .= "\t".$line[1]; $confoutfirst{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($line[4] =~ m/e-(\d+)/ and $beste1 >= 5) {
       if ($1 >= $thres) {$conffirst{$other} .= "\t".$line[1]; $confoutfirst{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      }
     }
    }
   }
  close FIRST;
  my $s2found = 0;
  open SECOND, $second[1] or die "${second[1]} not found.\n";
   while (my $line = <SECOND>) {
    chomp $line;
    my @line = split /\s+/, $line;
    if ($line[0] eq $other and $s2found == 0) {
     $s2found = 1; 
     $bestsecond{$other} = $line[1];
     if ($line[4] =~ m/e-(\d+)/) {$beste2=$1;$epre2 = 1} else {$beste2=$line[4]}
    }
    my $thres = ($beste2-$ediff);
    if ($line[0] eq $other) {
     if ($epre2 == 0) {
      if ($beste2 == 0 and $line[4] == 0) {$confsecond{$other} .= "\t".$line[1]; $confoutsecond{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($beste2 > 0) {$confsecond{$other} .= "\t".$line[1]; $confoutsecond{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
     } else {
      if ($beste2 < 5) {$confsecond{$other} .= "\t".$line[1]; $confoutsecond{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      if ($line[4] =~ m/e-(\d+)/ and $beste2 >= 5) {
       if ($1 >= $thres) {$confsecond{$other} .= "\t".$line[1]; $confoutsecond{$other} .= ("\t".$line[1]."\t".$line[3]."\t".$line[4])}
      }
     }
    }
   }
  close SECOND;
  close SECOND;
 }
#Printout compiled results of best hits and confidence set of hits for both searches
 chdir "..";
 chdir "blast_comparisons" or die "The folder ${returnpath}/blast_search/blast_comparisons not found.\n";
 open OUTFILE, ">Compiled_results_${alignfile}.txt";
  print OUTFILE "First BLAST search against ",$database1,":\nSequences suspected of paralogy:\n";
  foreach my $suspect (@inblast) {print OUTFILE $suspect,$confoutfirst{$suspect},"\n"} 
  print OUTFILE "\nOther sequences:\n";
  foreach my $other (@outblast) {print OUTFILE $other,$confoutfirst{$other},"\n"} 
  print OUTFILE "\n\nSecond BLAST search against ",$database2,":\nSequences suspected of paralogy:\n";
  foreach my $suspect (@inblast) {print OUTFILE $suspect,$confoutsecond{$suspect},"\n"} 
  print OUTFILE "\nOther sequences:\n";
  foreach my $other (@outblast) {print OUTFILE $other,$confoutsecond{$other},"\n"}
  print OUTFILE "\n";
 close OUTFILE;
#Determination if paralogy is given or not based on provided threshold values 
 my %besthitfirst=();
 my %besthitsecond=();
 my %confhitfirst=();
 my %confhitsecond=();
 my $cersortfirst = 0; 
 my $nosortfirst = 0; 
 my $cersortsecond = 0;
 my $nosortsecond = 0;
 my $definfirst = 0;
 my $definsecond = 0;
 my $defoutfirst = 0;
 my $defoutsecond = 0;
 foreach my $suspect (@inblast) {
  my $firstdivisor = 0;
  my $besthitfirstcount = 0;
  my $confhitfirstcount = 0;
  if (defined $bestfirst{$suspect}) {
   $definfirst++;
   foreach my $other (@outblast) {
    if (defined $bestfirst{$other}) {
     $defoutfirst++;
     $firstdivisor++;
     if ($bestfirst{$other} eq $bestfirst{$suspect}) {$besthitfirstcount++}
     my @conffirst = split /\t/, $conffirst{$other};
     my $cfound = 0;
     foreach my $a (@conffirst) {
      if ($a eq $bestfirst{$suspect} and $cfound == 0) {$confhitfirstcount++; $cfound = 1}
     }
    }
   }
   unless ($firstdivisor == 0) {$besthitfirst{$suspect} = ($besthitfirstcount/$firstdivisor)}
   unless ($firstdivisor == 0) {$confhitfirst{$suspect} = ($confhitfirstcount/$firstdivisor)}
   if (defined $besthitfirst{$suspect} and $besthitfirst{$suspect} <= $lowparathres) {$cersortfirst++} 
   if (defined $besthitfirst{$suspect} and $besthitfirst{$suspect} >= $upparathres) {$nosortfirst++} 
  }
  my $seconddivisor = 0;
  my $besthitsecondcount = 0;
  my $confhitsecondcount = 0;
  if (defined $bestsecond{$suspect}) {
   $definsecond++;
   foreach my $other (@outblast) {
    if (defined $bestsecond{$other}) {
     $defoutsecond++;
     $seconddivisor++;
     if ($bestsecond{$other} eq $bestsecond{$suspect}) {$besthitsecondcount++}
     my @confsecond = split /\t/, $confsecond{$other};
     my $cfound = 0;
     foreach my $a (@confsecond) {
      if ($a eq $bestsecond{$suspect} and $cfound == 0) {$confhitsecondcount++; $cfound = 1}
     }
    }
   }
   unless ($seconddivisor == 0) {$besthitsecond{$suspect} = ($besthitsecondcount/$seconddivisor)}
   unless ($seconddivisor == 0) {$confhitsecond{$suspect} = ($confhitsecondcount/$seconddivisor)}
   if (defined $besthitsecond{$suspect} and $besthitsecond{$suspect} <= $lowparathres) {$cersortsecond++} 
   if (defined $besthitsecond{$suspect} and $besthitsecond{$suspect} >= $upparathres) {$nosortsecond++} 
  }
 }
 if ($definfirst == 0 and $definsecond == 0) {
  chdir "No_hits" or die "The folder No_hits not found.\n";
   open OUTFILE, ">>No_hits_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_No_hits__${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir ".."; 
 } elsif ($defoutfirst == 0 and $defoutsecond == 0) {
  chdir "No_hits" or die "The folder No_hits not found.\n";
   open OUTFILE, ">>No_hits_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_No_hits__${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir ".."; 
 } elsif ($defoutfirst == 0 and $definsecond == 0) {
  chdir "No_hits" or die "The folder No_hits not found.\n";
   open OUTFILE, ">>No_hits_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_No_hits__${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir ".."; 
 } elsif ($definfirst == 0 and $defoutsecond == 0) {
  chdir "No_hits" or die "The folder No_hits not found.\n";
   open OUTFILE, ">>No_hits_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_No_hits__${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir ".."; 
 } elsif ($definfirst != 0 and $defoutfirst != 0 and $cersortfirst == $definfirst) {
  chdir "Certain_paralogy" or die "The folder Certain_paralogy not found.\n";
   open OUTFILE, ">>Certain_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_Certain_paralogs_${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir "..";
 } elsif ($definsecond != 0 and $defoutsecond != 0 and $cersortsecond == $definsecond) {
  chdir "Certain_paralogy" or die "The folder Certain_paralogy not found.\n";
   open OUTFILE, ">>Certain_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_Certain_paralogs_${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir "..";
 } elsif ($definsecond != 0 and $defoutsecond != 0 and $definfirst != 0 and $defoutfirst != 0 and $nosortsecond == $definsecond and $nosortfirst == $definfirst) {
  chdir "No_paralogy" or die "The folder No_paralogy not found.\n";
   open OUTFILE, ">>No_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_No_paralogs_${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir "..";
 } else {
  chdir "Uncertain_paralogy" or die "The folder Uncertain_paralogy not found.\n";
   open OUTFILE, ">>Uncertain_${paralogfile}";
    print OUTFILE $paralogline;
   close OUTFILE;
   open OUTFILE, ">Results_Uncertain_paralogs_${alignfile}.txt";
    print OUTFILE "Taxon\tProportion of best hits against\n\tFirst BLAST search\t\Second BLAST search\n\tBest hits\tConfidence set\tBest hits\tConfidence set\n";
    foreach my $suspect (@inblast) {
     print OUTFILE $suspect,"\t",$besthitfirst{$suspect},"\t",$confhitfirst{$suspect},"\t",$besthitsecond{$suspect},"\t",$confhitsecond{$suspect},"\n";
    }
   close OUTFILE;
  chdir "..";
 }
 
 chdir "..";
 chdir "..";
 
}

chdir "$trespexpath" or die "The folder ${trespexpath} not found.\n"; 
 chdir "blast/bin" or die "The folder ${trespexpath}/blast/bin not found.\n";
 my $oldlocation = getcwd();
 my $newlocation = $returnpath."/blast_search/blast_results";
 my $oldfile = $oldlocation;
 my $newfile = $newlocation;
 $oldfile .= "/BLAST_log.txt";
 $newfile .= "/BLAST_log.txt"; 
 move ($oldfile,$newfile); 
chdir "$returnpath" or die "The folder ${returnpath} not found.\n";

}

#Help--------------------------------------------------------------------
sub help {

my $a = $_[0];
my $b = $a+1;
if ($ARGV[$b] =~ m/Paralogy/i) {&paralogy}
elsif ($ARGV[$b] =~ m/Conflict/i) {&conflict}
elsif ($ARGV[$b] =~ m/Mis/i) {&fune;&fung;&funf}
elsif ($ARGV[$b] =~ m/Usage/i) {&usage}
elsif ($ARGV[$b] =~ m/Input/i) {&input}
elsif ($ARGV[$b] =~ m/Options/i) {&options}
elsif ($ARGV[$b] =~ m/^a$/i) {&funa}
elsif ($ARGV[$b] =~ m/^b$/i) {&funb}
elsif ($ARGV[$b] =~ m/^c$/i) {&func}
elsif ($ARGV[$b] =~ m/^d$/i) {&fund}
elsif ($ARGV[$b] =~ m/^e$/i) {&fune}
elsif ($ARGV[$b] =~ m/^f$/i) {&funf}
elsif ($ARGV[$b] =~ m/^g$/i) {&fung}
elsif ($ARGV[$b] =~ m/^h$/i) {&funh}
elsif ($ARGV[$b] =~ m/^i$/i) {&funi}
elsif ($ARGV[$b] =~ m/^j$/i) {&funj}
else {&general}

 sub usage {
 print "The chapter only provides a general outline of the usage of the different functions of TreSpEx. Details about the individual option and input files can be found in the section Options and Input file formats, respectively. TreSpEx contains 10 different functions. However, these functions can be grouped into three different types of analyses. The first type is the detection of paralogous sequences based on single-gene analyses as suggested by different authors and combines the functions a-d. The second one is the detection of conflict in datasets based on the PABA principle and combines the functions h-j. And the third is tree-based measurements of different forms of biases such as saturation or long branches as well as the calculation of average bootstrap support for a tree and combines the functions e-g.\n\n";
 &paralogy;
 &conflict;
 &fune;
 &fung;
 &funf; 
 }
  
 sub paralogy {
 print "The procedure of detection of paralogous sequences implemented in TreSpEx is an a posteriori procedure. This means, that by some way (e.g., HaMStR or OrthoMCL, ReMark, MultiMSOAR 2.0) sets of supposedly orthologous genes have already been generated. However, as these processes have a certain chance to group paralogous genes together, which eventually might mislead the analysis of the combined data as for these sets the gene tree is inferred instead of the species tree. Therefore, the individual sets of supposedly orthologous sequences should be screened for paralogous sequences before the data are used for further analyses. To do so, single gene trees are reconstructed and screened for clades with strong bootstrap support, as clades reflecting the splitting between two sets of paralogous sequences within a gene tree will gain strong support. However, as strong nodal can also be related to actual support for natural grouping additional filtering steps are necessary and have been implemented in TreSpEx. For more detail on this method please refer to the corresponding literature.\n\n";
 &funa;
 &funb;
 &func;
 &fund;
 }

 sub funa {
 print "A priori screening (fun –a) / Example in folder Paralogy/A_priori\n
Command line:\n
perl TreSpEx.v1.pl -fun a -gts Yor N -lowbs value -upbs value -possc 0,1,2,3 -poslb 0,1,2,3 -lowbl value -upbl value -possb 0,1,2,3 -maxtaxa value -blt value -ipt Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun a -gts Y -lowbs 95 -upbs 100 -possc 1 -poslb 2 -lowbl 4 -upbl 4 -possb 3 -maxtaxa 3 -blt 0.00001 -ipt Input_bipartitions_trees.txt -path /Users/ torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/Paralogy/A_priori/\n
This screening is done prior to any concatenation and analysis of the data. To use this function a list of names of tree files in Newick format with bootstrap support values has to be provided via –ipt Name-of-file. (e.g., -ipt Input_bipartitions_trees.txt). The tree files have to be in the same folder as the file containing the list. All trees in this list (i.e. the RAxML_bipartitions.-files) will then be screened for bootstrap support values. \n
All clades, which possess a certain BS value (e.g., 95 or 95 to 100), will be extracted from these trees for further analysis. Via the option –lowbs value and –upbs value the lower and upper bound of the range of bootstrap values will be defined. In the provided example –lowbs 95 and –upbs 100 have been used, so all clades with a bootstrap value from 95 to 100 will be extracted. If –lowbs 95 and –upbs 95 is chosen, all clades with a bootstrap value of 95 will be extracted\n
These detected clades will be written to the file PotentialParalogsBootstrap.txt and the tab-delineated file NOT_filtered_Hits_per_BS_value.csv summarizes how many clades across all trees have been found at each bootstrap value.\n
The next steps are that certain detected clades can be masked for the further analyses and/or the remaining detected clades be sorted based on certain criteria.\n
Masking of detected clades can be invoked to exclude clades from further analyses, for which there is evidence that the clade is a natural unit. For example, if a clade consists only of mammalian species it is more likely that the strong bootstrap is due to actual signal rather than being an artifact. To mask clades the option –gts has to be used (e.g., -gts Y). Moreover, a file or files beginning with GroupedTaxa and ending with .txt (e.g., GroupedTaxaFamily.txt) has to be provided. In this file each line defines a clade, which shall be masked. More than one file can be provided if different settings shall be tested. For example, masking clades based on different taxonomic levels (e.g, belonging to the same genus, class or phylum) can be done in that way. TreSpEx will use each file separately and write the results into individual folders. Therefore, the text between GroupedTaxa and .txt will be taken as a specifier and the folder named correspondingly beginning with TaxaScreening (e.g., TaxaScreeningFamily). TreSpEx will mask all clades, which consists only of taxa provided by the individual lines. For example, the file GroupedTaxaFamily.txt contains in the first line the names of all clitellate taxa present across all trees, even if not all clitellate species are present in all individual trees. For each individual tree all clades, which consists only of clitellate species as defined by the first line in the file GroupedTaxaFamily.txt will then be masked for further analyses.\n
The detected clades, which are not masked, will be saved in the file Pruned_PotentialParalogsBootstrap.txt in the corresponding result folder. Moreover, the corresponding trees, which contain a detected clade listed in Pruned_PotentialParalogsBootstrap.txt, will be compiled into the single file PrunedTrees_PotentialParalogsBootstrap.tre.\n
Besides masking the detected clades can be sorted based on different criteria. The criteria are strong conflict (invoked by –possc), long branch (invoked by –poslb) and short branch (invoked by –possb). This sorting is done in an ordered fashion provided by the user. In the example, the criterion strong conflict is checked first (i.e., -possc 1). If the clade matches this criterion it is written to the result files of this criterion. If not, the criterion long branch is checked next (i.e., -poslb 2). If that does not fit, the criterion short branch is checked (-possb 3). If that also does not fit, the clade is sorted into a result file compiling the remaining files. The user can invoke any combination of these criteria. For example, -possc 0 –poslb 1 –possb 2 would have the order long branch followed by short branch and the criterion strong conflict would not be invoked at all.\n
As for the masking the clades written into a corresponding result file (e.g., Pruned_PotentialStrongConflictBootstrap.txt) and the tree are compiled into one file (e.g., PrunedTrees_PotentialLongBranchParalogsBootstrap.tre) as well. Moreover, the tab-delineated file PrunedHits_per_BS_value.csv will be generated summarizing how many non-masked clades across all trees have been found at each bootstrap value for each criterion.\n
The criterion strong conflict means that the detected clade is in conflict with an a priori defined clade for masking. Therefore, to use the strong conflict criterion the masking procedure is a prerequisite to first filter out all clades, which are in agreement with this clade. Thus, all clades left must be a composition of members of at least two different groups of taxa (e.g., flatworms and an annelid). Invoking the strong conflict criterion now checks if a member of the predefined clade in GroupedTaxa*.txt file from above, which was not present in the detected clade, is present in the tree some where else. If that is the case, a conflict occurred between a clade with evidence for monophyly and a strongly supported clade found in the corresponding tree. For example, the file GroupedTaxaFamily.txt contains in the first line the names of all clitellate taxa as Clitellata is well established as a monophyly at present. If now a tree has a strongly supported clade of some clitellates and a polychaete, while other clitellates are present in the tree, but not in this clade, this finding contradicts monophyly of Clitellata and hence is at conflict with it.\n
The criterion long branch checks if a long internal branch is leading to the detected clade. TreSpEx determines if the internal branch leading to the detected clade is at least x times longer than the average of all internal branches in the tree. The x value has to be provided by the user using –lowbl value and –upbl value. For example, using –lowbl 4 and –upbl 4 the branch has to be at least 4 times longer. If the results based on different minimal x values shall be investigated this can be done in a single run by, for example, –lowbl 4 and –upbl 6. Then it will be tested if the branch is at least 4 times, 5 times or 6 times longer. The results of each x value will be written into an individual folder named LBfactor plus the x value (e.g., LBfactor4).  This means, that at least 5 times longer is a subset of the results of at least 4 times longer, and at least 6 times of both them.\n
The criterion short branch checks if at least one of taxa in the detected clade has a terminal branch length below a certain threshold. This criterion can be used to detect possible contamination issues between taxa of the dataset by looking for zero- or close to zero-branch length taxa. As it is seems that, for example, RAxML does calculate true zero terminal branch length (although they appear that way in the figure, in the tree file they still possess a very short branch) a threshold has to be provided by -blt value (e.g., -blt 0.00001). All terminal branch length below this value will be regarded as short-branched. However, as in very conserved genes most terminal branches have nearly zero branch length this is cannot be taken as an indication of contamination in such cases. As it is very unlikely that contamination occurred between all or most taxa of a dataset a maximum number of taxa present in a detected clade has to be provided by –maxtaxa value (e.g., -maxtaxa 3). If more taxa are present in clade, TreSpEx will not check for short branches. \n
";
 }

 sub funb {
 print "A posteriori screening (fun –b) / Example in folder Paralogy/A_posteriori\n
Command line:\n
perl TreSpEx.v1.pl -fun b -gts Yor N -lowbs value -upbs value -possc 0,1,2,3 -poslb 0,1,2,3 -lowbl value -upbl value -possb 0,1,2,3 -maxtaxa value -blt value -ipt Name-of-file -trf Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun b -gts Y -lowbs 95 -upbs 100 -possc 1 -poslb 2 -lowbl 4 -upbl 4 -possb 3 -maxtaxa 3 -blt 0.00001 -ipt Input_bipartitions_trees.txt -trf Tree_Fig3.txt -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/Paralogy/A_posteriori/\n
In contrast to the a priori screening (-fun a), herein the screening is done only for clades provided via a tree file. All detected clades not congruent with these clades will be masked. By this way one can a posteriori test if clades present, for example, in the best tree of an analysis of the concatenated data are potentially due to the misleading potential of paralogy. The tree file has not be completely resolved and is provided via the option –trf Name-of-file (e.g., -trf Tree_Fig3.txt). All resolved clades in that tree will be determined and saved in the file GroupedTaxa_A_posteriori_Groups.txt. All detected clades not in accordance with these clades will be masked and the remaining detected clades be written to the file AP_PotentialParalogsBootstrap.txt for further analyses. \n
Otherwise, it is the same as -fun a and the same options can be used. \n
";
 }

 sub func {
 print "Blast search against reference database (fun –c) / Example in folder Paralogy/BLAST\n
Command line:\n
perl TreSpEx.v1.pl -fun c -ppf Name-of-file -ipt Name-of-file -ipa Name-of-file –db1 Name-of-database –db2 Name-of-database -ediff value -ltp value -utp value -evalue value -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun c -ppf Pruned_PotentialParalogsBootstrap.txt -ipt Input_bipartitions_trees.txt -ipa Input_alignments.txt -db1 Bos_taurus -db2 Branchiostoma_floridae -ediff 5 -ltp 0.0 -utp 1.0 -evalue 1e-20 -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/Paralogy/BLAST/\n
To determine if suspicious clades detected by the screening procedures above or any other one are indeed paralogous sequences blast searches of sequences of that alignment against two reference databases can be conducted in TreSpEx. This allows comparing if the same hits are returned for the suspicious sequences and the other sequences in the alignment. Hereby, TreSpEx regards the smaller of the two groups of the bipartition as the one with the suspicious sequences. TreSpEx will sort the results of the blast searches into four different categories given certain threshold condition are met (see below): no hits in either all suspicious sequences, all other sequences or all sequences, certain paralogy, no paralogy and uncertain cases.\n
Therefore, a list of the suspicious clades, which shall be tested for paralogy by blast searches, has to be provided via the option –ppf Name-of-file (e.g., -ppf Pruned_PotentialParalogsBootstrap.txt). The file has to have the same format as the result files generated by the two above screening procedures (e.g., Pruned_PotentialParalogsBootstrap.txt). Each new line defines a clade and starts with the name of the tree file followed by the bootstrap value and the branch length of that clade and then by the names of taxa of the clade as they are in the tree and the corresponding alignment.\n
To be able to relate the name of the tree in each line to an alignment a list of names of tree files has to be provided via the option -ipt Name-of-file (e.g., -ipt Input_bipartitions_trees.txt), which is in the same order as a list of corresponding relaxed phylip files provided via the option -ipa Name-of-file (e.g., -ipa Input_alignments.txt). Both the tree files and the alignment files have also to be provided as some cross-checks are done between these two files and the taxa names of the clade.\n
Two reference databases, against which the blast search is conducted, have to be chosen via the options -db1 Name-of-database and -db2 Name-of-database (e.g., -db1 Bos_taurus -db2 Branchiostoma_floridae). Ideally, these databases are non-redundant, that is each orthologous sequence is represented in the database only by one sequence. Pre-compiled databases (http://annelida.de/research/bioinformatics/software.html) can also be used if placed in the folder blast/bin/Databases (see Installation). However, own databases can be deposited in the same folder as fasta files. TreSpEx will convert them into databases for blast searches.\n
The threshold of the e value can be set for the blast searches using the option –evalue value (e.g., -evalue 1e-20). TreSpEx will automatically determine, which blast search has to be conducted give the database and the alignment file. The results of the blast search of both the suspicious sequences and the other sequences will be placed in the folder blast_results, which will contain the text files with the actual blast results, the alignment files used and a log file.\n
Based on the results of the blast searches against the two reference data the results for each suspicious clade will be sorted into different categories. First, it is determined if at least one of the blast searches returned hits for both suspicious and other sequences. If that is not given the corresponding clade information will be written to the file No_hits_Pruned_PotentialParalogsBootstrap.txt in the folder blast_comparisons/No_hits. All remaining detected clades will be classified as certain paralogy, no paralogy and uncertain cases. Therefore, for each suspicious sequence the proportion of best hits in the searches of the other non-suspicious sequences is determined, which is identical with the best hit of that suspicious sequence. For the sorting two threshold values have to be provided via -ltp value and -utp value (e.g., -ltp 0.0 -utp 1.0). The ltp value provides a lower threshold. If for ALL suspicious sequences in the searches against one of the two databases this proportion is equal to or lower than this lower threshold this detected clade is regarded as a certain case of paralogy and written to the file Certain_Pruned_PotentialParalogsBootstrap.txt in the folder blast_comparisons/ Certain_paralogy. For example with -ltp 0.0, if the clade of suspicious sequences contains three sequences and for all three sequences the proportion of identical best hits is 0 in the search against either database 1 or 2 this case is given. On the other hand, the utp value provides an upper threshold. If for ALL suspicious sequences in the searches against both databases this proportion is equal to or higher than this upper threshold this detected clade is regarded as a certain case of none paralogy and written to the file No_Pruned_PotentialParalogsBootstrap.txt in the folder blast_comparisons/No_paralogy. For example with -utp 1.0, if the clade of suspicious sequences contains three sequences and for all three sequences the proportion of identical best hits is 1.0 in the searches against both databases 1 and 2 this case is given. All remaining clades will be written to the file Uncertain_Pruned_PotentialParalogsBootstrap.txt in the folder blast_comparisons/ Uncertain_paralogy.\n
Besides the files with clade information mentioned above also a file is generated in the corresponding folder with the proportion of identical sequences for each suspicious sequence in the searches against both databases, for further exploration by the user if needed. Moreover, these file contain not only the proportion of identical best hits, but also the proportion of identical hits within a confidence set of hits. Therefore, the option –ediff value (e.g., -ediff 5) has to be provided. This value provides the difference in the order of magnitude by which the e value can be higher to be still considered part of the confidence set of best hits. For example, a value of 5 and a best hit with an e value of e-78 means that all hits will be considered for this set, which have an e value not higher than e-73. However, this is only provided for further exploration by the user and is not relevant for the sorting by TreSpEx.\n
Finally, the results of the blast searches against both datasets comprising only the best hits and the hits of the confidence set are compiled for each clade of suspicios sequences into a single file (e.g., Compiled_results_23816Ali_22.txt) within the folder blast_comparisons.\n
";
 }

 sub fund {
 print "Pruning of suspicious sequences from the individual datasets (fun –d) / Example in folder Paralogy/Pruning\n
Command line:\n
perl TreSpEx.v1.pl -fun d -ppf Name-of-file -ipt Name-of-file -ipa Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun d -ppf Certain_Pruned_PotentialParalogsBootstrap.txt -ipt Input_bipartitions_trees.txt -ipa Input_alignments.txt -path /Users/torstenstruck/ Documents/Data/Arbeit/Auswertung/TreSpEx_Example/Paralogy/Pruning/\n
Using this function the sequences of a suspicious clade can be pruned from the corresponding relaxed phylip files. Therefore, a list of the suspicious clades, which shall be pruned, has to be provided via the option –ppf Name-of-file (e.g., -ppf Certain_Pruned_PotentialParalogsBootstrap.txt). The file has to have the same format as the result files generated by the two above screening procedures or by the blast searches in function c (e.g., Pruned_PotentialParalogsBootstrap.txt or Certain_Pruned_PotentialParalogsBootstrap.txt). Each new line defines a clade and starts with the name of the tree file followed by the bootstrap value and the branch length of that clade and then by the names of taxa of the clade as they are in the tree and the corresponding alignment. Hereby, TreSpEx regards the smaller of the two groups of the bipartition as the one with the suspicious sequences to minimize the number of sequences to be pruned from the dataset. As the sequences of the suspicious clade are paralogs of the other clade of the bipartition this is also true the other way around, the sequences of the sequences of the other clade of the bipartition are the paralogs of the suspicious clade. The information for the smaller clade will be written to the file IngroupOutgroup_Name-of-file (e.g., IngroupOutgroup_Certain_Pruned_Potential ParalogsBootstrap.txt)\n
To be able to relate the name of the tree in each line to an alignment a list of names of tree files has to be provided via the option -ipt Name-of-file (e.g., -ipt Input_bipartitions_trees.txt), which is in the same order as a list of corresponding relaxed phylip files provided via the option -ipa Name-of-file (e.g., -ipa Input_alignments.txt). Both the tree files and the alignment files have also to be provided as some cross-checks are done between these two files and the taxa names of the clade.\n
As there can be more than two sets of suspicious sequences within an individual dataset a first step of TreSpEx is to compile all instances of suspicious clades into a single line for each individual dataset and write this information to the file Compiled_Certain_ Pruned_PotentialParalogsBootstrap.txt. Then the sequences will be pruned from the corresponding relaxed phylip alignment file and the modified alignment saved in the folder FilesPruned. All alignment files from which no sequences were pruned will be copied into the folder FilesNotPruned.\n
";
 }

 sub conflict {
 print "TreSpEx in combination with a program for phylogenetic reconstruction such as RAxML [15] or PhyloBayes [16] can also be used to detect conflicts in datasets based on the PABA principle [2, 6]. Using this principle conflict is detected on a node-by-node and partition-by-partition basis utilizing nodal support values. The PABA (Partition Addition Bootstrap Alteration) principle was first proposed using bootstrap values [2], but can also be conducted with any other nodal support values such as Bremer support (PABSA) or posterior probabilities (PAPPA) [6]. For reasons of simplicity it will be referred to in the following as PABA. To conduct a PABA analyses four steps have to be done. \n
First, the partitions of the datasets have to be combined in all possibilities or in a range of possibilities (function h). The latter means, that, for example, if a dataset has 10 partitions the user can also choose to generate only all possible combination with, for example, 8 or 9 combined partitions. Second, a phylogenetic reconstruction of each of these combinations has to be conducted using a program of the users choice. Third, the nodal support values for each bipartition have to be summarized across all datasets of the second step. If the program used by the user provides for each dataset a file, in which all tree files for determining the support are compiled, the function i of TreSpEx can be used to for this summary step. For example, RAxML provides all trees of the bootstrap analysis in a files beginning with RAxML_bootstrap*, while PhyloBayes compiles all trees of the chain in the file *.treelist. Fourth, the actual PABA analysis has to be conducted, which determines how the support for a bipartition (node) changes as a partition is added given the order of addition (as 8th or 9th partition added for example). This can be conducted using function j in TreSpEx.\n
Moreover, using TreSpEx two tests of significance of the PABA results can be conducted as proposed by Struck [6] derived from a test developed for ILD tests [17, 18] and from the Tempelton test [19-23]. First, using a permutation test it can be assessed for each node and partition if the same PABA value can be obtained just by chance due to randomly partitioning the data set in partitions of same sizes as defined partitions. To conduct the permutation test all four steps of the PABA analysis above (including the usually time consuming phylogenetic reconstruction) have to be done not only for the original dataset, but also for each of the, for example, 100 permutations of the original dataset. The second test employs a Wilcoxon-Signed-Rank test (WSRT) to determine if for a given set of bipartition (e.g., all nodes of the best tree or an alternative tree) the addition of a partition is overall beneficial to nodal support or detrimental. Thus, it can be assessed if the addition of partition its positive contribution outweighs, if present, a negative impact on a just few nodes or not. Hence, it can guide the decision if a partition should be completely excluded.\n
\n";
 &funh;
 &funi;
 &funj;
 }

 sub funh {
 print "Combination of partitions and generation of corresponding datasets (fun –h) / Example in folder PABA/Combination_and_permutation\n
Command line:\n
perl TreSpEx.v1.pl -fun h -aln Name-of-file -partf Name-of-file -ubnp value -lbnp value -per Y/N -nper value -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun h -aln Eunicida4Genes_nonInterleaved.phy -partf Partitions_Eunicida.txt -ubnp 4 -lbnp 1 -per Y -nper 100 -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/PABA/\n
Using this function the partitions of a dataset will be combined in all possibilities for a given range of numbers of partition to be combined. This range will be determined by the options -ubnp value and -lbnp value (e.g., -ubnp 4 -lbnp 1). For example, if the dataset consists of 4 partitions -ubnp 4 -lbnp 1 would mean that all possible combinations of partitions will be generated ranging from combining only 1 to 2, 3 and 4. However, if the dataset comprised 5 or more partitions not all possible combinations would be generated, but only the ones in which up to four partitions are combined. Within this range though all possible generations will be generated. \n
The alignment of the dataset is provided via the option –aln Name-of-file (e.g., -aln Eunicida4Genes_nonInterleaved.phy) in the relaxed phylip format and a definition of partition by the option -partf Name-of-file (e.g., -partf Partitions_Eunicida.txt). The format of the latter is similar to partition definition used by RAxML. Finally, if a permutation test shall be conducted permutated datasets will generated (including the same combinations of partitions as for the original dataset) the option –per has to be set to Y or Yes and the number of permutations to be conducted via the option –nper value (e.g., -nper 100).\n
TreSpEx will write out relaxed phylip files for each generated combination into the folder Combinations. In parallel is will as generate a corresponding partition file, if partitioned phylogenetic analyses of these data shall be conducted. If permutated data are generated as well, a corresponding permutated relaxed phylip file for each combination will be written to an individual subfolder for each permutation. For these permutated files the same partition file applies as for the original dataset. For the sake of reproducibility each permutation of the sequence positions is compiled in the file Permutations_used.txt.\n
";
 }

 sub funi {
 print "Summarizing nodal support across different datasets (fun –i) / Example in folder PABA/Summary_bootstrap\n
Command line:\n
perl TreSpEx.v1.pl -fun i -tf Name-of-file -mint value -ipt Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun i -tf TaxaNames.txt -mint 1 -ipt Bootstrap_trees_Original.txt -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/ PABA/RAxML/Original/\n
Using this function the bipartition table for each compilation file, in which all tree files for determining the support for the analyses of a dataset are compiled (e.g., RAxML_bootstrap* files of RAxML or *.treelist of PhyloBayes), is generated and these are summarized into one matrix for all files. A plain text file, which lists the names of all such compilation files to be summarized, is provided via the option -ipt Name-of-file (e.g., -ipt Bootstrap_trees_Original.txt). For each bipartition present in any of the trees in that compilation file the number of occurrences is counted and the support value is calculated as a percentage of all trees present in that compilation file. These are written into a bipartition table for each file (i.e., dataset) if the support value of the bipartition is equal to or larger than a threshold value. The file name begins with Bipartition_Table_*. Via option –mint value (e.g., -mint 1) this threshold is provided.\n
Then the results of these individual bipartition tables are compiled into one matrix for all files (i.e., dataset) analyzed in this run. If a bipartition is present in the tables of some datasets, but not in the others the threshold value minus 1 is written as lowest possible value for all instances, which lack this bipartition. For example, if the threshold is 1, this value would be 0 for all datasets, which would not have this bipartition. To generate the summary table it is necessary that all trees have the same set of taxa and this is checked using a list of taxon names via the option -tf Name-of-file (e.g., -tf TaxaNames.txt). This prerequisite is necessary as it is otherwise not possible to align the bipartitions across different datasets to each other. The summary matrix is written to the file Compiled_Bipartition_Tables.txt.\n
If a permutation test shall be conducted in function j this summary has to be done for each permutation.\n
";
 }

 sub funj {
 print "PABA analyses and significance tests (fun –j) / Example in folder PABA/PABA_WSRT_and_Permutation\n
Command line:\n
perl TreSpEx.v1.pl -fun j -per Y/N -wsrt Y/N -partf Name-of-file -tabf Name-of-file –mint value -maxt value -ori Name-of-file -bpf Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun j -per Y -wsrt Y -partf Partitions_Eunicida.txt -tabf Tabulated_bootstrap_files.txt -mint 0 -maxt 100 -ori Original_Compiled_Bipartition_Tables.txt -bpf Biparts_for_WSRT.txt -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/PABA\n
Using this function the actual PABA analysis is conducted. Therefore, a definition of partition has to be provided by the option -partf Name-of-file (e.g., -partf Partitions_Eunicida.txt), which has the format similar to the partition definition used by RAxML. Moreover, via -tabf Name-of-file (e.g., -tabf Tabulated_bootstrap_files.txt) a list of names of tabulated summary files of support values is provided. These summary files (e.g., Original_Compiled_Bipartition_Tables.txt) have to have a format similar to the one generated by –fun i (see Input file formats). For all summary files of this list a PABA analysis conducted. The results of each analysis is written into a file beginning PABA_results_woSignificance_* with in separate Results folder within the folder PABA_calculations.\n
As some measurements of nodal support such as bootstrap values or posterior probabilities have upper and lower bounds (e.g., bootstrap values can only be as high as 100 or as low as 0) the PABA calculations would be misled in cases where these upper and lower values are not altered by the addition of a partition [2, 6]. Therefore, such cases will not be further considered for the PABA calculation. To do so, the user has to provide the upper and lower bounds in the summary files using the options –mint value and -maxt value (e.g., -mint 0 -maxt 100). These values have not always to be 0 and 100. For example, posterior probabilities –maxt is more likely to be 1.0. If the summary files were generated with the function i of TreSpEx and –mint was set to, for example, 5 then (as it is, for example, the default setting for bipartition tables in PAUP), then using this function –mint would have to be 4 (i.e., 5-1; see function i) as this would be lowest value possible in the summary files generated by function i.\n
If a WSRT shall be conducted the option –wsrt has to set to Y or Yes (e.g., -wsrt Y). Moreover, via option -bpf Name-of-file (e.g., -bpf Biparts_for_WSRT.txt) a list of names of files has to be provided, each of which contains a set of bipartitions to be tested by the WSRT. The format of these files is simple: each new line provides the identifier of the bipartition (e.g., iiiiiiiiiiiiiioi or a specific name) and all identifier present in this file will be regarded as one set. Therefore, for each set, which shall be test its own file has to be generated (e.g., Nodes_28Sonly.txt and Nodes_All4Genes.txt). It is important to notice that the identifiers of the bipartitions have to be the exact same bipartition identifiers as used in the first column of the tabulated summary file. The results of the WSRT will be written into the file beginning with WSRT_results_ in the same folder as the results of the PABA calculation.\n
If a permutation test shall be conducted the option –per has to set to Y or Yes (e.g., -per Y). Moreover, via option -ori Name-of-file (e.g., -ori Original_Compiled_Bipartition_ Tables.txt) the name of the summary file of the original dataset, which shall be tested, has to be provided. Moreover, this name has also to be in the list of names provided by option –tabf. This list has also to include the names of the summary files of all permutated datasets, but no other datasets as all summary files from this list except the original one will be taken for the permutation test and compared to the results from the original data. The results of the permutation test are written into the file beginning with PABA_results_Significance_ in the Results folder of the original data in the folder PABA_calculations.\n
";
 }

 sub fune {
 print "Indices for long-branched taxa based patristic distances in a tree (fun –e) / Example in folder Miscellaneous/LBscore_TipToRoot_EvolDist\n
Command line:\n
perl TreSpEx.v1.pl -fun e -ipt Name-of-file -tf Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun e -ipt Input_bipartitions_trees.txt -tf TaxaNames.txt -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/Saturation_and_such/LBscore_TipToRoot_EvolDist\n
Using this function, different indices for long-branched taxa can be calculated based on patristic distances (PD), i.e. the distance between two taxa based on the connecting branches, in a tree. One index, which is often used as a proxy in this regard, is the evolutionary rate of a dataset [8, 9]. To gauge the evolutionary rate the average PD between any pair of taxa in the tree is calculated instead of an uncorrected genetic distance p calculated from the alignment. Another measurement is the tip-to-root distance of each taxon to detect long-branched taxa [8, 9], which is also calculated by TreSpEx. Finally, the new LB score is calculated. To assess the branch length heterogeneity in a tree the LB score is based on the mean pairwise PD of a taxon i to all other taxa in the tree relative to the average pairwise PD over all taxa.\n
In specific, the score measures for each taxon the percentage deviation from the average and is independent of the root of the tree in contrast to tip-to-root indices.\n
Therefore, a list of names of tree files in Newick format with branch length has to be provided via –ipt Name-of-file. (e.g., -ipt Input_bipartitions_trees.txt). For each tree file the calculations above will be performed. To generate the taxon vs tree matrices for the taxon-specific indices (i.e., tip-to-root distance and LB score) it is necessary to provide a list of taxon names, which occur across all trees analysed, via the option -tf Name-of-file (e.g., -tf TaxaNames.txt).\n
In addition to the taxon-specific values, for both tip-to-root distances and LB scores tree-specific indices will be calculated to allow a direct comparison of the results of different trees. These indices for each tree are for both indices the average of the upper quartile of values as well as the standard deviation of the values as an indication of heterogeneity in the data. These four indices and the average PD of each tree will be saved in the file LB_scores_summary_perPartition.txt. The taxon vs. tree matrices will be written to the files LB_scores_perTaxon.txt and TR_scores_perTaxon.txt for and LB scores and tip-to-root distances. Moreover, the taxon vs. taxon PD matrices of each tree will be saved in the folder PD_matrices.\n
";
 }

 sub fung {
 print "Index for saturation based patristic distances in a dataset (fun –g) / Example in folder Miscellaneous/Saturation\n
Command line:\n
perl TreSpEx.v1.pl -fun g -ipt Name-of-file -ipa Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun g -ipa Input_alignments.txt -ipt Input_bipartitions_trees.txt -path /Users/torstenstruck/Documents/Data/Arbeit/Auswertung/TreSpEx_Example/ Saturation_and_such/Saturation\n
Using this function, two indices for saturation in dataset have been proposed, which are derived from plotting the patristic distances (PD), i.e. the distance between two taxa based on the connecting branches in a tree, against the uncorrected genetic distance p calculated from the alignment for each pair of taxa [7]. One index, which has been used, is the slope of the linear regression of this plot as well as the R2 fit of the data to this slope. The higher the slope or the better the fit the less saturated is the dataset. \n
Therefore, a list of names of tree files in Newick format with branch length has to be provided via –ipt Name-of-file. (e.g., -ipt Input_bipartitions_trees.txt), which is in the same order as a list of corresponding relaxed phylip files provided via the option -ipa Name-of-file (e.g., -ipa Input_alignments.txt). For each pair of tree and alignment file the indices above will be calculated.\n
The taxon vs. taxon matrices of both the PD and p distance of each tree and alignment will be saved in the folders PD_matrices and P_matrices, respectively. The slope and R2 of each plot will be written to the file Correlation_Slope_Summary.txt.\n
";
 }

 sub funf {
 print "Average bootstrap support of trees (fun –f) / Example in folder Miscellaneous/Average_bootstrap\n
Command line:\n
perl TreSpEx.v1.pl -fun f -ipt Name-of-file -path Path-from-root\n
Example command line:\n
perl TreSpEx.v1.pl -fun f -ipt Input_bipartitions_trees.txt -path /Users/torstenstruck/ Documents/Data/Arbeit/Auswertung/TreSpEx_Example/Saturation_and_such/Average_bootstrap\n
Average bootstrap support has been used to assess if the alteration of certain conditions such as the addition of data or taxa was beneficial or detrimental for the phylogenetic reconstruction [24]. Therefore, a list of names of tree files in Newick format with bootstrap values has to be provided via –ipt Name-of-file. (e.g., -ipt Input_bipartitions_trees.txt). For each tree the average bootstrap support across all resolved nodes in the tree will be calculated and written into the file Average_BS_perPartition.txt.\n
";
 }

 sub input {
 print "Tree files have to be in the Newick format as they are, for example, generated by RAxML (e.g., RAxML_besttree..., RAxML_bootstrap...  or RAxML_bipartition....). For all analyses except the tip-to-root distance TreSpEx does not take the root of the tree into account. Thus, TreSpEx essentially works with unrooted trees even if rooted trees are provided. 
Alignment files have to be in the relaxed sequential phylip format and database files in the fasta format.
Sequences being suspected to be paralogs to the other sequences in the alignment and which shall be blasted against reference databases or pruned from alignments will be provided via a plain text file with following format:
name-of-treefile	bootstrap-support	branch-length	name-of-sequence name-of-sequence name-of-sequence…
For example,
RAxML_bipartitions.RPL24Ali.phy_fastBoot_Out.txt 96 1.75007055 Lanice_conchilega Eisenia_fetida 
RAxML_bipartitions.22680Ali.phy_fastBoot_Out.txt 99 0.25451239 Pomatoceros_lamarckii Alvinella_pompejana 
RAxML_bipartitions.22433Ali.phy_fastBoot_Out.txt 100 4.17620957 Scoloplos_armiger Sthenelais_boa Eurythoe_complanata_EST 
RAxML_bipartitions.22606Ali.phy_fastBoot_Out.txt 100 0.99360951 Cerebratulus_lacteus Owenia_TS 
Some options require the listing of taxon names, names of files containing lists of bipartition sets to be tested, the names of the tabulated summary files for the PABA calculation, names of tree files or names of alignment files via a plain text file. Be aware that in the latter two cases these names have to be in the same order, so that the name of a tree file at that position corresponds to the name of the alignment file, from which the tree is was reconstructed, at the same position in the other file. The format of these files is always the same. A new line provides each name:
	Taxa example	or tree example	or alignment example
Name-1	Eurythoe_complanata	RAxML_bipartitions.21904Ali.phy_fastBoot_Out.txt	21904Ali.phy
Name-2	Terebratalia_transversa	RAxML_bipartitions.21912Ali.phy_fastBoot_Out.txt	21912Ali.phy
Name-3	Arenicola_marina	RAxML_bipartitions.21942Ali.phy_fastBoot_Out.txt	21942Ali.phy
Name-4	Bugula_neritina\t	RAxML_bipartitions.21952Ali.phy_fastBoot_Out.txt	21952Ali.phy
...	...\t\t	...\t\t\t\t\t\t	...
Some options require the definition of the partitions. The format of this plain text file is similar to the format used by RAxML, though it is not required to provide a substitution model, but it can be done. Each new line provides a new definition, which has the following general format:
name-of-substitution-model, name-of-partition = definition-of-partition
The definition of the partition can have different possibilities. The simplest definition is that the partition starts at a position and ends at one (e.g., 1-500). However, it is also possible that the partition is located in two or more parts of the datasets (e.g., 250-350, 400-500, 700-800) or that codon positions are defined (e.g., 1-333\3 for first codon position, 2-333\3 for second and 3-333\3 for third). Finally, all these can be combined into a single file, so that a file can either look like this:
DNA, Partition1 = 1-600\t	or like this:\tDNA, Partition1 = 1-300\3, 2-300\3
WAG, Partition2 = 601-700\t\t\tDNA, Partition2 = 3-300\3, 701-900
DNA, Partition3 = 701-900\t\t\tDNA, Partition3 = 301-400, 461-600
...\t\t\t\t\t\tDNA, Partition4 = 401-460\3
\t\t\t\t\t\tDNA, Partition5 = 402-460\3
\t\t\t\t\t\tDNA, Partition6 = 403-460\3
\t\t\t\t\t\tWAG, Partition7 = 601-700
\t\t\t\t\t	...
For the PABA calculation tabulated summary files of the support values are required in a format as is generated by –fun i. This is a plain text file in which the columns of the table are separated by tabs (this is important as white spaces are allowed to list taxa names of a bipartition with a single column). In this table the first row defines the names of the datasets, which have to include the names of partitions used to build the supermatrix and as they are defined in the also provided partition file (as it will be the case when the datasets have been generated with –fun h). For example taken the left example from above, if the dataset was build from Partition1 and Partition2 the name of the dataset has also to include these names, for example, as Partition1Partition2. The first column of the table serves as the identifier of the bipartition (e.g. iiiiiiiiiiiiiioi or a specific name) and which will also be used in the files determining sets of bipartitions for WSRT tests. Then additional can be provided, but have to include at least one non-digit character (this requirement is not necessary for the first column). This can be used, for example, to provide proper taxon names for the each of the two bipartitions. After these columns the columns with the support values (e.g., bootstrap values) follow and each column has as a heading the name of the corresponding dataset, which includes the names of the partition present in this dataset. The general setup for three partitions in all combinations would be as follows:
\t\t\tPart1\tPart2\tPart3\tPart1Part2\tPart2Part3\tPart1Part3\tPart1Part2Part3
id1\tt1 t2 t3\tvalue\tvalue\tvalue\tvalue\t\tvalue\t\tvalue\t\tvalue 
id2\tt2 t4\t\tvalue\tvalue\tvalue\tvalue\t\tvalue\t\tvalue\t\tvalue 
id3\tt5 t6\t\tvalue\tvalue\tvalue\tvalue\t\tvalue\t\tvalue\t\tvalue 
id4\tt19 t1\t\tvalue\tvalue\tvalue\tvalue\t\tvalue\t\tvalue\t\tvalue 
...\t...\t\t...\t...\t...\t...\t\t...\t\t...\t\t...
For an example see the files provided in the TreSpEx_Example/PABA/PABA_WSRT_and_Permutation.

For the Wilcoxon-Signed-Rank test sets of bipartitions have to be provided. For the whole set it will tested if the contribution of a partition over all bipartitions in that set is more positive or negative based on the PABA results. The format of the plain text file is simple as each new line provides the identifier of the bipartition (e.g. iiiiiiiiiiiiiioi or a specific name). Important: This identifier has to be the exact same bipartition identifier as used in the first column of the tabulated summary file of the support values across the different datasets. 
	For example:	\tor
	Nodes_28Sonly.txt	Nodes_All4Genes.txt
id1	iiiiiiiiiiiiiioi	iiiiiiiiiiiiiioi
id2	iioiiiiiiiiiiooo	oooooooooooooioi
id3	ooiooooooooooooi	iiiiiiiiiiiiiooo
id4	iioiiiiiiiiiiooo	ooooooiioooooooo
…	ooooooiioooooooo	ooooooiioioooooo
	ooooooiioioooooo	iiiiiiooioiiiooo
	iioiiiooioiiiooo	iooiooooooiooooo
	iooiooooooiooooo	iooioooooooooooo
	iooioooooooooooo	ooioooooooooiooo
	oiooiiooiooiiooo	oiioiiooiooiiooo
	oooooioooooioooo	oioooooooooioooo
	oiooooooiooooooo	oioooioooooioooo
	oiooooooioooiooo	ooooioooiooooooo
	oiooioooioooiooo	oiooiiooiooioooo

For masking a plain text file or files beginning with GroupedTaxa and ending with .txt (e.g., GroupedTaxaFamily.txt) has to be provided. The format of the plain text file is simple as each new line provides the names of a clade, which shall be masked, separated by a white space. However the names have to be identical to the ones in the trees and should contain all representatives of that clade across all datasets, so all representatives might be present in all datasets. The general setup would be as follows:
TaxonA TaxonB TaxonC
TaxonE TaxonG
TaxonH TaxonD
For example,
Hirudo_medicinalis Helobdella_robusta Haementeria_depressa Perionyx_excavatus …
Sipunculus_nudus Themiste_lageniformes
Myzostoma_cirriferum_EST Myzostoma_seymourcollegiorum
Alvinella_pompejana Lanice_conchilega Pectinaria
Capitella_teleta Urechis_caupo
Terebratalia_transversa Bugula_neritina_EST
…

IMPORTANT: All input files have to be in the same folder!!!
Except for database files, which have to be placed in the folder blast/bin/Databases within the TreSpEx folder.
";
 }

 sub options {
 print "Options (in alphabetical order)
\n-aln:\n	name-of-file = name of an alignment in the relaxed sequential phylip format for the combination of different partitions in -fun h
\n-blt:\n	≥ 0 = this option provides the maximal length a short branch leading to a terminal taxon can have in option –possb (default = 0), this option has to be turned on for option -possb
\n-bpf:\n	name-of-file = name of a plain text file providing the names of files, each of which contains a list of bipartitions, which shall be independently tested by the WSRT test in –fun j; a new line provides each name:
		For example,
	Testset-1	Nodes_28Sonly.txt
	Testset -2	Nodes_All4Genes.txt
	…	…

\n-db1, –db2:\n	name-of-database = name of the database, which shall be used as the first or second database against which the blast searches shall be conducted; these databases should be non-redundant, that is, each orthologous sequence shall be represented by only one sequence (as for example is the case for the genome databases used by HaMStR); there are two possibilities with these options; first, a database is used, which is provided by TreSpEx already (see below) or, second, the user provides a database. In the latter case the file of the database has to be a fasta file and be placed into the folder blast/bin/Databases within the TreSpEx folder. TreSpEx will automatically recognize, if it is a protein and nucleotide database, and generate a database suitable for blast searches. 
The databases of the following species are provided and the name used for –db1 or –db2 has to be exactly written as here or as in the corresponding folder in the folder blast/bin/Databases within the TreSpEx folder:
Anolis_carolinensis	Apis_mellifera	Bombyx_mori	Bos_taurus
Branchiostoma_floridae	Caenorhabditis_elegans	Capitella_teleta	Ciona_intestinalis
Danio_rerio	Daphnia_pulex	Drosophila_melanogaster
Gallus_gallus	Helobdella_robusta	Homo_sapiens	Lottia_gigantia
Monodelphis_domestica	Mus_musculus	Ornithorhynchus_anatinus
Pediculus_humanus	Schmidtea_mediterranea	Taeniopygia guttata
Tribolium_castaneum	Xenopus_tropicalis

\n-ediff:\n	≥ 0 = the difference in magnitude between the best e-value and the highest e-value, which is still considered to be part of the confidence set in –fun c, (default 5); this number has to be an integer; even if provided, there are some exceptions: if e-value of the best hit = 0.0 than only other hits, which also have an e-value of 0.0 will be considered for the confidence set or if e-value of the best hit > 0 or the magnitude of the e-value of the best hit smaller than provide magnitude (e.g. e-5) than all hits will be part of the confidence set
\n-evalue:\n	any value = the e-value threshold for the BLAST search can be chosen as well in –fun c (default 10, optional); the input format can either 0.00001 or 1e-5 as required by the BLAST program
\n-fun:\n	a = a priori screening of bootstrap support of individual gene trees as well as masking and sorting of results in accordance with certain criteria
	b = a posteriori screening of bootstrap support of individual gene trees as well as masking and sorting of results in accordance with certain criteria, 
	c = blast searches of alignments suspected to be affected by paralogous sequences against two reference databases
	d = automatic pruning of suspicious sequences from an alignment 
	e = calculation of LB scores, tip-to-root distance and evolutionary rate
	f = calculation of average bootstrap support of a tree
	g = calculation of saturation indices based on pair-wise uncorrected p and patristic tree distances
	h = generation of all possible combinations of partitions and the corresponding supermatrices and partition files as well as optionally permutated datasets of these for statistical analyses
	i = summarizing bootstrap support across different datasets into a single bipartition table
	j = calculation of PABA results for conflict assessment of a node-by-node and partition-by-partition basis including Wilcoxon-Signed-Rank tests and permutation significance tests
\n-gts:\n	Y, Yes, N or No = when a masking approach for strongly supported clades in –fun a or b shall be used, the filtering has to be turned on using this option with Y or Yes (Default=No). The clades to masked have to be provided via txt-files starting with GroupedTaxa (e.g., GroupedTaxaPhylum.txt).
\n-ipa:\n	= name-of-file = name of a plain text file providing the names of alignment files, which shall be investigated for -fun c, d and g. Be aware that for these names have to be in the same order as the list of tree files provided via -ipt, so that the name of an alignment file at that position corresponds to the name of the tree file, that was reconstructed from this alignment, at the same position in the other file provided by -ipt. The format of this file is simple and each name of an alignment file is provided in a new line:
		For example,
	Alignment-name-1	21904Ali
	Alignment -name-2	21912Ali
	Alignment -name-3	21942Ali
	Alignment -name-4	21952Ali
	…	…

\n-ipt:\n	= name-of-file = name of a plain text file providing the names of tree files, which shall be investigated for -fun a, b, c, d, e, f, g and i. Be aware that for all option also requiring the option –ipa these names have to be in the same order in both lists, so that the name of a tree file at that position corresponds to the name of the alignment file, from which the tree is was reconstructed, at the same position in the other file provided by -ipa. The format of this file is simple and each name of a tree file is provided in a new line:
		For example,
	Tree-name-1	RAxML_bipartitions.21904Ali.phy_fastBoot_Out.txt
	Tree-name-2	RAxML_bipartitions.21912Ali.phy_fastBoot_Out.txt
	Tree-name-3	RAxML_bipartitions.21942Ali.phy_fastBoot_Out.txt
	Tree-name-4	RAxML_bipartitions.21952Ali.phy_fastBoot_Out.txt
	…	…

\n-h:\n	Usage = returns the help section for usage
	Input = returns the help section for input file formats
	Options = returns the help section for options
	Paralogy = returns the help section for the group of functions subsumed under Detection and pruning of paralogous sequences
	Conflict = returns the help section for the group of functions subsumed under Detection of conflict
	Mis = returns the help section for the group of functions subsumed under Miscellaneous
	a, b, c, d, e, f, g, h, i or j = returns the help section for the function a, b, c, d, e, f, g, h, i or j, respectively
\n-lbnp:\n	≥ 0 = lower bound of numbers of partitions to be combined in -fun h, so that at least this number of partitions will be combined into a super matrix (default 0); this number has to be an integer 
\n-lowbl:\n	≥ 0 = lowest factor by which the branch leading to the affected clade in option –poslb has to be at least larger than the average of all internal branch length (default = 0), this option has to be turned on for option -poslb
\n-lowbs:\n	0 – 100 = lowest BS value, which will still be considered in the screening procedure of –fun a and b (default = 95) or the lowest BS value still to be summarized in –fun f (default = 1), in both cases values between 0 and 100 are possible as long as they are integers
\n-ltp:\n	0 – 1 = the lower threshold below which a suspected sequence will be regarded as being paralogous to the other non-suspected sequences in the alignment; the value is a proportion and has to be between 0 and 1 and higher than the value provided by -upt (default 0.0); in specific, this proportion reflects the proportion of all sequences, which are not suspected of being paralogous sequences, that return the same best hit as the suspected sequence
\n-maxtaxa:\n	> 0 = maximum number of taxa, which are allowed in a clade for option -possb (default = 2), this option has to be turned on for option –possb
\n-maxt:\n	any value or N, None = maximum threshold of support values possible or which shall be considered for the PABA calculations in -fun j; for example, the maximum threshold of bootstrap support is 100 (default = 100); the input format can either be a number or N/None if not applicable (e.g. Bremer support based values) 
\n-mint:\n	any value or N, None = minimum threshold of support values possible or which shall be considered for the PABA calculations in -fun j and the summary of bootstrap values across different dataset in -fun i; for example, the minimum threshold of bootstrap support is 0 (default = 0); the input format can either be a number or N/None if not applicable (e.g. Bremer support based values) 
\n-nper:\n	≥ 0 = the number of permutated datasets, which shall be generated in -fun h (default 100, optional); this number has to be an integer
\n-ori:\n	name-of-file = if the significance test based on permutations shall be used to test the significance of the PABA results in –fun j the name of the tabulated summary file of support values across the different datasets, in which the support values obtained from the original, non-permutated datasets were stored, has to be provided by this option; this name has also to be part of the list of summary files provided by option -tabf 
\n-partf:\n	name-of-file = name of a plain text file, which defines the partitions for -fun h and j; the format of this file is similar to the format used by RAxML, though it is not required to provide a substitution model, but it can be done. Each new line provides a new definition, which has the following general format:
name-of-substitution-model, name-of-partition = definition-of-partition
The definition of the partition can have different possibilities. The simplest definition is that the partition starts at a position and ends at one (e.g., 1-500). However, it is also possible that the partition is located in two or more parts of the datasets (e.g., 250-350, 400-500, 700-800) or that codon positions are defined (e.g., 1-333\3 for first codon position, 2-333\3 for second and 3-333\3 for third). Finally, all these can be combined into a single file, so that a file can either look like this:
DNA, Partition1 = 1-600	or like this:	DNA, Partition1 = 1-300\3, 2-300\3
WAG, Partition2 = 601-700	DNA, Partition2 = 3-300\3, 701-900
DNA, Partition3 = 701-900	DNA, Partition3 = 301-400, 461-600
…	DNA, Partition4 = 401-460\3
	DNA, Partition5 = 402-460\3
	DNA, Partition6 = 403-460\3
	WAG, Partition7 = 601-700
	…

\n-path:\n	path-to-files = if files are not in the same folder as TreSpEx, provide the path from the root to folder with the files root by -path path-from-the-root; this option can used for all -fun
\n-per:\n	Y, Yes, N or No = if permutated datasets shall be generated in –fun h or the significance of PABA results be tested using permutated datasets in –fun j, this option has to be turned on using Y or Yes (Default=No)
\n-poslb:\n	0, 1, 2, 3 = in the screening procedure the results can be sorted based on different criteria, poslb provides the position for the long branch leading to the affected clade criterion in this sorting; 0 = turned off, 1-3 = turned on and position in the sorting order is 1st, 2nd, or 3rd, respectively (default = 0); this option cannot have the same value as –possb and –possc except for 0
\n-possb:\n	0, 1, 2, 3 = in the screening procedure the results can be sorted based on different criteria, possb provides the position for the short branch leading to terminal taxon criterion in this sorting; 0 = turned off, 1-3 = turned on and position in the sorting order is 1st, 2nd, or 3rd, respectively (default = 0); this option cannot have the same value as –possc and –poslb except for 0
\n-possc:\n	0, 1, 2, 3 = in the screening procedure the results can be sorted based on different criteria, possc provides the position for the strong conflict criterion in this sorting; 0 = turned off, 1-3 = turned on and position in the sorting order is 1st, 2nd, or 3rd, respectively (default = 0); this option cannot have the same value as –possb and –poslb except for 0. If this option is used, this option –gts has also to be turned on as the masking and strong conflict are logically connected.
\n-ppf:\n	name-of-file = name of a plain text file containing the suspected paralogous sequences, the corresponding tree, the bootstrap support value for that clade and the length of the branch leading to the clade; this option is needed for -fun c and d. The format of this file is the same as the output file generated in –fun a or b:
name-of-treefile	bootstrap-support	branch-length	name-of-sequence name-of-sequence name-of-sequence…
For example,
RAxML_bipartitions.RPL24Ali.phy_fastBoot_Out.txt 96 1.75007055 Lanice_conchilega Eisenia_fetida 
RAxML_bipartitions.22680Ali.phy_fastBoot_Out.txt 99 0.25451239 Pomatoceros_lamarckii Alvinella_pompejana 
RAxML_bipartitions.22433Ali.phy_fastBoot_Out.txt 100 4.17620957 Scoloplos_armiger Sthenelais_boa Eurythoe_complanata_EST 
RAxML_bipartitions.22606Ali.phy_fastBoot_Out.txt 100 0.99360951 Cerebratulus_lacteus Owenia_TS 

\n-tabf:\n	name-of-file = name of a plain text file providing the names of the tabulated summary files of the support values across the different datasets, for example as generated by -fun i, for each of which PABA calculations in -fun j shall conducted independently; usually the file with contain only a single name (the original dataset), but if the significance of the PABA results shall be assessed using permutations this file also has to contain the summary file of each single permutation; a new line provides each name: 
		For example,
	Summary-file-1	Original_Compiled_Bipartition_Tables.txt
	Summary-file -2	Permutations_1_Compiled_Bipartition_Tables.txt
	Summary-file-3	Permutations_2_Compiled_Bipartition_Tables.txt
	Summary-file -4	Permutations_3_Compiled_Bipartition_Tables.txt
	…	…

\n-tf:\n	name-of-file = name of a plain text file providing the names of the taxa present in the different trees, which will be used for –fun e and i. The format of this file is simple and each taxon name is provided in a new line:
		For example,
	Taxon-name-1	Eurythoe_complanata
	Taxon-name-2	Terebratalia_transversa
	Taxon-name-3	Arenicola_marina
	Taxon-name-4	Bugula_neritina
	…	…

\n-trf:\n	name-of-file = name of a treefile containing a single tree, which provides the clades for the a posteriori screening approach in -fun b, in Newick format (e.g. RAxML_besttree... or RAxML_bipartition....)
\n-ubnp:\n	≥ 0 = upper bound of numbers of partitions to be combined in -fun h, so that maximal numbers of partitions to be combined into a super matrix will not be higher than this number even if more partitions could be combined (default 0); this number has to be an integer and also has to be equal to or larger than the value provided by –lbnp, but can be higher than the maximum number of defined partitions, which be then automatic maximum number of partitions to be combined into a supermatrix
\n-upbl:\n	≥ 0 = highest factor by which the branch leading to the affected clade in option –poslb has to be at least larger than the average of all internal branch length (default = 0), this option has to be turned on for option –poslb and be equal to or larger than the value of -lowbl
\n-upbs:\n	0 – 100 = highest BS value, which will still be considered in the screening procedure of –fun a and b (default = 100), values between 0 and 100 are possible as long as they are integers and equal to or larger than –lowbs
\n-utp:\n	0 – 1 = the upper threshold above which a suspected sequence will be regarded as being orthologous to the other non-suspected sequences in the alignment; the value is a proportion and has to be between 0 and 1 and higher than the value provided by -lpt (default 1.0); in specific, this proportion reflects the proportion of all sequences, which are not suspected of being paralogous sequences, that return the same best hit as the suspected sequence
\n-wsrt:\n	Y, Yes, N or No = if the significance of the contribution of a partition to a certain set of bipartitions (e.g., certain clades or nodes of a tree or all nodes of a tree) shall be assessed based on the PABA results and a Wilcoxon-Signed-Rank test in –fun j, this option has to be turned on using Y or Yes (Default=No)
";
 }

 sub general {
 print "The help function has to be called together with one of the following terms:
 -h:	-h Usage = returns the help section for usage
	-h Input = returns the help section for input file formats
	-h Options = returns the help section for options
	-h Paralogy = returns the help section for the group of functions subsumed under Detection and pruning of paralogous sequences
	-h Conflict = returns the help section for the group of functions subsumed under Detection of conflict
	-h Mis = returns the help section for the group of functions subsumed under Miscellaneous
	-h a, b, c, d, e, f, g, h, i or j = returns the help section for the function a, b, c, d, e, f, g, h, i or j, respectively
"
 }
 
die "\n"; 
}
